@@ -1,7 +1,7 @@
 /* cfile.h
  * capture_file definition & GUI-independent manipulation
  *
- * $Id: cfile.h 11400 2004-07-18 00:24:25Z guy $
+ * $Id: cfile.h 15534 2005-08-25 21:29:54Z ulfl $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -43,11 +43,10 @@ typedef enum {
 
 typedef struct _capture_file {
   file_state   state;     /* Current state of capture file */
-  int          filed;     /* File descriptor of capture file */
   gchar       *filename;  /* Name of capture file */
   gboolean     is_tempfile; /* Is capture file a temporary file? */
   gboolean     user_saved;/* If capture file is temporary, has it been saved by user yet? */
-  long         f_len;     /* Length of capture file */
+  long         f_datalen; /* Size of capture file data (uncompressed) */
   guint16      cd_t;      /* File type of capture file */
   int          lnk_t;     /* Link-layer type with which to save capture */
   guint32      vers;      /* Version.  For tcpdump minor is appended to major */
@@ -55,21 +54,15 @@ typedef struct _capture_file {
   int          displayed_count; /* Number of displayed frames */
   int          marked_count; /* Number of marked frames */
   gboolean     drops_known; /* TRUE if we know how many packets were dropped */
-  guint32      drops;     /* Dropped packets */
-  guint32      esec;      /* Elapsed seconds */
-  guint32      eusec;     /* Elapsed microseconds */
+  guint32      drops;       /* Dropped packets */
+  nstime_t     elapsed_time;/* Elapsed time */
   gboolean     has_snap;  /* TRUE if maximum capture packet length is known */
   int          snap;      /* Maximum captured packet length */
-  gchar       *iface;     /* Interface */
-  gchar       *save_file; /* File that user saved capture to */
-  int          save_file_fd; /* File descriptor for saved file */
   wtap        *wth;       /* Wiretap session */
-  dfilter_t   *rfcode;    /* Compiled read filter program */
+  dfilter_t   *rfcode;    /* Compiled read (display) filter program */
   gchar       *dfilter;   /* Display filter string */
   dfilter_t   *dfcode;    /* Compiled display filter program */
-#ifdef HAVE_LIBPCAP
-  gchar       *cfilter;   /* Capture filter string */
-#endif
+  /* search */
   gchar       *sfilter;   /* Search filter string */
   gboolean     sbackward; /* TRUE if search is backward, FALSE if forward */
   gboolean     hex;       /* TRUE is raw data search is being performed */
@@ -78,6 +71,7 @@ typedef struct _capture_file {
   gboolean     case_type; /* TRUE if case-insensitive text search */
   gboolean     decode_data; /* TRUE if searching protocol tree text */
   gboolean     summary_data; /* TRUE if searching Info column text */
+  /* packet data */
   union wtap_pseudo_header pseudo_header;      /* Packet pseudo_header */
   guint8       pd[WTAP_MAX_PACKET_SIZE];  /* Packet data */
   GMemChunk   *plist_chunk; /* Memory chunk for frame_data structures */
