@@ -2,7 +2,7 @@
  * Routines for AJP13 dissection
  * Copyright 2002, Christopher K. St. John <cks@distributopia.com>
  *
- * $Id: packet-ajp13.c 13243 2005-02-02 20:07:03Z etxrab $
+ * $Id: packet-ajp13.c 15484 2005-08-20 22:06:31Z sahlberg $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -34,6 +34,7 @@
 #include <glib.h>
 
 #include <epan/packet.h>
+#include <epan/emem.h>
 #include <epan/conversation.h>
 #include "packet-tcp.h"
 
@@ -138,6 +139,7 @@ static const value_string req_header_codes[] = {
   { 0x0C, "pragma" },
   { 0x0D, "referer" },
   { 0x0E, "user-agent" },
+  { 0, NULL}
 };
 
 
@@ -153,6 +155,7 @@ static const value_string rsp_header_codes[] = {
   { 0x09, "Servlet-Engine" },
   { 0x0A, "Status" },
   { 0x0B, "WWW-Authenticate" },
+  { 0, NULL}
 };
 
 
@@ -165,6 +168,7 @@ static const value_string mtype_codes[] = {
   { 5, "END RESPONSE" },
   { 6, "GET BODY CHUNK" },
   { 7, "SHUTDOWN" },
+  { 0, NULL }
 };
 
 
@@ -190,6 +194,7 @@ static const value_string http_method_codes[] = {
   { 19, "CHECKOUT" },
   { 20, "UNCHECKOUT" },
   { 21, "SEARCH" },
+  { 0, NULL }
 };
 
 
@@ -222,16 +227,10 @@ typedef struct ajp13_conv_data {
   gboolean was_get_body_chunk;	/* XXX - not used */
 } ajp13_conv_data;
 
-static GMemChunk *ajp13_conv_data_chunk = NULL;
-
 typedef struct ajp13_frame_data {
   gboolean is_request_body;
 } ajp13_frame_data;
 
-static GMemChunk  *ajp13_frame_data_chunk = NULL;
-
-static int ajp13_packet_init_count = 100;
-
 /* ajp13, in sort of a belt-and-suspenders move, encodes strings with
  * both a leading length field, and a trailing null. Mostly, see
  * AJPv13.html. The returned length _includes_ the trailing null, if
@@ -268,10 +267,10 @@ get_nstring(tvbuff_t *tvb, gint offset, guint8* cbuf, size_t cbuflen)
 static void
 display_rsp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *ajp13_tree)
 {
-  gchar* msg_code = NULL;
+  const gchar* msg_code = NULL;
   int pos = 0;
   guint8 mcode = 0;
-  char mcode_buf[1024];
+  char *mcode_buf;
   int i;
 
   /* MAGIC
@@ -290,7 +289,8 @@ display_rsp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *ajp13_tree)
    */
   mcode = tvb_get_guint8(tvb, pos);
   msg_code = val_to_str(mcode, mtype_codes, "UNKNOWN");
-  sprintf(mcode_buf, "(%d) %s", mcode, msg_code);
+  mcode_buf=ep_alloc(32);
+  g_snprintf(mcode_buf, 32, "(%d) %s", mcode, msg_code);
   if (ajp13_tree)
     proto_tree_add_string(ajp13_tree, hf_ajp13_code, tvb, pos, 1, mcode_buf);
   pos+=1;
@@ -346,11 +346,10 @@ display_rsp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *ajp13_tree)
       char hval[8192];
       guint16 hval_len;
       int orig_pos = pos;
-      gchar* hname = NULL;
+      const gchar* hname = NULL;
       int dp = 0;
       int cl = 0;
       guint8 hname_bytes[1024];
-      gchar hname_value[8*1024];
 
       /* HEADER CODE/NAME
        */
@@ -382,7 +381,9 @@ display_rsp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *ajp13_tree)
       pos+=hval_len+2;
       dp = pos - orig_pos;
       if (ajp13_tree) {
-        sprintf(hname_value, "%s : %s", hname, hval);
+        gchar *hname_value;
+        hname_value=ep_alloc(512);
+        g_snprintf(hname_value, 512, "%s : %s", hname, hval);
         proto_tree_add_string(ajp13_tree, hf_ajp13_hval, tvb, orig_pos, dp, hname_value);
       }
     }
@@ -478,10 +479,11 @@ display_req_forward(tvbuff_t *tvb, packet_info *pinfo,
    */
   cod = tvb_get_guint8(tvb, 4);
   if (ajp13_tree) {
-    gchar* msg_code = NULL;
-    char mcode_buf[1024];
+    const gchar* msg_code = NULL;
+    char *mcode_buf;
     msg_code = val_to_str(cod, mtype_codes, "UNKNOWN");
-    sprintf(mcode_buf, "(%d) %s", cod, msg_code);
+    mcode_buf=ep_alloc(32);
+    g_snprintf(mcode_buf, 32, "(%d) %s", cod, msg_code);
     proto_tree_add_string(ajp13_tree, hf_ajp13_code, tvb, pos, 1, mcode_buf);
   }
   pos+=1;
@@ -489,12 +491,13 @@ display_req_forward(tvbuff_t *tvb, packet_info *pinfo,
   /* HTTP METHOD (ENCODED AS INTEGER)
    */
   {
-    gchar* meth_code = NULL;
+    const gchar* meth_code = NULL;
     meth = tvb_get_guint8(tvb, pos);
     meth_code = val_to_str(meth, http_method_codes, "UNKNOWN");
     if (ajp13_tree) {
-      char mcode_buf[1024];
-      sprintf(mcode_buf, "(%d) %s", meth, meth_code);
+      char *mcode_buf;
+      mcode_buf=ep_alloc(32);
+      g_snprintf(mcode_buf, 32, "(%d) %s", meth, meth_code);
       proto_tree_add_string(ajp13_tree, hf_ajp13_method, tvb, pos, 1, mcode_buf);
     }
     if(check_col(pinfo->cinfo, COL_INFO))
@@ -574,13 +577,12 @@ display_req_forward(tvbuff_t *tvb, packet_info *pinfo,
 
     guint8 hcd;
     guint8 hid;
-    char hval[8192];
-    guint16 hval_len;
     int orig_pos = pos;
-    gchar* hname = NULL;
+    const gchar* hname = NULL;
     int dp = 0;
     int cl = 0;
-    guint8 hname_bytes[1024];
+    char *hval;
+    guint16 hval_len;
 
     /* HEADER CODE/NAME
      */
@@ -595,10 +597,13 @@ display_req_forward(tvbuff_t *tvb, packet_info *pinfo,
       if (hid == 0x08)
         cl = 1;
     } else {
-      int hname_len = get_nstring(tvb, pos, hname_bytes, sizeof hname_bytes);
+      guint8 *hname_bytes;
+      int hname_len;
 
+      hname_bytes=ep_alloc(1024);
+      hname_len = get_nstring(tvb, pos, hname_bytes, 1024);
       pos+=hname_len+2;
-      hname = (gchar*)hname_bytes; /* VERY EVIL */
+      hname = (gchar*)hname_bytes;
     }
 
     dp = pos-orig_pos;
@@ -606,7 +611,8 @@ display_req_forward(tvbuff_t *tvb, packet_info *pinfo,
     /* HEADER VALUE
      */
     orig_pos = pos;
-    hval_len = get_nstring(tvb, pos, hval, sizeof hval);
+    hval=ep_alloc(8192);
+    hval_len = get_nstring(tvb, pos, hval, 8192);
 
     pos+=hval_len+2;
     dp = pos - orig_pos;
@@ -648,7 +654,7 @@ dissect_ajp13_tcp_pdu(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   }
   cd = (ajp13_conv_data*)conversation_get_proto_data(conv, proto_ajp13);
   if (!cd) {
-    cd = (ajp13_conv_data*)g_mem_chunk_alloc(ajp13_conv_data_chunk);
+    cd = se_alloc(sizeof(ajp13_conv_data));
     cd->content_length = 0;
     cd->was_get_body_chunk = FALSE;
     conversation_add_proto_data(conv, proto_ajp13, cd);
@@ -665,7 +671,7 @@ dissect_ajp13_tcp_pdu(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
      * time we've see the packet, and it must be the first "in order"
      * pass through the data.
      */
-    fd = (ajp13_frame_data*)g_mem_chunk_alloc(ajp13_frame_data_chunk);
+    fd = se_alloc(sizeof(ajp13_frame_data));
     p_add_proto_data(pinfo->fd, proto_ajp13, fd);
     fd->is_request_body = FALSE;
     if (cd->content_length) {
@@ -751,25 +757,6 @@ dissect_ajp13(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
 
 
-static void
-ajp13_init_protocol(void)
-{
-  if (ajp13_conv_data_chunk)
-    g_mem_chunk_destroy(ajp13_conv_data_chunk);
-  if (ajp13_frame_data_chunk)
-    g_mem_chunk_destroy(ajp13_frame_data_chunk);
-
-  ajp13_conv_data_chunk = g_mem_chunk_new("ajp13_conv_data_chunk",
-					  sizeof(ajp13_conv_data),
-					  ajp13_packet_init_count * sizeof(ajp13_conv_data),
-					  G_ALLOC_ONLY);
-
-  ajp13_frame_data_chunk = g_mem_chunk_new("ajp13_frame_data_chunk",
-					   sizeof(ajp13_frame_data),
-					   ajp13_packet_init_count * sizeof(ajp13_frame_data),
-					   G_ALLOC_ONLY);
-}
-
 void
 proto_register_ajp13(void)
 {
@@ -863,7 +850,6 @@ proto_register_ajp13(void)
   proto_register_field_array(proto_ajp13, hf, array_length(hf));
   proto_register_subtree_array(ett, array_length(ett));
 
-  register_init_routine(&ajp13_init_protocol);
 }
 
 
