@@ -2,7 +2,7 @@
  * Definitions of routines for ARP packet disassembly that are used
  * elsewhere
  *
- * $Id: packet-arp.h 14786 2005-06-26 19:56:52Z guy $
+ * $Id: packet-arp.h 15114 2005-07-27 07:57:01Z jmayer $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -26,7 +26,7 @@
 #ifndef __PACKET_ARP_H__
 #define __PACKET_ARP_H__
 
-gchar *arphrdaddr_to_str(const guint8 *ad, int ad_len, guint16 type);
+const gchar *arphrdaddr_to_str(const guint8 *ad, int ad_len, guint16 type);
 const gchar *arphrdtype_to_str(guint16 hwtype, const char *fmt);
 
 void dissect_atm_nsap(tvbuff_t *tvb, int offset, int len, proto_tree *tree);
