@@ -5,7 +5,7 @@
  * Routines for DCERPC over SMB packet disassembly
  * Copyright 2001-2003, Tim Potter <tpot@samba.org>
  *
- * $Id: packet-dcerpc-nt.c 15079 2005-07-26 08:01:16Z jmayer $
+ * $Id: packet-dcerpc-nt.c 17671 2006-03-19 20:21:56Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -34,6 +34,7 @@
 
 #include <glib.h>
 #include <epan/packet.h>
+#include <epan/emem.h>
 #include "packet-dcerpc.h"
 #include "packet-dcerpc-nt.h"
 #include "packet-windows-common.h"
@@ -80,8 +81,8 @@ dissect_ndr_counted_string_cb(tvbuff_t *tvb, int offset,
 
 	if (di->conformant_run)
 		return offset;
-	
-	/* 
+
+	/*
            struct {
                short len;
                short size;
@@ -94,7 +95,7 @@ dissect_ndr_counted_string_cb(tvbuff_t *tvb, int offset,
 			hf_nt_cs_len, &len);
 
 	offset = dissect_ndr_uint16(tvb, offset, pinfo, tree, drep,
-			hf_nt_cs_size, &size);	
+			hf_nt_cs_size, &size);
 
 	offset = dissect_ndr_pointer_cb(tvb, offset, pinfo, tree, drep,
 			dissect_ndr_wchar_cvstring, NDR_POINTER_UNIQUE,
@@ -117,7 +118,7 @@ dissect_ndr_counted_string_helper(tvbuff_t *tvb, int offset,
 	if (add_subtree) {
 
 		item = proto_tree_add_text(
-			tree, tvb, offset, 0, 
+			tree, tvb, offset, 0,
 			proto_registrar_get_name(hf_index));
 
 		subtree = proto_item_add_subtree(item, ett_nt_counted_string);
@@ -184,12 +185,12 @@ dissect_ndr_counted_byte_array_cb(tvbuff_t *tvb, int offset,
 	if (di->conformant_run)
 		return offset;
 
-	item = proto_tree_add_text(tree, tvb, offset, 0, 
+	item = proto_tree_add_text(tree, tvb, offset, 0,
 		proto_registrar_get_name(hf_index));
 
 	subtree = proto_item_add_subtree(item, ett_nt_counted_byte_array);
-	
-	/* 
+
+	/*
            struct {
                short len;
                short size;
@@ -202,7 +203,7 @@ dissect_ndr_counted_byte_array_cb(tvbuff_t *tvb, int offset,
 			hf_nt_cs_len, &len);
 
 	offset = dissect_ndr_uint16(tvb, offset, pinfo, subtree, drep,
-			hf_nt_cs_size, &size);	
+			hf_nt_cs_size, &size);
 
 	offset = dissect_ndr_pointer_cb(tvb, offset, pinfo, subtree, drep,
 			dissect_ndr_char_cvstring, NDR_POINTER_UNIQUE,
@@ -211,8 +212,8 @@ dissect_ndr_counted_byte_array_cb(tvbuff_t *tvb, int offset,
 	return offset;
 }
 
-void cb_byte_array_postprocess(packet_info *pinfo, proto_tree *tree _U_,
-			proto_item *item, tvbuff_t *tvb, 
+static void cb_byte_array_postprocess(packet_info *pinfo, proto_tree *tree _U_,
+			proto_item *item, tvbuff_t *tvb,
 			int start_offset, int end_offset,
 			void *callback_args)
 {
@@ -290,12 +291,12 @@ dissect_ndr_counted_ascii_string_cb(tvbuff_t *tvb, int offset,
 	if (di->conformant_run)
 		return offset;
 
-	item = proto_tree_add_text(tree, tvb, offset, 0, 
+	item = proto_tree_add_text(tree, tvb, offset, 0,
 		proto_registrar_get_name(hf_index));
 
 	subtree = proto_item_add_subtree(item, ett_nt_counted_ascii_string);
-	
-	/* 
+
+	/*
            struct {
                short len;
                short size;
@@ -308,7 +309,7 @@ dissect_ndr_counted_ascii_string_cb(tvbuff_t *tvb, int offset,
 			hf_nt_cs_len, &len);
 
 	offset = dissect_ndr_uint16(tvb, offset, pinfo, subtree, drep,
-			hf_nt_cs_size, &size);	
+			hf_nt_cs_size, &size);
 
 	offset = dissect_ndr_pointer_cb(tvb, offset, pinfo, subtree, drep,
 			dissect_ndr_char_cvstring, NDR_POINTER_UNIQUE,
@@ -411,12 +412,7 @@ typedef struct {
 	pol_value *list;                 /* List of policy handle entries */
 } pol_hash_value;
 
-#define POL_HASH_INIT_COUNT 100
-
-static GHashTable *pol_hash;
-static GMemChunk *pol_hash_key_chunk;
-static GMemChunk *pol_value_chunk;
-static GMemChunk *pol_hash_value_chunk;
+static GHashTable *pol_hash = NULL;
 
 /* Hash function */
 
@@ -510,10 +506,10 @@ static void add_pol_handle(e_ctx_hnd *policy_hnd, guint32 frame,
 		 * and put the hash value in the policy handle hash
 		 * table.
 		 */
-		value = g_mem_chunk_alloc(pol_hash_value_chunk);
+		value = se_alloc(sizeof(pol_hash_value));
 		value->list = pol;
 		pol->next = NULL;
-		key = g_mem_chunk_alloc(pol_hash_key_chunk);
+		key = se_alloc(sizeof(pol_hash_key));
 		memcpy(&key->policy_hnd, policy_hnd, sizeof(key->policy_hnd));
 		g_hash_table_insert(pol_hash, key, value);
 	} else {
@@ -540,7 +536,7 @@ static void add_pol_handle(e_ctx_hnd *policy_hnd, guint32 frame,
 			value->list = pol;
 		else
 			polprev->next = pol;
-		
+
 		/*
 		 * "polnext" points to the entry in the list before
 		 * which we should put the new entry; if it's null,
@@ -610,7 +606,7 @@ void dcerpc_smb_store_pol_pkts(e_ctx_hnd *policy_hnd, packet_info *pinfo,
 
 	/* Create a new value */
 
-	pol = g_mem_chunk_alloc(pol_value_chunk);
+	pol = se_alloc(sizeof(pol_value));
 
 	pol->open_frame = is_open ? pinfo->fd->num : 0;
 	pol->close_frame = is_close ? pinfo->fd->num : 0;
@@ -664,7 +660,7 @@ void dcerpc_smb_store_pol_name(e_ctx_hnd *policy_hnd, packet_info *pinfo,
 
 	/* Create a new value */
 
-	pol = g_mem_chunk_alloc(pol_value_chunk);
+	pol = se_alloc(sizeof(pol_value));
 
 	pol->open_frame = 0;
 	pol->close_frame = 0;
@@ -744,29 +740,6 @@ static void free_pol_keyvalue(gpointer key _U_, gpointer value_arg,
 
 static void init_pol_hash(void)
 {
-	/* Initialise memory chunks */
-
-	if (pol_hash_key_chunk)
-		g_mem_chunk_destroy(pol_hash_key_chunk);
-
-	pol_hash_key_chunk = g_mem_chunk_new(
-		"Policy handle hash keys", sizeof(pol_hash_key),
-		POL_HASH_INIT_COUNT * sizeof(pol_hash_key), G_ALLOC_ONLY);
-
-	if (pol_value_chunk)
-		g_mem_chunk_destroy(pol_value_chunk);
-
-	pol_value_chunk = g_mem_chunk_new(
-		"Policy handle values", sizeof(pol_value),
-		POL_HASH_INIT_COUNT * sizeof(pol_value), G_ALLOC_ONLY);
-
-	if (pol_hash_value_chunk)
-		g_mem_chunk_destroy(pol_hash_value_chunk);
-
-	pol_hash_value_chunk = g_mem_chunk_new(
-		"Policy handle hash values", sizeof(pol_hash_value),
-		POL_HASH_INIT_COUNT * sizeof(pol_hash_value), G_ALLOC_ONLY);
-
 	/* Initialise hash table */
 
 	if (pol_hash) {
@@ -828,13 +801,25 @@ static int hf_nt_policy_close_frame = -1;
 
 static gint ett_nt_policy_hnd = -1;
 
-int
-dissect_nt_policy_hnd(tvbuff_t *tvb, gint offset, packet_info *pinfo,
+/* this function is used to dissect a "handle".
+ * it will keep track of which frame a handle is opened from and in which 
+ * frame it is closed.
+ * normally, this function would be used for tracking 20 byte policy handles
+ * as used in dcerpc  but it has shown VERY useful to also use it for tracking
+ * GUIDs such as for the file ids in smb2.
+ */
+typedef enum {
+	HND_TYPE_CTX_HANDLE,
+	HND_TYPE_GUID
+} e_hnd_type;
+
+static int
+dissect_nt_hnd(tvbuff_t *tvb, gint offset, packet_info *pinfo,
 		      proto_tree *tree, guint8 *drep, int hfindex,
 		      e_ctx_hnd *pdata, proto_item **pitem,
-		      gboolean is_open, gboolean is_close)
+		      gboolean is_open, gboolean is_close, e_hnd_type type)
 {
-	proto_item *item;
+	proto_item *item=NULL;
 	proto_tree *subtree;
 	e_ctx_hnd hnd;
 	guint32 open_frame = 0, close_frame = 0;
@@ -855,13 +840,29 @@ dissect_nt_policy_hnd(tvbuff_t *tvb, gint offset, packet_info *pinfo,
 
 	/* Add to proto tree */
 
-	item = proto_tree_add_text(tree, tvb, offset, sizeof(e_ctx_hnd),
-				   "Policy Handle");
+	switch(type){
+	case HND_TYPE_CTX_HANDLE:
+		item = proto_tree_add_text(tree, tvb, offset, sizeof(e_ctx_hnd),
+					   "Policy Handle");
 
-	subtree = proto_item_add_subtree(item, ett_nt_policy_hnd);
+		subtree = proto_item_add_subtree(item, ett_nt_policy_hnd);
 
-	offset = dissect_ndr_ctx_hnd(tvb, offset, pinfo, subtree, drep,
-				     hfindex, &hnd);
+		offset = dissect_ndr_ctx_hnd(tvb, offset, pinfo, subtree, drep,
+					     hfindex, &hnd);
+		break;
+	case HND_TYPE_GUID:
+		item = proto_tree_add_text(tree, tvb, offset, 16,
+					   "GUID handle");
+
+		subtree = proto_item_add_subtree(item, ett_nt_policy_hnd);
+
+		hnd.attributes=0;
+		offset=dissect_ndr_uuid_t(tvb, offset, pinfo, subtree, drep, hfindex, &hnd.uuid);
+		break;
+	default:
+		DISSECTOR_ASSERT_NOT_REACHED();
+		return offset;
+	}
 
 	/*
 	 * Create a new entry for this handle if it's not a null handle
@@ -909,6 +910,36 @@ dissect_nt_policy_hnd(tvbuff_t *tvb, gint offset, packet_info *pinfo,
 	return offset;
 }
 
+
+int
+dissect_nt_policy_hnd(tvbuff_t *tvb, gint offset, packet_info *pinfo,
+		      proto_tree *tree, guint8 *drep, int hfindex,
+		      e_ctx_hnd *pdata, proto_item **pitem,
+		      gboolean is_open, gboolean is_close)
+{
+	offset=dissect_nt_hnd(tvb, offset, pinfo,
+		      tree, drep, hfindex,
+		      pdata, pitem,
+		      is_open, is_close, HND_TYPE_CTX_HANDLE);
+
+	return offset;
+}
+
+/* this function must be called with   hfindex being HF_GUID */
+int
+dissect_nt_guid_hnd(tvbuff_t *tvb, gint offset, packet_info *pinfo,
+		      proto_tree *tree, guint8 *drep, int hfindex,
+		      e_ctx_hnd *pdata, proto_item **pitem,
+		      gboolean is_open, gboolean is_close)
+{
+	offset=dissect_nt_hnd(tvb, offset, pinfo,
+		      tree, drep, hfindex,
+		      pdata, pitem,
+		      is_open, is_close, HND_TYPE_GUID);
+
+	return offset;
+}
+
 /* Some helper routines to dissect a range of uint8 characters.  I don't
    think these are "official" NDR representations and are probably specific
    to NT so for the moment they're put here instead of in packet-dcerpc.c
@@ -986,9 +1017,8 @@ dissect_ndr_uint16s(tvbuff_t *tvb, gint offset, packet_info *pinfo,
 /*
  * Helper routines for dissecting NDR strings
  */
-
 void cb_wstr_postprocess(packet_info *pinfo, proto_tree *tree _U_,
-			proto_item *item, tvbuff_t *tvb, 
+			proto_item *item, tvbuff_t *tvb,
 			int start_offset, int end_offset,
 			void *callback_args)
 {
@@ -1014,7 +1044,7 @@ void cb_wstr_postprocess(packet_info *pinfo, proto_tree *tree _U_,
 	 * some way we can get that string, rather than duplicating the
 	 * efforts of that routine?
 	 */
-	s = tvb_fake_unicode(
+	s = tvb_get_ephemeral_faked_unicode(
 		tvb, start_offset + 12, (end_offset - start_offset - 12) / 2,
 		TRUE);
 
@@ -1048,15 +1078,14 @@ void cb_wstr_postprocess(packet_info *pinfo, proto_tree *tree _U_,
 	if (options & CB_STR_SAVE) {
 		dcerpc_info *di = (dcerpc_info *)pinfo->private_data;
 		dcerpc_call_value *dcv = (dcerpc_call_value *)di->call_data;
-		
+
+/* FIXME EPHEMERAL need to get rid of the g_strdup() and later g_free() */
 		dcv->private_data = g_strdup(s);
 	}
-
-	g_free(s);
 }
 
 void cb_str_postprocess(packet_info *pinfo, proto_tree *tree _U_,
-			proto_item *item, tvbuff_t *tvb, 
+			proto_item *item, tvbuff_t *tvb,
 			int start_offset, int end_offset,
 			void *callback_args)
 {
@@ -1082,7 +1111,7 @@ void cb_str_postprocess(packet_info *pinfo, proto_tree *tree _U_,
 	 * some way we can get that string, rather than duplicating the
 	 * efforts of that routine?
 	 */
-	s = tvb_get_string(
+	s = tvb_get_ephemeral_string(
 		tvb, start_offset + 12, (end_offset - start_offset - 12) );
 
 	/* Append string to COL_INFO */
@@ -1115,24 +1144,22 @@ void cb_str_postprocess(packet_info *pinfo, proto_tree *tree _U_,
 	if (options & CB_STR_SAVE) {
 		dcerpc_info *di = (dcerpc_info *)pinfo->private_data;
 		dcerpc_call_value *dcv = (dcerpc_call_value *)di->call_data;
-		
+
 		dcv->private_data = g_strdup(s);
 	}
-
-	g_free(s);
 }
 
 /* Dissect a pointer to a NDR string and append the string value to the
    proto_item. */
 
-int dissect_ndr_str_pointer_item(tvbuff_t *tvb, gint offset, 
-				 packet_info *pinfo, proto_tree *tree, 
-				 guint8 *drep, int type, const char *text, 
+int dissect_ndr_str_pointer_item(tvbuff_t *tvb, gint offset,
+				 packet_info *pinfo, proto_tree *tree,
+				 guint8 *drep, int type, const char *text,
 				 int hf_index, int levels)
 {
 	return dissect_ndr_pointer_cb(
-		tvb, offset, pinfo, tree, drep, 
-		dissect_ndr_wchar_cvstring, type, text, hf_index, 
+		tvb, offset, pinfo, tree, drep,
+		dissect_ndr_wchar_cvstring, type, text, hf_index,
 		cb_wstr_postprocess, GINT_TO_POINTER(levels + 1));
 }
 
@@ -1142,7 +1169,7 @@ static int hf_nt_count = -1;
 static int hf_nt_domain_sid = -1;
 
 int
-dissect_ndr_nt_SID(tvbuff_t *tvb, int offset, packet_info *pinfo, 
+dissect_ndr_nt_SID(tvbuff_t *tvb, int offset, packet_info *pinfo,
 		   proto_tree *tree, guint8 *drep)
 {
 	dcerpc_info *di = (dcerpc_info *)pinfo->private_data;
@@ -1166,7 +1193,7 @@ dissect_ndr_nt_SID(tvbuff_t *tvb, int offset, packet_info *pinfo,
 	offset = dissect_ndr_uint32 (tvb, offset, pinfo, tree, drep,
 			hf_nt_count, NULL);
 
-	offset = dissect_nt_sid(tvb, offset, tree, name, &sid_str, 
+	offset = dissect_nt_sid(tvb, offset, tree, name, &sid_str,
 				hf_nt_domain_sid);
 
 	/* dcv can be null, for example when this ndr structure is embedded
@@ -1195,8 +1222,8 @@ dissect_ndr_nt_SID_with_options(tvbuff_t *tvb, int offset, packet_info *pinfo, p
 		proto_item *item=(proto_item *)tree;
 
 		if ((options & CB_STR_COL_INFO)&&(!di->conformant_run)) {
-			/* kludge, ugly,   but this is called twice for all 
-			   dcerpc interfaces due to how we chase pointers 
+			/* kludge, ugly,   but this is called twice for all
+			   dcerpc interfaces due to how we chase pointers
 			   and putting the sid twice on the summary line
 			   looks even worse.
 			   Real solution would be to block updates to col_info
@@ -1230,7 +1257,7 @@ dissect_ndr_nt_SID_with_options(tvbuff_t *tvb, int offset, packet_info *pinfo, p
 }
 
 static int
-dissect_ndr_nt_SID_hf_through_ptr(tvbuff_t *tvb, int offset, packet_info *pinfo, 
+dissect_ndr_nt_SID_hf_through_ptr(tvbuff_t *tvb, int offset, packet_info *pinfo,
 		   proto_tree *tree, guint8 *drep)
 {
 	offset = dissect_ndr_nt_SID(tvb, offset, pinfo, tree, drep);
@@ -1548,17 +1575,17 @@ void dcerpc_smb_init(int proto_dcerpc)
 
 		{ &hf_nt_cs_size,
 		  { "Size", "nt.str.size", FT_UINT16, BASE_DEC,
-		    NULL, 0x0, "Size of string in short integers", 
+		    NULL, 0x0, "Size of string in short integers",
 		    HFILL }},
-		
+
 		{ &hf_nt_cs_len,
 		  { "Length", "nt.str.len", FT_UINT16, BASE_DEC,
-		    NULL, 0x0, "Length of string in short integers", 
+		    NULL, 0x0, "Length of string in short integers",
 		    HFILL }},
-		
+
 		/* GUIDs */
 		{ &hf_nt_guid,
-		  { "GUID", "nt.guid", FT_STRING, BASE_NONE, 
+		  { "GUID", "nt.guid", FT_STRING, BASE_NONE,
 		    NULL, 0x0, "GUID (uuid for groups?)", HFILL }},
 
 		/* Policy handles */
@@ -1579,90 +1606,90 @@ void dcerpc_smb_init(int proto_dcerpc)
 		  { "Acct Ctrl", "nt.acct_ctrl", FT_UINT32, BASE_HEX,
 		    NULL, 0x0, "Acct CTRL", HFILL }},
 
-		{ &hf_nt_acb_disabled, 
-		  { "", "nt.acb.disabled", FT_BOOLEAN, 32, 
-		    TFS(&tfs_nt_acb_disabled), 0x0001, 
+		{ &hf_nt_acb_disabled,
+		  { "", "nt.acb.disabled", FT_BOOLEAN, 32,
+		    TFS(&tfs_nt_acb_disabled), 0x0001,
 		    "If this account is enabled or disabled", HFILL }},
 
-		{ &hf_nt_acb_homedirreq, 
+		{ &hf_nt_acb_homedirreq,
 		  { "", "nt.acb.homedirreq", FT_BOOLEAN, 32,
-		    TFS(&tfs_nt_acb_homedirreq), 0x0002, 
+		    TFS(&tfs_nt_acb_homedirreq), 0x0002,
 		    "Is hom,edirs required for this account?", HFILL }},
 
-		{ &hf_nt_acb_pwnotreq, 
-		  { "", "nt.acb.pwnotreq", FT_BOOLEAN, 32, 
-		    TFS(&tfs_nt_acb_pwnotreq), 0x0004, 
+		{ &hf_nt_acb_pwnotreq,
+		  { "", "nt.acb.pwnotreq", FT_BOOLEAN, 32,
+		    TFS(&tfs_nt_acb_pwnotreq), 0x0004,
 		    "If a password is required for this account?", HFILL }},
 
-		{ &hf_nt_acb_tempdup, 
-		  { "", "nt.acb.tempdup", FT_BOOLEAN, 32, 
-		    TFS(&tfs_nt_acb_tempdup), 0x0008, 
+		{ &hf_nt_acb_tempdup,
+		  { "", "nt.acb.tempdup", FT_BOOLEAN, 32,
+		    TFS(&tfs_nt_acb_tempdup), 0x0008,
 		    "If this is a temporary duplicate account", HFILL }},
 
-		{ &hf_nt_acb_normal, 
-		  { "", "nt.acb.normal", FT_BOOLEAN, 32, 
-		    TFS(&tfs_nt_acb_normal), 0x0010, 
+		{ &hf_nt_acb_normal,
+		  { "", "nt.acb.normal", FT_BOOLEAN, 32,
+		    TFS(&tfs_nt_acb_normal), 0x0010,
 		    "If this is a normal user account", HFILL }},
 
-		{ &hf_nt_acb_mns, 
+		{ &hf_nt_acb_mns,
 		  { "", "nt.acb.mns", FT_BOOLEAN, 32,
-		    TFS(&tfs_nt_acb_mns), 0x0020, 
+		    TFS(&tfs_nt_acb_mns), 0x0020,
 		    "MNS logon user account", HFILL }},
 
-		{ &hf_nt_acb_domtrust, 
+		{ &hf_nt_acb_domtrust,
 		  { "", "nt.acb.domtrust", FT_BOOLEAN, 32,
-		    TFS(&tfs_nt_acb_domtrust), 0x0040, 
+		    TFS(&tfs_nt_acb_domtrust), 0x0040,
 		    "Interdomain trust account", HFILL }},
-		
-		{ &hf_nt_acb_wstrust, 
+
+		{ &hf_nt_acb_wstrust,
 		  { "", "nt.acb.wstrust", FT_BOOLEAN, 32,
-		    TFS(&tfs_nt_acb_wstrust), 0x0080, 
+		    TFS(&tfs_nt_acb_wstrust), 0x0080,
 		    "Workstation trust account", HFILL }},
-		
-		{ &hf_nt_acb_svrtrust, 
+
+		{ &hf_nt_acb_svrtrust,
 		  { "", "nt.acb.svrtrust", FT_BOOLEAN, 32,
-		    TFS(&tfs_nt_acb_svrtrust), 0x0100, 
+		    TFS(&tfs_nt_acb_svrtrust), 0x0100,
 		    "Server trust account", HFILL }},
 
-		{ &hf_nt_acb_pwnoexp, 
+		{ &hf_nt_acb_pwnoexp,
 		  { "", "nt.acb.pwnoexp", FT_BOOLEAN, 32,
-		    TFS(&tfs_nt_acb_pwnoexp), 0x0200, 
+		    TFS(&tfs_nt_acb_pwnoexp), 0x0200,
 		    "If this account expires or not", HFILL }},
 
-		{ &hf_nt_acb_autolock, 
+		{ &hf_nt_acb_autolock,
 		  { "", "nt.acb.autolock", FT_BOOLEAN, 32,
-		    TFS(&tfs_nt_acb_autolock), 0x0400, 
+		    TFS(&tfs_nt_acb_autolock), 0x0400,
 		    "If this account has been autolocked", HFILL }},
 
 		/* SIDs */
 
 		{ &hf_nt_domain_sid,
-		  { "Domain SID", "nt.domain_sid", 
-		    FT_STRING, BASE_NONE, NULL, 0x0, 
+		  { "Domain SID", "nt.domain_sid",
+		    FT_STRING, BASE_NONE, NULL, 0x0,
 		    "The Domain SID", HFILL }},
 
 		{ &hf_nt_count,
-		  { "Count", "nt.count", 
-		    FT_UINT32, BASE_DEC, NULL, 0x0, 
+		  { "Count", "nt.count",
+		    FT_UINT32, BASE_DEC, NULL, 0x0,
 		    "Number of elements in following array", HFILL }},
 
 		/* Logon hours */
 
-		{ &hf_logonhours_divisions, 
-		  { "Divisions", "logonhours.divisions", 
-		    FT_UINT16, BASE_DEC, NULL, 0, 
+		{ &hf_logonhours_divisions,
+		  { "Divisions", "logonhours.divisions",
+		    FT_UINT16, BASE_DEC, NULL, 0,
 		    "Number of divisions for LOGON_HOURS", HFILL }},
 
-		{ &hf_logonhours_unknown_char, 
-		  { "Unknown char", "nt.unknown.char", 
-		    FT_UINT8, BASE_HEX, NULL, 0x0, 
+		{ &hf_logonhours_unknown_char,
+		  { "Unknown char", "nt.unknown.char",
+		    FT_UINT8, BASE_HEX, NULL, 0x0,
 		    "Unknown char. If you know what this is, contact "
 		    "ethereal developers.", HFILL }},
 
 		/* Misc */
 
-                { &hf_nt_attrib, 
-		  { "Attributes", "nt.attr", 
+                { &hf_nt_attrib,
+		  { "Attributes", "nt.attr",
 		    FT_UINT32, BASE_HEX, NULL, 0x0, "", HFILL }},
 	};
 
@@ -1688,5 +1715,5 @@ void dcerpc_smb_init(int proto_dcerpc)
 
 	/* Initialise policy handle hash */
 
-	init_pol_hash();
+	register_init_routine(&init_pol_hash);
 }
