@@ -2,7 +2,7 @@
  * Routines for DLSw packet dissection (Data Link Switching)
  * Copyright 2001, Paul Ionescu <paul@acorp.ro>
  *
- * $Id: packet-dlsw.c 14178 2005-04-24 01:15:53Z gerald $
+ * $Id: packet-dlsw.c 17681 2006-03-20 10:52:53Z sahlberg $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -34,6 +34,7 @@
 #include <string.h>
 #include <glib.h>
 #include <epan/packet.h>
+#include <epan/emem.h>
 #include "packet-tcp.h"
 
 static int proto_dlsw = -1;
@@ -174,7 +175,8 @@ static const value_string dlsw_pri_vals[] = {
         { 4        , "Highest Priority" },
         { 5        , "Reserved" },
         { 6        , "Reserved" },
-        { 7        , "Reserved" }
+        { 7        , "Reserved" },
+	{ 0, NULL }
 };
 
 
@@ -244,7 +246,7 @@ dissect_dlsw_pdu(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
    dlsw_header_tree = proto_item_add_subtree(ti2, ett_dlsw_header);
    proto_tree_add_text (dlsw_header_tree,tvb,0 ,1,"Version        = %s",
-    val_to_str(version , dlsw_version_vals, "Unknown Version, dissection may be innacurate"));
+    val_to_str(version , dlsw_version_vals, "Unknown Version, dissection may be inaccurate"));
    proto_tree_add_text (dlsw_header_tree,tvb,1 ,1,"Header Length  = %u",hlen) ;
    mlen=tvb_get_ntohs(tvb,2);
    proto_tree_add_text (dlsw_header_tree,tvb,2 ,2,"Message Length = %u",mlen);
