@@ -3,7 +3,7 @@
  * (From IEEE Draft P802.1X/D11; is there a later draft, or a
  * final standard?  If so, check it.)
  *
- * $Id: packet-eapol.c 14321 2005-05-05 17:07:58Z jmayer $
+ * $Id: packet-eapol.c 15844 2005-09-17 00:02:31Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -31,7 +31,7 @@
 #include <glib.h>
 #include <epan/packet.h>
 #include "packet-ieee80211.h"
-#include "etypes.h"
+#include <epan/etypes.h>
 
 static int proto_eapol = -1;
 static int hf_eapol_version = -1;
