@@ -11,7 +11,7 @@
  *   Technical realization of Short Message Service (SMS)
  *   (3GPP TS 23.040 version 5.4.0 Release 5)
  *
- * $Id: packet-gsm_sms.c 13056 2005-01-15 23:45:37Z guy $
+ * $Id: packet-gsm_sms.c 17506 2006-03-07 19:53:57Z gerald $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -94,8 +94,8 @@
     }
 
 
-static char *gsm_sms_proto_name = "GSM SMS TPDU (GSM 03.40)";
-static char *gsm_sms_proto_name_short = "GSM SMS";
+static const char *gsm_sms_proto_name = "GSM SMS TPDU (GSM 03.40)";
+static const char *gsm_sms_proto_name_short = "GSM SMS";
 
 /* Initialize the subtree pointers */
 static gint ett_gsm_sms = -1;
@@ -140,25 +140,6 @@ static gint ett_udh_ieis[NUM_UDH_IEIS];
 
 /* FUNCTIONS */
 
-gchar *
-my_match_strval(guint32 val, const value_string *vs, gint *idx)
-{
-    gint i = 0;
-
-    while (vs[i].strptr) {
-	if (vs[i].value == val)
-	{
-	    *idx = i;
-	    return(vs[i].strptr);
-	}
-
-	i++;
-    }
-
-    *idx = -1;
-    return(NULL);
-}
-
 /* 9.2.3.1 */
 #define DIS_FIELD_MTI(m_tree, m_bitmask, m_offset) \
 { \
@@ -231,19 +212,18 @@ my_match_strval(guint32 val, const value_string *vs, gint *idx)
 }
 
 static void
-dis_field_addr(tvbuff_t *tvb, proto_tree *tree, guint32 *offset_p, gchar *title)
+dis_field_addr(tvbuff_t *tvb, proto_tree *tree, guint32 *offset_p, const gchar *title)
 {
     static gchar	digit_table[] = {"0123456789*#abc\0"};
     proto_item		*item;
     proto_tree		*subtree = NULL;
-    gchar		*str = NULL;
+    const gchar		*str = NULL;
     guint8		oct;
     guint32		offset;
     guint32		numdigocts;
     guint32		length;
     guint32		i, j;
 
-
     offset = *offset_p;
 
     oct = tvb_get_guint8(tvb, offset);
@@ -331,16 +311,25 @@ dis_field_addr(tvbuff_t *tvb, proto_tree *tree, guint32 *offset_p, gchar *title)
     offset++;
 
     j = 0;
-    for (i = 0; i < numdigocts; i++)
+    switch ((oct & 0x70) >> 4)
     {
-	oct = tvb_get_guint8(tvb, offset + i);
+    case 0x05: /* "Alphanumeric (coded according to 3GPP TS 23.038 GSM 7-bit default alphabet)" */
+	i = gsm_sms_char_7bit_unpack(0, numdigocts, sizeof(bigbuf), tvb_get_ptr(tvb, offset, numdigocts), bigbuf);
+	bigbuf[i] = '\0';
+	gsm_sms_char_ascii_decode(bigbuf, bigbuf, i);
+	break;
+    default:
+	for (i = 0; i < numdigocts; i++)
+	{
+	    oct = tvb_get_guint8(tvb, offset + i);
 
-        bigbuf[j++] = digit_table[oct & 0x0f];
-        bigbuf[j++] = digit_table[(oct & 0xf0) >> 4];
+	    bigbuf[j++] = digit_table[oct & 0x0f];
+	    bigbuf[j++] = digit_table[(oct & 0xf0) >> 4];
+	}
+	bigbuf[j++] = '\0';
+	break;
     }
 
-    bigbuf[j++] = '\0';
-
     proto_tree_add_text(subtree,
 	tvb, offset, numdigocts,
 	"Digits: %s",
@@ -348,7 +337,7 @@ dis_field_addr(tvbuff_t *tvb, proto_tree *tree, guint32 *offset_p, gchar *title)
 
     proto_item_append_text(item, " - (%s)", bigbuf);
 
-    *offset_p = offset + i;
+    *offset_p = offset + numdigocts;
 }
 
 /* 9.2.3.7 */
@@ -365,7 +354,7 @@ dis_field_pid(tvbuff_t *tvb, proto_tree *tree, guint32 offset, guint8 oct)
     proto_tree	*subtree = NULL;
     guint8	form;
     guint8	telematic;
-    gchar	*str = NULL;
+    const gchar	*str = NULL;
 
 
     item =
@@ -525,7 +514,7 @@ dis_field_dcs(tvbuff_t *tvb, proto_tree *tree, guint32 offset, guint8 oct,
     proto_item	*item;
     proto_tree	*subtree = NULL;
     guint8	form;
-    gchar	*str = NULL;
+    const gchar	*str = NULL;
     gboolean	default_5_bits;
     gboolean	default_3_bits;
     gboolean	default_data;
@@ -1118,15 +1107,15 @@ dis_field_dt(tvbuff_t *tvb, proto_tree *tree, guint32 *offset_p)
 static void
 dis_field_st(tvbuff_t *tvb, proto_tree *tree, guint32 offset, guint8 oct)
 {
-    static gchar	*sc_complete = "Short message transaction completed";
-    static gchar	*sc_temporary = "Temporary error, SC still trying to transfer SM";
-    static gchar	*sc_perm = "Permanent error, SC is not making any more transfer attempts";
-    static gchar	*sc_tempfin = "Temporary error, SC is not making any more transfer attempts";
+    static const gchar	*sc_complete = "Short message transaction completed";
+    static const gchar	*sc_temporary = "Temporary error, SC still trying to transfer SM";
+    static const gchar	*sc_perm = "Permanent error, SC is not making any more transfer attempts";
+    static const gchar	*sc_tempfin = "Temporary error, SC is not making any more transfer attempts";
     proto_item		*item;
     proto_tree		*subtree = NULL;
     guint8		value;
-    gchar		*str = NULL;
-    gchar		*str2 = NULL;
+    const gchar		*str = NULL;
+    const gchar	*str2 = NULL;
 
 
     item =
@@ -1319,7 +1308,7 @@ dis_field_fcs(tvbuff_t *tvb, proto_tree *tree, guint32 offset, guint8 oct)
 {
     proto_item	*item;
     proto_tree	*subtree = NULL;
-    gchar	*str = NULL;
+    const gchar	*str = NULL;
 
 
     item =
@@ -1542,7 +1531,7 @@ gsm_sms_char_ascii_decode(unsigned char* dest, const unsigned char* src, int len
 static void
 dis_iei_apa_8bit(tvbuff_t *tvb, proto_tree *tree, guint32 offset, guint8 length)
 {
-    gchar	*str = NULL;
+    const gchar	*str = NULL;
     guint8	oct;
 
 
@@ -1587,7 +1576,7 @@ dis_iei_apa_8bit(tvbuff_t *tvb, proto_tree *tree, guint32 offset, guint8 length)
 static void
 dis_iei_apa_16bit(tvbuff_t *tvb, proto_tree *tree, guint32 offset, guint8 length)
 {
-    gchar	*str = NULL;
+    const gchar	*str = NULL;
     guint32	value;
 
 
@@ -1644,7 +1633,7 @@ dis_field_ud_iei(tvbuff_t *tvb, proto_tree *tree, guint32 offset, guint8 length)
     guint8	oct;
     proto_item	*item;
     proto_tree	*subtree = NULL;
-    gchar	*str = NULL;
+    const gchar	*str = NULL;
     guint8	iei_len;
 
 
@@ -1767,17 +1756,19 @@ dis_field_ud_iei(tvbuff_t *tvb, proto_tree *tree, guint32 offset, guint8 length)
 }
 
 /* 9.2.3.24 */
+#define NUM_FILL_BITS_MASKS 6
 static void
 dis_field_ud(tvbuff_t *tvb, proto_tree *tree, guint32 offset, guint32 length, gboolean udhi, guint8 udl,
     gboolean seven_bit, gboolean eight_bit, gboolean ucs2, gboolean compressed)
 {
-    static guint8	fill_bits_mask[] = { 0x80, 0xc0, 0xe0, 0xf0, 0xf8, 0xfc };
+    static guint8	fill_bits_mask[NUM_FILL_BITS_MASKS] =
+	{ 0x80, 0xc0, 0xe0, 0xf0, 0xf8, 0xfc };
     proto_item	*item;
     proto_item	*udh_item;
     proto_tree	*subtree = NULL;
     proto_tree	*udh_subtree = NULL;
-    guint8	oct;
-    guint8	fill_bits;
+    guint	oct;
+    guint	fill_bits;
     guint32	out_len;
     char	*ustr;
 
@@ -1794,78 +1785,79 @@ dis_field_ud(tvbuff_t *tvb, proto_tree *tree, guint32 offset, guint32 length, gb
 
     if (udhi)
     {
-	/* step over header */
 
-	udh_item =
-	    proto_tree_add_text(subtree, tvb,
-		offset, oct + 1,
-		"User-Data Header");
+		/* step over header */
 
-	udh_subtree = proto_item_add_subtree(udh_item, ett_udh);
+		udh_item =
+		    proto_tree_add_text(subtree, tvb,
+			offset, oct + 1,
+			"User-Data Header");
 
-	proto_tree_add_text(udh_subtree,
-	    tvb, offset, 1,
-	    "User Data Header Length (%d)",
-	    oct);
+		udh_subtree = proto_item_add_subtree(udh_item, ett_udh);
 
-	offset++;
-	udl--;
-	length--;
-
-	dis_field_ud_iei(tvb, udh_subtree, offset, oct);
-
-	offset += oct;
-	udl -= oct;
-	length -= oct;
-
-	if (seven_bit)
-	{
-	    /* step over fill bits ? */
-
-	    fill_bits = 7 - (((oct + 1) * 8) % 7);
-	    if (fill_bits != 7)
-	    {
-		oct = tvb_get_guint8(tvb, offset);
-
-		other_decode_bitfield_value(bigbuf, oct, fill_bits_mask[fill_bits], 8);
 		proto_tree_add_text(udh_subtree,
 		    tvb, offset, 1,
-		    "%s :  Fill bits",
-		    bigbuf);
-	    }
-	}
+		    "User Data Header Length (%d)",
+		    oct);
+
+		offset++;
+		udl--;
+		length--;
+
+		dis_field_ud_iei(tvb, udh_subtree, offset, oct);
+
+		offset += oct;
+		udl -= oct;
+		length -= oct;
+
+		if (seven_bit)
+			{
+		    /* step over fill bits ? */
+
+		    fill_bits = 7 - (((oct + 1) * 8) % 7);
+		    if (fill_bits < NUM_FILL_BITS_MASKS)
+			    {
+				oct = tvb_get_guint8(tvb, offset);
+
+				other_decode_bitfield_value(bigbuf, oct, fill_bits_mask[fill_bits], 8);
+				proto_tree_add_text(udh_subtree,
+					tvb, offset, 1,
+					"%s :  Fill bits",
+					bigbuf);
+			}
+		}
     }
 
     if (compressed)
     {
-	proto_tree_add_text(subtree, tvb,
-	    offset, length,
-	    "Compressed data");
+		proto_tree_add_text(subtree, tvb,
+		    offset, length,
+		    "Compressed data");
     }
     else
     {
-	if (seven_bit)
-	{
-	    out_len =
-		gsm_sms_char_7bit_unpack(fill_bits, length, sizeof(bigbuf),
+		if (seven_bit)
+		{
+		    out_len =
+			gsm_sms_char_7bit_unpack(fill_bits, length, sizeof(bigbuf),
 		    tvb_get_ptr(tvb, offset, length), bigbuf);
-	    bigbuf[out_len] = '\0';
-	    gsm_sms_char_ascii_decode(bigbuf, bigbuf, out_len);
-	    bigbuf[udl] = '\0';
+		    bigbuf[out_len] = '\0';
+		    gsm_sms_char_ascii_decode(bigbuf, bigbuf, out_len);
+			bigbuf[udl] = '\0';
 
-	    proto_tree_add_text(subtree, tvb, offset, length, "%s", bigbuf);
-	}
-	else if (eight_bit)
-	{
-	    proto_tree_add_text(subtree, tvb, offset, length, "%s",
+			proto_tree_add_text(subtree, tvb, offset, length, "%s", bigbuf);
+		}
+		else if (eight_bit)
+			{
+			proto_tree_add_text(subtree, tvb, offset, length, "%s",
 	        tvb_format_text(tvb, offset, length));
-	}
-	else if (ucs2)
-	{
-	    ustr = tvb_fake_unicode(tvb, offset, length, FALSE);
-	    proto_tree_add_text(subtree, tvb, offset, length, "%s", ustr);
-	    g_free(ustr);
-	}
+		}
+		else if (ucs2)
+			{
+			/* tvb_get_ephemeral_faked_unicode takes the lengt in number of guint16's */
+			ustr = tvb_get_ephemeral_faked_unicode(tvb, offset, (length>>1), FALSE);
+			proto_tree_add_text(subtree, tvb, offset, length, "%s", ustr);
+		}
     }
 }
 
@@ -2035,7 +2027,8 @@ dis_msg_deliver_report(tvbuff_t *tvb, proto_tree *tree, guint32 offset)
 
     DIS_FIELD_UDHI(tree, 0x40, offset, udhi);
 
-    DIS_FIELD_MTI(tree, 0x03, offset);
+	DIS_FIELD_MMS(tree, 0x04, offset); /* Bit 2			*/
+    DIS_FIELD_MTI(tree, 0x03, offset); /* Bit 0 and 1	*/
 
     if (length < 2)
     {
@@ -2139,7 +2132,7 @@ dis_msg_submit(tvbuff_t *tvb, proto_tree *tree, guint32 offset)
     guint8	oct;
     guint8	vp_form;
     guint8	udl;
-    gchar	*str = NULL;
+    const gchar	*str = NULL;
     gboolean	seven_bit;
     gboolean	eight_bit;
     gboolean	ucs2;
@@ -2360,6 +2353,15 @@ dis_msg_status_report(tvbuff_t *tvb, proto_tree *tree, guint32 offset)
     dis_field_st(tvb, tree, offset, oct);
 
     offset++;
+	/* Parameter indicating the presence of any of
+	 * the optional parameters which follow
+	 * 4) Mandatory if any of the optional parameters following TP-PI is present,
+	 * otherwise optional.
+	 */
+	if (length <= (offset - saved_offset))
+	{
+	    return;
+	}
     pi = tvb_get_guint8(tvb, offset);
 
     dis_field_pi(tvb, tree, offset, pi);
@@ -2433,7 +2435,7 @@ dis_msg_command(tvbuff_t *tvb, proto_tree *tree, guint32 offset)
     guint32	length;
     guint8	oct;
     guint8	cdl;
-    gchar	*str = NULL;
+    const gchar	*str = NULL;
     gboolean	udhi;
 
 
@@ -2493,13 +2495,13 @@ static gint ett_msgs[NUM_MSGS];
 static void (*gsm_sms_msg_fcn[])(tvbuff_t *tvb, proto_tree *tree, guint32 offset) = {
     dis_msg_deliver,		/* SMS-DELIVER */
     dis_msg_deliver_report,	/* SMS-DELIVER REPORT */
-    dis_msg_submit,		/* SMS-SUBMIT */
+    dis_msg_submit,			/* SMS-SUBMIT */
     dis_msg_submit_report,	/* SMS-SUBMIT REPORT */
     dis_msg_status_report,	/* SMS-STATUS REPORT */
     dis_msg_command,		/* SMS-COMMAND */
-    NULL,			/* Reserved */
-    NULL,			/* Reserved */
-    NULL,	/* NONE */
+    NULL,					/* Reserved */
+    NULL,					/* Reserved */
+    NULL,					/* NONE */
 };
 
 /* GENERIC DISSECTOR FUNCTIONS */
@@ -2514,7 +2516,7 @@ dissect_gsm_sms(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     guint8	msg_type;
     guint8	oct;
     gint	idx;
-    gchar	*str = NULL;
+    const gchar	*str = NULL;
     gint	ett_msg_idx;
 
 
@@ -2555,7 +2557,7 @@ dissect_gsm_sms(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	    msg_type |= ((pinfo->p2p_dir == P2P_DIR_RECV) ? 0x04 : 0x00);
 	}
 
-	str = my_match_strval(msg_type, msg_type_strings, &idx);
+	str = match_strval_idx(msg_type, msg_type_strings, &idx);
 
 	/*
 	 * create the GSM_SMS protocol tree
