@@ -3,7 +3,7 @@
  * Copyright 2004 _FF_ 
  * Francesco Fondelli <fondelli dot francesco, tiscali dot it>
  *
- * $Id: packet-icep.c 15027 2005-07-24 02:43:41Z sahlberg $
+ * $Id: packet-icep.c 17681 2006-03-20 10:52:53Z sahlberg $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -53,7 +53,8 @@
 #include <glib.h>
 
 #include <epan/packet.h>
-#include <epan/dissectors/packet-tcp.h>
+#include <epan/emem.h>
+#include "packet-tcp.h"
 
 #if 0
 #define DBG(str, args...)       do {\
@@ -256,7 +257,7 @@ static void dissect_ice_string(proto_tree *tree, int hf_icep,
 	
 		
 	if ( Size != 0 ) {
-		s = ep_tvb_get_string(tvb, offset, Size);
+		s = tvb_get_ephemeral_string(tvb, offset, Size);
 		if (tree && add_hf)
 			proto_tree_add_string(tree, hf_icep, tvb, offset, Size, s);
 	} else {
@@ -326,12 +327,9 @@ static void dissect_ice_facet(proto_tree *tree, int hf_icep,
 	if ( Size == 0 ) {
 		
 		if (tree) {
-			s = g_malloc( strlen("(empty)") + 1 );
-			sprintf(s, "(empty)");
-			s[strlen("(empty)")] = '\0';
+			s = ep_strdup( "(empty)" );
 			/* display the 0x00 Size byte when click on a empty ice_string */
 			proto_tree_add_string(tree, hf_icep, tvb, offset - 1, 1, s);
-			g_free(s);
 		}
 		return;
 	}
@@ -453,13 +451,10 @@ static void dissect_ice_context(proto_tree *tree, tvbuff_t *tvb, guint32 offset,
 	}
 	
 	if (Size == 0) {
-		s = g_malloc( strlen("(empty)") + 1 );
-		sprintf(s, "(empty)");
-		s[strlen("(empty)")] = '\0';
+		s = "(empty)";
 		/* display the 0x00 Size byte when click on a empty context */
 		if (tree)
 			proto_tree_add_string(tree, hf_icep_context, tvb, offset - 1, 1, s);
-		g_free(s);
 		return;
 	}
 	
