@@ -1,7 +1,7 @@
 /* packet-ipv6.c
  * Routines for IPv6 packet disassembly
  *
- * $Id: packet-ipv6.c 15079 2005-07-26 08:01:16Z jmayer $
+ * $Id: packet-ipv6.c 16983 2006-01-09 23:00:20Z jake $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -34,16 +34,16 @@
 #include <epan/packet.h>
 #include "packet-ipsec.h"
 #include "packet-ipv6.h"
-#include "ip_opts.h"
+#include <epan/ip_opts.h>
 #include <epan/addr_resolv.h>
 #include <epan/prefs.h>
 #include <epan/reassemble.h>
 #include <epan/ipproto.h>
-#include "etypes.h"
-#include "ppptypes.h"
-#include "aftypes.h"
-#include "nlpid.h"
-#include "arcnet_pids.h"
+#include <epan/etypes.h>
+#include <epan/ppptypes.h>
+#include <epan/aftypes.h>
+#include <epan/nlpid.h>
+#include <epan/arcnet_pids.h>
 
 /*
  * NOTE: ipv6.nxt is not very useful as we will have chained header.
@@ -100,7 +100,7 @@ static dissector_handle_t data_handle;
 static dissector_table_t ip_dissector_table;
 
 /* Reassemble fragmented datagrams */
-static gboolean ipv6_reassemble = FALSE;
+static gboolean ipv6_reassemble = TRUE;
 
 #ifndef offsetof
 #define	offsetof(type, member)	((size_t)(&((type *)0)->member))
@@ -178,6 +178,7 @@ again:
       ld->tcp++;
       break;
     case IP_PROTO_UDP:
+    case IP_PROTO_UDPLITE:
       ld->udp++;
       break;
     case IP_PROTO_ICMP:
@@ -383,7 +384,7 @@ dissect_ipv6_options(tvbuff_t *tvb, int offset, guint length,
       optp = NULL;	/* indicate that we don't know this option */
       len_type = VARIABLE_LENGTH;
       optlen = 0;
-      snprintf(name_str, sizeof name_str, "Unknown (0x%02x)", opt);
+      g_snprintf(name_str, sizeof name_str, "Unknown (0x%02x)", opt);
       name = name_str;
       dissect = NULL;
     } else {
@@ -637,10 +638,10 @@ dissect_ipv6(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 
     proto_tree_add_ipv6_hidden(ipv6_tree, hf_ipv6_addr, tvb,
 			       offset + offsetof(struct ip6_hdr, ip6_src), 16,
-			       ipv6.ip6_src.s6_addr8);
+			       ipv6.ip6_src.bytes);
     proto_tree_add_ipv6_hidden(ipv6_tree, hf_ipv6_addr, tvb,
 			       offset + offsetof(struct ip6_hdr, ip6_dst), 16,
-			       ipv6.ip6_dst.s6_addr8);
+			       ipv6.ip6_dst.bytes);
 
     proto_tree_add_ipv6_format(ipv6_tree, hf_ipv6_src, tvb,
 		offset + offsetof(struct ip6_hdr, ip6_src), 16,
