@@ -1,6 +1,6 @@
 /* packet-llc.h
  *
- * $Id: packet-llc.h 13845 2005-03-21 03:55:50Z guy $
+ * $Id: packet-llc.h 15126 2005-07-28 07:53:38Z jmayer $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -33,7 +33,7 @@ void dissect_snap(tvbuff_t *, int, packet_info *, proto_tree *,
 /*
  * Add an entry for a new OUI.
  */
-void llc_add_oui(guint32, const char *, char *, hf_register_info *);
+void llc_add_oui(guint32, const char *, const char *, hf_register_info *);
 
 extern const value_string sap_vals[];
 
