@@ -5,7 +5,7 @@
  *
  * Copyright 2001, Heinz Prantner <heinz.prantner[AT]radisys.com>
  *
- * $Id: packet-m2tp.c 11410 2004-07-18 18:06:47Z gram $
+ * $Id: packet-m2tp.c 15844 2005-09-17 00:02:31Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -36,7 +36,7 @@
 #include <glib.h>
 
 #include <epan/packet.h>
-#include "sctpppids.h"
+#include <epan/sctpppids.h>
 
 #define SCTP_PORT_M2TP        9908  /* unassigned port number (not assigned by IANA) */
 
