@@ -2,7 +2,7 @@
  * Routines for Mobile IP dissection
  * Copyright 2000, Stefan Raab <sraab@cisco.com>
  *
- * $Id: packet-mip.c 13560 2005-03-01 06:03:08Z guy $
+ * $Id: packet-mip.c 16956 2006-01-05 13:01:08Z jmayer $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -98,6 +98,7 @@ static const value_string mip_reply_codes[]= {
   {74, "Reg Deny (FA)- Requested Reverse Tunnel Unavailable"},
   {75, "Reg Deny (FA)- Reverse Tunnel is Mandatory and 'T' Bit Not Set"},
   {76, "Reg Deny (FA)- Mobile Node Too Distant"},
+  {77, "Reg Deny (FA)- Invalid Care-of Address"},
   {79, "Reg Deny (FA)- Delivery Style Not Supported"},
   {80, "Reg Deny (FA)- Home Network Unreachable"},
   {81, "Reg Deny (FA)- HA Host Unreachable"},
@@ -262,7 +263,6 @@ dissect_mip( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   guint8         flags;
   size_t         offset=0;
   const guint8  *reftime;
-  gchar          buff[NTP_TS_SIZE];
 
   /* Make entries in Protocol column and Info column on summary display */
 
@@ -322,7 +322,7 @@ dissect_mip( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	  proto_tree_add_bytes_format(mip_tree, hf_mip_ident, tvb, offset, 8,
 				      reftime,
 				      "Identification: %s",
-				      ntp_fmt_ts(reftime, buff));
+				      ntp_fmt_ts(reftime));
 	  offset += 8;
 
 	} /* if tree */
@@ -362,7 +362,7 @@ dissect_mip( tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	  proto_tree_add_bytes_format(mip_tree, hf_mip_ident, tvb, offset, 8,
 				      reftime,
 				      "Identification: %s",
-				      ntp_fmt_ts(reftime, buff));
+				      ntp_fmt_ts(reftime));
 	  offset += 8;
 	} /* if tree */
 
@@ -403,7 +403,7 @@ void proto_register_mip(void)
 		   "Broadcast Datagrams requested", HFILL }
 	  },
 	  { &hf_mip_d,
-		 { "Co-lcated Care-of Address",           "mip.d",
+		 { "Co-located Care-of Address",           "mip.d",
 		   FT_BOOLEAN, 8, NULL, 32,
 		   "MN using Co-located Care-of address", HFILL }
 	  },
