@@ -1,7 +1,7 @@
 /* packet-nlm.c
  * Routines for nlm dissection
  *
- * $Id: packet-nlm.c 14884 2005-07-09 00:53:17Z guy $
+ * $Id: packet-nlm.c 17480 2006-03-06 14:30:34Z sahlberg $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -208,14 +208,8 @@ nlm_print_msgres_reply(packet_info *pinfo, proto_tree *tree, tvbuff_t *tvb)
 	if(md){
 		nstime_t ns;
 		proto_tree_add_uint(tree, hf_nlm_request_in, tvb, 0, 0, md->req_frame);
-		ns.secs= pinfo->fd->abs_secs-md->ns.secs;
-		ns.nsecs=pinfo->fd->abs_usecs*1000-md->ns.nsecs;
-		if(ns.nsecs<0){
-			ns.nsecs+=1000000000;
-			ns.secs--;
-		}
+		nstime_delta(&ns, &pinfo->fd->abs_ts, &md->ns);
 		proto_tree_add_time(tree, hf_nlm_time, tvb, 0, 0, &ns);
-
 	}
 }
 
@@ -236,14 +230,8 @@ nlm_match_fhandle_reply(packet_info *pinfo, proto_tree *tree)
 
 	md=g_hash_table_lookup(nlm_msg_res_matched, GINT_TO_POINTER(pinfo->fd->num));
 	if(md && md->rep_frame){
-		nfs_fhandle_data_t *fhd;
-		fhd=(nfs_fhandle_data_t *)g_hash_table_lookup(
-			nfs_fhandle_frame_table,
-			GINT_TO_POINTER(md->req_frame));
-		if(fhd){
-			dissect_fhandle_hidden(pinfo,
-				tree, fhd);
-		}
+		dissect_fhandle_hidden(pinfo,
+				tree, md->req_frame);
 	}
 }
 static void
@@ -253,14 +241,8 @@ nlm_match_fhandle_request(packet_info *pinfo, proto_tree *tree)
 
 	md=g_hash_table_lookup(nlm_msg_res_matched, GINT_TO_POINTER(pinfo->fd->num));
 	if(md && md->rep_frame){
-		nfs_fhandle_data_t *fhd;
-		fhd=(nfs_fhandle_data_t *)g_hash_table_lookup(
-			nfs_fhandle_frame_table,
-			GINT_TO_POINTER(md->rep_frame));
-		if(fhd){
-			dissect_fhandle_hidden(pinfo,
-				tree, fhd);
-		}
+		dissect_fhandle_hidden(pinfo,
+				tree, md->rep_frame);
 	}
 }
 
@@ -300,8 +282,7 @@ nlm_register_unmatched_msg(packet_info *pinfo, tvbuff_t *tvb, int offset)
 	/* allocate and build the unmatched structure for this request */
 	umd=g_malloc(sizeof(nlm_msg_res_unmatched_data));
 	umd->req_frame=pinfo->fd->num;
-	umd->ns.secs=pinfo->fd->abs_secs;
-	umd->ns.nsecs=pinfo->fd->abs_usecs*1000;
+	umd->ns=pinfo->fd->abs_ts;
 	umd->cookie_len=tvb_get_ntohl(tvb, offset);
 	umd->cookie=tvb_memdup(tvb, offset+4, umd->cookie_len);
 
