@@ -3,7 +3,7 @@
  * Devin Heitmueller <dheitmueller@netilla.com>
  * Copyright 2003, Tim Potter <tpot@samba.org>
  *
- * $Id: packet-ntlmssp.c 13690 2005-03-10 10:16:49Z sahlberg $
+ * $Id: packet-ntlmssp.c 17215 2006-02-08 08:48:23Z sahlberg $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -36,9 +36,10 @@
 
 #include "packet-windows-common.h"
 #include "packet-smb-common.h"
-#include <epan/asn1.h>		/* XXX - needed for subid_t */
 #include "packet-frame.h"
 #include <epan/prefs.h>
+#include <epan/emem.h>
+#include <epan/tap.h>
 #include <epan/crypt-rc4.h>
 #include <epan/crypt-md4.h>
 #include <epan/crypt-des.h>
@@ -47,6 +48,8 @@
 
 #include "packet-ntlmssp.h"
 
+static int ntlmssp_tap = -1;
+
 /* Message types */
 
 #define NTLMSSP_NEGOTIATE 1
@@ -203,6 +206,7 @@ static int hf_ntlmssp_ntlmv2_response_unknown = -1;
 static int hf_ntlmssp_ntlmv2_response_name = -1;
 static int hf_ntlmssp_ntlmv2_response_name_type = -1;
 static int hf_ntlmssp_ntlmv2_response_name_len = -1;
+static int hf_ntlmssp_ntlmv2_response_client_time = -1;
 
 static gint ett_ntlmssp = -1;
 static gint ett_ntlmssp_negotiate_flags = -1;
@@ -214,7 +218,7 @@ static gint ett_ntlmssp_ntlmv2_response = -1;
 static gint ett_ntlmssp_ntlmv2_response_name = -1;
 
 /* Configuration variables */
-static char *nt_password = NULL;
+static const char *nt_password = NULL;
 
 #define MAX_BLOB_SIZE 256
 typedef struct _ntlmssp_blob {
@@ -233,12 +237,6 @@ typedef struct _ntlmssp_info {
   ntlmssp_blob lm_response;
 } ntlmssp_info;
 
-/*
- * GMemChunk from which ntlmssp_info structures are allocated.
- */
-static GMemChunk  *ntlmssp_info_chunk;
-static int ntlmssp_info_count = 10;
-
 /* If this struct exists in the payload_decrypt, then we have already
    decrypted it once */
 typedef struct _ntlmssp_packet_info {
@@ -249,11 +247,6 @@ typedef struct _ntlmssp_packet_info {
   gboolean verifier_decrypted;
 } ntlmssp_packet_info;
 
-/*
- * GMemChunk from which ntlmssp_packet_info structures are allocated.
- */
-static GMemChunk  *ntlmssp_packet_info_chunk;
-static int ntlmssp_packet_info_count = 10;
 
 /*
  * GSlist of decrypted payloads.
@@ -363,7 +356,8 @@ static int
 dissect_ntlmssp_string (tvbuff_t *tvb, int offset,
 			proto_tree *ntlmssp_tree, 
 			gboolean unicode_strings,
-			int string_hf, int *start, int *end)
+			int string_hf, int *start, int *end,
+			const char **stringp)
 {
   proto_tree *tree = NULL;
   proto_item *tf = NULL;
@@ -380,6 +374,8 @@ dissect_ntlmssp_string (tvbuff_t *tvb, int offset,
     if (ntlmssp_tree)
 	    proto_tree_add_string(ntlmssp_tree, string_hf, tvb,
 				  offset, 8, "NULL");
+    if (stringp != NULL)
+      *stringp = "";
     return offset+8;
   }
 
@@ -387,6 +383,8 @@ dissect_ntlmssp_string (tvbuff_t *tvb, int offset,
   string_text = get_unicode_or_ascii_string(tvb, &string_offset,
 					    unicode_strings, &result_length,
 					    FALSE, TRUE, &bc);
+  if (stringp != NULL)
+    *stringp = string_text;
 
   if (ntlmssp_tree) {
     tf = proto_tree_add_string(ntlmssp_tree, string_hf, tvb,
@@ -597,6 +595,7 @@ dissect_ntlmssp_negotiate_flags (tvbuff_t *tvb, int offset,
 #define NTLM_NAME_NB_DOMAIN  0x0002
 #define NTLM_NAME_DNS_HOST   0x0003
 #define NTLM_NAME_DNS_DOMAIN 0x0004
+#define NTLM_NAME_CLIENT_TIME 0x0007
 
 static const value_string ntlm_name_types[] = {
 	{ NTLM_NAME_END, "End of list" },
@@ -604,6 +603,7 @@ static const value_string ntlm_name_types[] = {
 	{ NTLM_NAME_NB_DOMAIN, "NetBIOS domain name" },
 	{ NTLM_NAME_DNS_HOST, "DNS host name" },
 	{ NTLM_NAME_DNS_DOMAIN, "DNS domain name" },
+	{ NTLM_NAME_CLIENT_TIME, "Client Time" },
 	{ 0, NULL }
 };
 
@@ -689,28 +689,39 @@ dissect_ntlmv2_response(tvbuff_t *tvb, proto_tree *tree, int offset, int len)
 
 		/* Dissect name */
 
-		if (name_len > 0) {
-			name = tvb_fake_unicode(
+		switch(name_type){
+		case NTLM_NAME_END:
+			name = "NULL";
+			proto_item_append_text(
+				name_item, "%s", 
+				val_to_str(name_type, ntlm_name_types,
+					   "Unknown"));
+			break;
+		case NTLM_NAME_CLIENT_TIME:
+			dissect_nt_64bit_time(
+				tvb, name_tree, offset, 
+				hf_ntlmssp_ntlmv2_response_client_time);
+			proto_item_append_text(
+				name_item, "Client Time");
+			break;
+		case NTLM_NAME_NB_HOST:
+		case NTLM_NAME_NB_DOMAIN:
+		case NTLM_NAME_DNS_HOST:
+		case NTLM_NAME_DNS_DOMAIN:
+		default:
+			name = tvb_get_ephemeral_faked_unicode(
 				tvb, offset, name_len / 2, TRUE);
 
 			proto_tree_add_text(
 				name_tree, tvb, offset, name_len, 
 				"Name: %s", name);
-		} else
-			name = g_strdup("NULL");
-
-		if (name_type == 0)
-			proto_item_append_text(
-				name_item, "%s", 
-				val_to_str(name_type, ntlm_name_types,
-					   "Unknown"));
-		else
 			proto_item_append_text(
 				name_item, "%s, %s",
 				val_to_str(name_type, ntlm_name_types,
 					   "Unknown"), name);
+			break;
+		}
 
-		g_free(name);
 
 		offset += name_len;
 
@@ -734,8 +745,9 @@ dissect_ntlmv2_response(tvbuff_t *tvb, proto_tree *tree, int offset, int len)
 	return offset;
 }
 
+/* tapping into ntlmssph not yet implemented */
 static int
-dissect_ntlmssp_negotiate (tvbuff_t *tvb, int offset, proto_tree *ntlmssp_tree)
+dissect_ntlmssp_negotiate (tvbuff_t *tvb, int offset, proto_tree *ntlmssp_tree, ntlmssp_header_t *ntlmssph _U_)
 {
   guint32 negotiate_flags;
   int start;
@@ -754,10 +766,10 @@ dissect_ntlmssp_negotiate (tvbuff_t *tvb, int offset, proto_tree *ntlmssp_tree)
    */
   offset = dissect_ntlmssp_string(tvb, offset, ntlmssp_tree, FALSE, 
 				  hf_ntlmssp_negotiate_domain,
-				  &start, &workstation_end);
+				  &start, &workstation_end, NULL);
   offset = dissect_ntlmssp_string(tvb, offset, ntlmssp_tree, FALSE, 
 				  hf_ntlmssp_negotiate_workstation,
-				  &start, &domain_end);
+				  &start, &domain_end, NULL);
 
   /* XXX - two blobs after this one, sometimes? */
 
@@ -883,9 +895,10 @@ dissect_ntlmssp_address_list (tvbuff_t *tvb, int offset,
   return offset;
 }
 
+/* tapping into ntlmssph not yet implemented */
 static int
 dissect_ntlmssp_challenge (tvbuff_t *tvb, packet_info *pinfo, int offset,
-			   proto_tree *ntlmssp_tree)
+			   proto_tree *ntlmssp_tree, ntlmssp_header_t *ntlmssph _U_)
 {
   guint32 negotiate_flags;
   int item_start, item_end;
@@ -909,7 +922,7 @@ dissect_ntlmssp_challenge (tvbuff_t *tvb, packet_info *pinfo, int offset,
    */
   offset = dissect_ntlmssp_string(tvb, offset, ntlmssp_tree, unicode_strings, 
 			 hf_ntlmssp_challenge_domain,
-			 &item_start, &item_end);
+			 &item_start, &item_end, NULL);
   data_start = item_start;
   data_end = item_end;
 
@@ -935,7 +948,7 @@ dissect_ntlmssp_challenge (tvbuff_t *tvb, packet_info *pinfo, int offset,
   }
 
   if (!conversation_get_proto_data(conversation, proto_ntlmssp)) {
-    conv_ntlmssp_info = g_mem_chunk_alloc(ntlmssp_info_chunk);
+    conv_ntlmssp_info = se_alloc(sizeof(ntlmssp_info));
     /* Insert the flags into the conversation */
     conv_ntlmssp_info->flags = negotiate_flags;
     /* Insert the RC4 state information into the conversation */
@@ -997,7 +1010,7 @@ dissect_ntlmssp_challenge (tvbuff_t *tvb, packet_info *pinfo, int offset,
 
 static int
 dissect_ntlmssp_auth (tvbuff_t *tvb, packet_info *pinfo, int offset,
-		      proto_tree *ntlmssp_tree)
+		      proto_tree *ntlmssp_tree, ntlmssp_header_t *ntlmssph)
 {
   int item_start, item_end;
   int data_start, data_end = 0;
@@ -1077,7 +1090,7 @@ dissect_ntlmssp_auth (tvbuff_t *tvb, packet_info *pinfo, int offset,
   offset = dissect_ntlmssp_string(tvb, offset, ntlmssp_tree, 
 				  unicode_strings, 
 				  hf_ntlmssp_auth_domain,
-				  &item_start, &item_end);
+				  &item_start, &item_end, &(ntlmssph->domain_name));
   data_start = MIN(data_start, item_start);
   data_end = MAX(data_end, item_end);
 
@@ -1086,16 +1099,20 @@ dissect_ntlmssp_auth (tvbuff_t *tvb, packet_info *pinfo, int offset,
   offset = dissect_ntlmssp_string(tvb, offset, ntlmssp_tree, 
 				  unicode_strings, 
 				  hf_ntlmssp_auth_username,
-				  &item_start, &item_end);
+				  &item_start, &item_end, &(ntlmssph->acct_name));
   data_start = MIN(data_start, item_start);
   data_end = MAX(data_end, item_end);
 
+  if (check_col(pinfo->cinfo, COL_INFO))
+    col_append_fstr(pinfo->cinfo, COL_INFO, ", User: %s\\%s",
+		    ntlmssph->domain_name, ntlmssph->acct_name);
+
   /* hostname */
   item_start = tvb_get_letohl(tvb, offset+4);
   offset = dissect_ntlmssp_string(tvb, offset, ntlmssp_tree, 
 				  unicode_strings, 
 				  hf_ntlmssp_auth_hostname,
-				  &item_start, &item_end);
+				  &item_start, &item_end, &(ntlmssph->host_name));
   data_start = MIN(data_start, item_start);
   data_end = MAX(data_end, item_end);
 
@@ -1120,10 +1137,16 @@ dissect_ntlmssp_auth (tvbuff_t *tvb, packet_info *pinfo, int offset,
 static void
 dissect_ntlmssp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 {
-  guint32 ntlmssp_message_type;
   volatile int offset = 0;
   proto_tree *volatile ntlmssp_tree = NULL;
   proto_item *tf = NULL;
+  ntlmssp_header_t *ntlmssph;
+
+  ntlmssph=ep_alloc(sizeof(ntlmssp_header_t));
+  ntlmssph->type=0;
+  ntlmssph->domain_name=NULL;
+  ntlmssph->acct_name=NULL;
+  ntlmssph->host_name=NULL;
 
   /* Setup a new tree for the NTLMSSP payload */
   if (tree) {
@@ -1156,28 +1179,28 @@ dissect_ntlmssp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     /* NTLMSSP Message Type */
     proto_tree_add_item (ntlmssp_tree, hf_ntlmssp_message_type,
 			 tvb, offset, 4, TRUE);
-    ntlmssp_message_type = tvb_get_letohl (tvb, offset);
+    ntlmssph->type = tvb_get_letohl (tvb, offset);
     offset += 4; 
 
     if (check_col(pinfo->cinfo, COL_INFO))
 	    col_append_fstr(pinfo->cinfo, COL_INFO, ", %s",
-			    val_to_str(ntlmssp_message_type, 
+			    val_to_str(ntlmssph->type, 
 				       ntlmssp_message_types,
 				       "Unknown message type"));
 
     /* Call the appropriate dissector based on the Message Type */
-    switch (ntlmssp_message_type) {
+    switch (ntlmssph->type) {
 
     case NTLMSSP_NEGOTIATE:
-      offset = dissect_ntlmssp_negotiate (tvb, offset, ntlmssp_tree);
+      offset = dissect_ntlmssp_negotiate (tvb, offset, ntlmssp_tree, ntlmssph);
       break;
 
     case NTLMSSP_CHALLENGE:
-      offset = dissect_ntlmssp_challenge (tvb, pinfo, offset, ntlmssp_tree);
+      offset = dissect_ntlmssp_challenge (tvb, pinfo, offset, ntlmssp_tree, ntlmssph);
       break;
 
     case NTLMSSP_AUTH:
-      offset = dissect_ntlmssp_auth (tvb, pinfo, offset, ntlmssp_tree);
+      offset = dissect_ntlmssp_auth (tvb, pinfo, offset, ntlmssp_tree, ntlmssph);
       break;
 
     default:
@@ -1191,6 +1214,8 @@ dissect_ntlmssp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
   } CATCH(ReportedBoundsError) {
     show_reported_bounds_error(tvb, pinfo, tree);
   } ENDTRY;
+
+  tap_queue_packet(ntlmssp_tap, pinfo, ntlmssph);
 }
 
 /*
@@ -1432,7 +1457,7 @@ dissect_ntlmssp_encrypted_payload(tvbuff_t *data_tvb,
   packet_ntlmssp_info = p_get_proto_data(pinfo->fd, proto_ntlmssp);
   if (packet_ntlmssp_info == NULL) {
     /* We don't have any packet state, so create one */
-    packet_ntlmssp_info = g_mem_chunk_alloc(ntlmssp_packet_info_chunk);
+    packet_ntlmssp_info = se_alloc(sizeof(ntlmssp_packet_info));
     memset(packet_ntlmssp_info, 0, sizeof(ntlmssp_packet_info));
     p_add_proto_data(pinfo->fd, proto_ntlmssp, packet_ntlmssp_info);
   }
@@ -1513,11 +1538,6 @@ free_payload(gpointer decrypted_payload, gpointer user_data _U_)
 static void
 ntlmssp_init_protocol(void)
 {
-	if (ntlmssp_info_chunk != NULL)
-		g_mem_chunk_destroy(ntlmssp_info_chunk);
-	if (ntlmssp_packet_info_chunk != NULL)
-		g_mem_chunk_destroy(ntlmssp_packet_info_chunk);
-
 	/*
 	 * Free the decrypted payloads, and then free the list of decrypted
 	 * payloads.
@@ -1528,14 +1548,6 @@ ntlmssp_init_protocol(void)
 		decrypted_payloads = NULL;
 	}
 
-	ntlmssp_info_chunk = g_mem_chunk_new("ntlmssp_info_chunk",
-	    sizeof(ntlmssp_info),
-	    ntlmssp_info_count * sizeof(ntlmssp_info),
-	    G_ALLOC_ONLY);
-	ntlmssp_packet_info_chunk = g_mem_chunk_new("ntlmssp_packet_info_chunk",
-	    sizeof(ntlmssp_packet_info),
-	    ntlmssp_packet_info_count * sizeof(ntlmssp_packet_info),
-	    G_ALLOC_ONLY);
 }
 
 void
@@ -1718,7 +1730,9 @@ proto_register_ntlmssp(void)
     { &hf_ntlmssp_ntlmv2_response_name_type,
       { "Name type", "ntlmssp.ntlmv2response.name.type", FT_UINT32, BASE_DEC, VALS(ntlm_name_types), 0x0, "", HFILL }},
     { &hf_ntlmssp_ntlmv2_response_name_len,
-      { "Name len", "ntlmssp.ntlmv2response.name.len", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }}
+      { "Name len", "ntlmssp.ntlmv2response.name.len", FT_UINT32, BASE_DEC, NULL, 0x0, "", HFILL }},
+    { &hf_ntlmssp_ntlmv2_response_client_time,
+      { "Client Time", "ntlmssp.ntlmv2response.client_time", FT_ABSOLUTE_TIME, BASE_NONE, NULL, 0, "", HFILL }}
   };
 
 
@@ -1834,4 +1848,5 @@ proto_reg_handoff_ntlmssp(void)
   register_dcerpc_auth_subdissector(DCE_C_AUTHN_LEVEL_PKT_PRIVACY,
 				    DCE_C_RPC_AUTHN_PROTOCOL_NTLMSSP,
 				    &ntlmssp_seal_fns);
+  ntlmssp_tap = register_tap("ntlmssp");
 }
