@@ -2,7 +2,7 @@
  * Routines for ISO/OSI network and transport protocol packet disassembly
  * Main entrance point and common functions
  *
- * $Id: packet-osi.c 13130 2005-01-20 05:40:56Z guy $
+ * $Id: packet-osi.c 15844 2005-09-17 00:02:31Z guy $
  * Laurent Deniel <laurent.deniel@free.fr>
  * Ralf Schneider <Ralf.Schneider@t-online.de>
  *
@@ -34,11 +34,11 @@
 #include <ctype.h>
 #include <glib.h>
 #include <epan/packet.h>
-#include "llcsaps.h"
-#include "aftypes.h"
-#include "nlpid.h"
-#include "ppptypes.h"
-#include "chdlctypes.h"
+#include <epan/llcsaps.h>
+#include <epan/aftypes.h>
+#include <epan/nlpid.h>
+#include <epan/ppptypes.h>
+#include <epan/chdlctypes.h>
 #include "packet-osi.h"
 #include "packet-isis.h"
 #include "packet-esis.h"
