@@ -4,7 +4,13 @@
  * Copyright 2004, Dominic Bchaz <bdo@zhwin.ch> , ZHW/InES
  * Copyright 2004, Markus Seehofer <mseehofe@nt.hirschmann.de>
  *
- * $Id: packet-ptp.c 12158 2004-09-30 19:54:01Z guy $
+ * Revisions:
+ * - Markus Seehofer 09.08.2005 <mseehofe@nt.hirschmann.de>
+ *   - Included the "startingBoundaryHops" field in
+ *     ptp_management messages.
+ * -
+ * 
+ * $Id: packet-ptp.c 17038 2006-01-16 07:59:44Z jmayer $
  *
  * A plugin for:
  *
@@ -37,10 +43,6 @@
 
 #include <glib.h>
 
-#ifdef NEED_SNPRINTF_H
-# include "snprintf.h"
-#endif
-
 #include <epan/packet.h>
 
 
@@ -121,6 +123,7 @@
 #define	PTP_MM_TARGETCOMMUNICATIONTECHNOLOGY_OFFSET			41
 #define	PTP_MM_TARGETUUID_OFFSET							42
 #define	PTP_MM_TARGETPORTID_OFFSET							48
+#define	PTP_MM_STARTINGBOUNDARYHOPS_OFFSET					50
 #define	PTP_MM_BOUNDARYHOPS_OFFSET							52
 #define	PTP_MM_MANAGEMENTMESSAGEKEY_OFFSET					55
 #define	PTP_MM_PARAMETERLENGTH_OFFSET						58
@@ -516,6 +519,7 @@ static int hf_ptp_dr_requestingsourcesequenceid = -1;
 static int hf_ptp_mm_targetcommunicationtechnology = -1;
 static int hf_ptp_mm_targetuuid = -1;
 static int hf_ptp_mm_targetportid = -1;
+static int hf_ptp_mm_startingboundaryhops = -1;
 static int hf_ptp_mm_boundaryhops = -1;
 static int hf_ptp_mm_managementmessagekey = -1;
 static int hf_ptp_mm_parameterlength = -1;
@@ -664,7 +668,7 @@ dissect_ptp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		col_set_str(pinfo->cinfo, COL_PROTOCOL, "PTP");
 
 
-/* Get control field (what kind of message is this? (Sync, DelayReq, ....) */
+/* Get control field (what kind of message is this? (Sync, DelayReq, ...) */
 
 	ptp_control = tvb_get_guint8 (tvb, PTP_CONTROL_OFFSET);
 	/* MGMT packet? */
@@ -931,9 +935,13 @@ dissect_ptp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 				proto_tree_add_item(ptp_tree,
 		    		    hf_ptp_mm_targetportid, tvb, PTP_MM_TARGETPORTID_OFFSET, 2, FALSE);
 
+				proto_tree_add_item(ptp_tree,
+		    		    hf_ptp_mm_startingboundaryhops, tvb, PTP_MM_STARTINGBOUNDARYHOPS_OFFSET, 2, FALSE);
+
 				proto_tree_add_item(ptp_tree,
 		    		    hf_ptp_mm_boundaryhops, tvb, PTP_MM_BOUNDARYHOPS_OFFSET, 2, FALSE);
 
+
 				proto_tree_add_item(ptp_tree,
 		    		    hf_ptp_mm_managementmessagekey, tvb, PTP_MM_MANAGEMENTMESSAGEKEY_OFFSET, 1, FALSE);
 
@@ -1605,6 +1613,11 @@ proto_register_ptp(void)
 			FT_UINT16, BASE_DEC, NULL, 0x00,
 			"", HFILL }
 		},
+		{ &hf_ptp_mm_startingboundaryhops,
+			{ "startingBoundaryHops",           "ptp.mm.startingboundaryhops",
+			FT_INT16, BASE_DEC, NULL, 0x00,
+			"", HFILL }
+		},
 		{ &hf_ptp_mm_boundaryhops,
 			{ "boundaryHops",           "ptp.mm.boundaryhops",
 			FT_INT16, BASE_DEC, NULL, 0x00,
