@@ -5,7 +5,7 @@
  * Changed 03/12/2003 Rui Carmo (http://the.taoofmac.com - added all 3GPP VSAs, some parsing)
  * Changed 07/2005 Luis Ontanon <luis.ontanon@gmail.com> - use FreeRADIUS' dictionary
  *
- * $Id: packet-radius.c 15089 2005-07-26 15:59:54Z gerald $
+ * $Id: packet-radius.c 17038 2006-01-16 07:59:44Z jmayer $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -60,6 +60,7 @@
 #include <epan/crypt-md5.h>
 #include <epan/sminmpec.h>
 #include <epan/filesystem.h>
+#include <epan/addr_resolv.h>
 #include <epan/emem.h>
 
 #include "packet-radius.h"
@@ -122,6 +123,10 @@ static int hf_radius_code = -1;
 static int hf_radius_length = -1;
 static int hf_radius_authenticator = -1;
 
+static int hf_radius_framed_ip_address = -1;
+static int hf_radius_login_ip_host = -1;
+static int hf_radius_framed_ipx_network = -1;
+
 static int hf_radius_cosine_vpi = -1;
 static int hf_radius_cosine_vci = -1;
 
@@ -129,12 +134,17 @@ static gint ett_radius = -1;
 static gint ett_radius_avp = -1;
 static gint ett_eap = -1;
 
+radius_vendor_info_t no_vendor = {"Unknown Vendor",0,NULL,-1};
 
 radius_attr_info_t no_dictionary_entry = {"Unknown-Attribute",0,FALSE,FALSE,radius_octets, NULL, NULL, -1, -1, -1, -1, -1 };
 
 dissector_handle_t eap_handle;
+dissector_handle_t radius_handle;
 
 static const gchar* shared_secret = "";
+static gboolean show_length = FALSE;
+static guint alt_port = 0;
+static guint alt_port_pref = 0;
 
 static guint8 authenticator[AUTHENTICATOR_LENGTH];
 
@@ -171,7 +181,90 @@ static const value_string radius_vals[] =
 	{0, NULL}
 };
 
-static gchar* dissect_cosine_vpvc(proto_tree* tree, tvbuff_t* tvb) {
+static const gchar *dissect_framed_ip_address(proto_tree* tree, tvbuff_t* tvb) {
+	int len;
+	guint32 ip;
+	guint32 ip_h;
+	const gchar *str;
+
+	len = tvb_length(tvb);
+	if (len != 4)
+		return "[wrong length for IP address]";
+	
+	ip=tvb_get_ipv4(tvb,0);
+	ip_h=g_ntohl(ip);
+
+	if (ip_h == 0xFFFFFFFF) {
+		str = "Negotiated";
+		proto_tree_add_ipv4_format(tree, hf_radius_framed_ip_address,
+		    tvb, 0, len, ip, "Framed-IP-Address: %s", str);
+	} else if (ip_h == 0xFFFFFFFE) {
+		str = "Assigned";
+		proto_tree_add_ipv4_format(tree, hf_radius_framed_ip_address,
+		    tvb, 0, len, ip, "Framed-IP-Address: %s", str);
+	} else {
+		str = ip_to_str((guint8 *)&ip);
+		proto_tree_add_ipv4_format(tree, hf_radius_framed_ip_address,
+		    tvb, 0, len, ip, "Framed-IP-Address: %s (%s)",
+		    get_hostname(ip), str);
+	}
+
+	return str;
+}
+
+static const gchar *dissect_login_ip_host(proto_tree* tree, tvbuff_t* tvb) {
+	int len;
+	guint32 ip;
+	guint32 ip_h;
+	const gchar *str;
+
+	len = tvb_length(tvb);
+	if (len != 4)
+		return "[wrong length for IP address]";
+	
+	ip=tvb_get_ipv4(tvb,0);
+	ip_h=g_ntohl(ip);
+
+	if (ip_h == 0xFFFFFFFF) {
+		str = "User-selected";
+		proto_tree_add_ipv4_format(tree, hf_radius_login_ip_host,
+		    tvb, 0, len, ip, "Login-IP-Host: %s", str);
+	} else if (ip_h == 0) {
+		str = "NAS-selected";
+		proto_tree_add_ipv4_format(tree, hf_radius_login_ip_host,
+		    tvb, 0, len, ip, "Login-IP-Host: %s", str);
+	} else {
+		str = ip_to_str((guint8 *)&ip);
+		proto_tree_add_ipv4_format(tree, hf_radius_framed_ip_address,
+		    tvb, 0, len, ip, "Login-IP-Host: %s (%s)",
+		    get_hostname(ip), str);
+	}
+
+	return str;
+}
+
+static const gchar *dissect_framed_ipx_network(proto_tree* tree, tvbuff_t* tvb) {
+	int len;
+	guint32 net;
+	const gchar *str;
+
+	len = tvb_length(tvb);
+	if (len != 4)
+		return "[wrong length for IPX network]";
+	
+	net=tvb_get_ntohl(tvb,0);
+
+	if (net == 0xFFFFFFFE)
+		str = "NAS-selected";
+	else
+		str = ep_strdup_printf("0x%08X", net);
+	proto_tree_add_ipxnet_format(tree, hf_radius_framed_ipx_network, tvb, 0,
+	    len, net, "Framed-IPX-Network: %s", str);
+
+	return str;
+}
+
+static const gchar* dissect_cosine_vpvc(proto_tree* tree, tvbuff_t* tvb) {
 	guint vpi, vci;
 	
 	if ( tvb_length(tvb) != 4 )
@@ -187,19 +280,21 @@ static gchar* dissect_cosine_vpvc(proto_tree* tree, tvbuff_t* tvb) {
 }
 
 static void
-radius_decrypt_avp(gchar *dest,tvbuff_t *tvb,int offset,int length)
+radius_decrypt_avp(gchar *dest,int dest_len,tvbuff_t *tvb,int offset,int length)
 {
     md5_state_t md_ctx;
     md5_byte_t digest[16];
     int i;
-    int totlen;
+    size_t totlen, returned_length;
     const guint8 *pd;
     guchar c;
 	
+    DISSECTOR_ASSERT(dest_len > 2);  /* \"\"\0 */
     dest[0] = '"';
     dest[1] = '\0';
     totlen = 1;
-	
+    dest_len -= 1; /* Need to add trailing \" */
+
     md5_init(&md_ctx);
     md5_append(&md_ctx,(const guint8*)shared_secret,strlen(shared_secret));
     md5_append(&md_ctx,authenticator, AUTHENTICATOR_LENGTH);
@@ -208,26 +303,29 @@ radius_decrypt_avp(gchar *dest,tvbuff_t *tvb,int offset,int length)
     pd = tvb_get_ptr(tvb,offset,length);
     for( i = 0 ; i < AUTHENTICATOR_LENGTH && i < length ; i++ ) {
 		c = pd[i] ^ digest[i];
-		if ( isprint(c)) {
-			dest[totlen] = c;
-			totlen++;
+		if ( isprint(c) ) {
+			returned_length = g_snprintf(&dest[totlen], dest_len-totlen,
+				"%c",c);
+			totlen += MIN(returned_length, dest_len-totlen-1);
 		} else {
-			sprintf(&(dest[totlen]),"\\%03o",c);
-			totlen += strlen(&(dest[totlen]));
+			returned_length = g_snprintf(&dest[totlen], dest_len-totlen,
+				"\\%03o",c);
+			totlen += MIN(returned_length, dest_len-totlen-1);
 		}
     }
     while(i<length) {
 		if ( isprint(pd[i]) ) {
-			dest[totlen] = (gchar)pd[i];
-			totlen++;
+			returned_length = g_snprintf(&dest[totlen], dest_len-totlen,
+				"%c", pd[i]);
+			totlen += MIN(returned_length, dest_len-totlen-1);
 		} else {
-			sprintf(&(dest[totlen]), "\\%03o", pd[i]);
-			totlen=totlen+strlen(&(dest[totlen]));
+			returned_length = g_snprintf(&dest[totlen], dest_len-totlen,
+				"\\%03o", pd[i]);
+			totlen += MIN(returned_length, dest_len-totlen-1);
 		}
 		i++;
     }
-    dest[totlen]='"';
-    dest[totlen+1] = '\0';
+    g_snprintf(&dest[totlen], dest_len+1-totlen, "%c", '"');
 }
 
 
@@ -270,8 +368,9 @@ void radius_string(radius_attr_info_t* a, proto_tree* tree, packet_info *pinfo _
 			proto_item_append_text(avp_item, "Encrypted");
 			proto_tree_add_item(tree, a->hf, tvb, offset, len, FALSE);
 		} else {
-			gchar buffer[1024]; /* an AVP value can be at most 253 bytes */
-			radius_decrypt_avp(buffer,tvb,offset,len);
+			gchar *buffer; 
+			buffer=ep_alloc(1024); /* an AVP value can be at most 253 bytes */
+			radius_decrypt_avp(buffer,1024,tvb,offset,len);
 			proto_item_append_text(avp_item, "Decrypted: %s", buffer);
 			proto_tree_add_string(tree, a->hf, tvb, offset, len, buffer);
 		}
@@ -287,7 +386,7 @@ void radius_octets(radius_attr_info_t* a, proto_tree* tree, packet_info *pinfo _
 }
 
 void radius_ipaddr(radius_attr_info_t* a, proto_tree* tree, packet_info *pinfo _U_, tvbuff_t* tvb, int offset, int len, proto_item* avp_item) {
-	guint8 ip[4];
+	guint32 ip;
 	gchar buf[16];
 	
 	if (len != 4) {
@@ -295,19 +394,18 @@ void radius_ipaddr(radius_attr_info_t* a, proto_tree* tree, packet_info *pinfo _
 		return;
 	}
 	
-	tvb_memcpy(tvb,ip,offset,4);
+	ip=tvb_get_ipv4(tvb,offset);
 	
 	proto_tree_add_item(tree, a->hf, tvb, offset, len, FALSE);
 
-	ip_to_str_buf(ip, buf);
+	ip_to_str_buf((guint8 *)&ip, buf);
 	proto_item_append_text(avp_item, "%s", buf);
 }
 
 void radius_ipv6addr(radius_attr_info_t* a, proto_tree* tree, packet_info *pinfo _U_, tvbuff_t* tvb, int offset, int len, proto_item* avp_item) {
-	guint8 ipv6_buff[64];
+	struct e_in6_addr ipv6_buff;
 	gchar txtbuf[256];
 
-	/* XXX ??? */
 	if (len != 16) {
 		proto_item_append_text(avp_item, "[wrong length for IPv6 address]");
 		return;
@@ -315,11 +413,26 @@ void radius_ipv6addr(radius_attr_info_t* a, proto_tree* tree, packet_info *pinfo
 	
 	proto_tree_add_item(tree, a->hf, tvb, offset, len, FALSE);
 	
-	tvb_memcpy(tvb, ipv6_buff, offset, len);
-	ip6_to_str_buf((struct e_in6_addr *)ipv6_buff, txtbuf);
+	tvb_get_ipv6(tvb, offset, &ipv6_buff);
+	ip6_to_str_buf(&ipv6_buff, txtbuf);
 	proto_item_append_text(avp_item, "%s", txtbuf);
 }
 
+void radius_ipxnet(radius_attr_info_t* a, proto_tree* tree, packet_info *pinfo _U_, tvbuff_t* tvb, int offset, int len, proto_item* avp_item) {
+	guint32 net;
+	
+	if (len != 4) {
+		proto_item_append_text(avp_item, "[wrong length for IPX network]");
+		return;
+	}
+	
+	net=tvb_get_ntohl(tvb,offset);
+	
+	proto_tree_add_item(tree, a->hf, tvb, offset, len, FALSE);
+
+	proto_item_append_text(avp_item, "0x%08X", net);
+}
+
 void radius_date(radius_attr_info_t* a, proto_tree* tree, packet_info *pinfo _U_, tvbuff_t* tvb, int offset, int len, proto_item* avp_item) {
 	nstime_t time_ptr; 
 
@@ -344,285 +457,326 @@ void radius_ifid(radius_attr_info_t* a, proto_tree* tree, packet_info *pinfo _U_
 	proto_item_append_text(avp_item, "%s", tvb_bytes_to_str(tvb, offset, len));
 }
 
+static void add_avp_to_tree(proto_tree* avp_tree, proto_item* avp_item, packet_info* pinfo, tvbuff_t* tvb, radius_attr_info_t* dictionary_entry, guint32 avp_length, guint32 offset) {
+    proto_item* pi;
+
+    if (dictionary_entry->tagged) {
+        guint tag;
+        
+        if (avp_length < 3) {
+            pi = proto_tree_add_text(avp_tree, tvb, offset,
+                                0, "AVP too short for tag");
+            PROTO_ITEM_SET_GENERATED(pi);
+            return;
+        }
+        
+        tag = tvb_get_guint8(tvb, offset);
+        
+        if (tag <=  0x1f) {
+            proto_tree_add_uint(avp_tree,
+                                dictionary_entry->hf_tag,
+                                tvb, offset, 1, tag);
+            
+            proto_item_append_text(avp_item,
+                                   " Tag=0x%.2x", tag);
+            
+            offset++;
+            avp_length--;
+        }
+    }
+    
+    if ( dictionary_entry->dissector ) {
+        tvbuff_t* tvb_value;
+        const gchar* str;
+        
+        tvb_value = tvb_new_subset(tvb, offset, avp_length, (gint) avp_length);
+        
+        str = dictionary_entry->dissector(avp_tree,tvb_value);
+        
+        proto_item_append_text(avp_item, ": %s",str);
+    } else {
+        proto_item_append_text(avp_item, ": ");
+        
+        dictionary_entry->type(dictionary_entry,avp_tree,pinfo,tvb,offset,avp_length,avp_item);
+    }
+}
 
 static void dissect_attribute_value_pairs(proto_tree *tree, packet_info *pinfo, tvbuff_t *tvb, int offset, guint length) {
-	gboolean last_eap = FALSE;
-	guint8* eap_buffer = NULL;
-	guint eap_seg_num = 0;
-	guint eap_tot_len_captured = 0;
-	guint eap_tot_len = 0;
-	proto_tree* eap_tree = NULL;
-	tvbuff_t* eap_tvb = NULL;
+    proto_item* item;
+    gboolean last_eap = FALSE;
+    guint8* eap_buffer = NULL;
+    guint eap_seg_num = 0;
+    guint eap_tot_len_captured = 0;
+    guint eap_tot_len = 0;
+    proto_tree* eap_tree = NULL;
+    tvbuff_t* eap_tvb = NULL;
 
-	/*
-	 * In case we throw an exception, clean up whatever stuff we've
-	 * allocated (if any).
-	 */
-	CLEANUP_PUSH(g_free, eap_buffer);
+    /*
+     * In case we throw an exception, clean up whatever stuff we've
+     * allocated (if any).
+     */
+    CLEANUP_PUSH(g_free, eap_buffer);
 
-	while (length > 0) {
-		radius_attr_info_t* dictionary_entry = NULL;
-		radius_vendor_info_t* vendor = NULL;
-		gint tvb_len;
-		guint32 avp_type;
-		guint32 avp_length;
-		guint32 vendor_id = 0;
-		guint32 avp_vsa_type = 0;
-		guint32 avp_vsa_len = 0;
-		proto_item* avp_item;
-		proto_item* avp_len_item;
-		proto_tree* avp_tree;
-		
-		if (length < 2) {
-			proto_tree_add_text(tree, tvb, offset, 0,
-					    "[Not enough room in packet for AVP header]");
-			return;
-		}
-		avp_type = tvb_get_guint8(tvb,offset);
-		avp_length = tvb_get_guint8(tvb,offset+1);
+    while (length > 0) {
+        radius_attr_info_t* dictionary_entry = NULL;
+        gint tvb_len;
+        guint32 avp_type;
+        guint32 avp_length;
+        guint32 vendor_id;
 
-		if (avp_length < 1) {
-			proto_tree_add_text(tree, tvb, offset+1, 1,
-					    "[AVP too short]");
-			return;
-		}
+        proto_item* avp_item;
+        proto_item* avp_len_item;
+        proto_tree* avp_tree;
+        
+        if (length < 2) {
+            item = proto_tree_add_text(tree, tvb, offset, 0,
+                        "Not enough room in packet for AVP header");
+            PROTO_ITEM_SET_GENERATED(item);
+            return;
+        }
+        avp_type = tvb_get_guint8(tvb,offset);
+        avp_length = tvb_get_guint8(tvb,offset+1);
 
-		if (length < avp_length) {
-			proto_tree_add_text(tree, tvb, offset, 0,
-					    "[Not enough room in packet for AVP]");
-			return;
-		}
-		
-		length -= avp_length;
+        if (avp_length < 2) {
+            item = proto_tree_add_text(tree, tvb, offset, 0,
+                        "AVP too short: length %u < 2", avp_length);
+            PROTO_ITEM_SET_GENERATED(item);
+            return;
+        }
 
-		avp_item = proto_tree_add_text(tree,tvb,offset,avp_length,"AVP: l=%u ",avp_length);
-		
-		if (avp_type == RADIUS_VENDOR_SPECIFIC_CODE) {
-			/* XXX TODO: handle 2 byte codes for USR */
-			
-			if (avp_length < 8) {
-				proto_item_append_text(avp_item, "[AVP too short]");
-				offset += avp_length;
-				continue;
-			}
-			vendor_id = tvb_get_ntohl(tvb,offset+2);
-			avp_vsa_type = tvb_get_guint8(tvb,offset+6);
-			avp_vsa_len = tvb_get_guint8(tvb,offset+7);
-			
-			vendor = g_hash_table_lookup(dict->vendors_by_id,GUINT_TO_POINTER(vendor_id));
-			
-			if (vendor) {
-				proto_item_append_text(avp_item, "v=%s(%u)", vendor->name,vendor_id);
-				
-				dictionary_entry = g_hash_table_lookup(vendor->attrs_by_id,GUINT_TO_POINTER(avp_vsa_type));
-			} else {
-				proto_item_append_text(avp_item, "v=Unknown(%u)", vendor_id);
-			}
-			
-			if (! dictionary_entry ) {
-				dictionary_entry = &no_dictionary_entry;
-			}
-			
-			proto_item_append_text(avp_item, " t=%s(%u)", dictionary_entry->name,avp_vsa_type);
-			
-			avp_length -= 8;
-			offset += 8;
-		} else {
-			dictionary_entry = g_hash_table_lookup(dict->attrs_by_id,GUINT_TO_POINTER(avp_type));
-			
-			if (! dictionary_entry ) {
-				dictionary_entry = &no_dictionary_entry;
-			}
-			
-			proto_item_append_text(avp_item, " t=%s(%u)", dictionary_entry->name, avp_type);
-			
-			if (avp_length < 2) {
-				proto_item_append_text(avp_item, "[AVP too short]");
-				offset += avp_length;
-				continue;
-			}
-			avp_length -= 2;
-			offset += 2;
-		}
-		
-		avp_tree = proto_item_add_subtree(avp_item,dictionary_entry->ett);
+        if (length < avp_length) {
+            item = proto_tree_add_text(tree, tvb, offset, 0,
+                        "Not enough room in packet for AVP");
+            PROTO_ITEM_SET_GENERATED(item);
+            return;
+        }
+        
+        length -= avp_length;
 
-		avp_len_item = proto_tree_add_uint(avp_tree,
-						   dictionary_entry->hf_len,
-						   tvb,0,0,avp_length);
-		PROTO_ITEM_SET_GENERATED(avp_len_item);
+        dictionary_entry = g_hash_table_lookup(dict->attrs_by_id,GUINT_TO_POINTER(avp_type));
+            
+        if (! dictionary_entry ) {
+            dictionary_entry = &no_dictionary_entry;
+        }
+            
+        avp_item = proto_tree_add_text(tree, tvb, offset, avp_length,
+                                       "AVP: l=%u  t=%s(%u)", avp_length,
+                                       dictionary_entry->name, avp_type);
+        
+        avp_length -= 2;
+        offset += 2;
 
-		tvb_len = tvb_length_remaining(tvb, offset);
-		if ((gint)avp_length < tvb_len)
-			tvb_len = avp_length;
+        if (avp_type == RADIUS_VENDOR_SPECIFIC_CODE) {
+            radius_vendor_info_t* vendor;
+            proto_tree* vendor_tree;
+            gint max_offset = offset + avp_length;
+            const gchar* vendor_str;
 
-		if (avp_type == RADIUS_EAP_MESSAGE_CODE) {
-			eap_seg_num++;
+            /* XXX TODO: handle 2 byte codes for USR */
+            
+            if (avp_length < 4) {
+                proto_item_append_text(avp_item, " [AVP too short; no room for vendor ID]");
+                offset += avp_length;
+                continue;
+            }
+            vendor_id = tvb_get_ntohl(tvb,offset);
+            
+            avp_length -= 4;
+            offset += 4;
+            
+            vendor = g_hash_table_lookup(dict->vendors_by_id,GUINT_TO_POINTER(vendor_id));
+            if (vendor) {
+            	vendor_str = vendor->name;
+            } else {
+                vendor_str = val_to_str(vendor_id, sminmpec_values, "Unknown");
+                vendor = &no_vendor;
+            }
+            proto_item_append_text(avp_item, " v=%s(%u)", vendor_str,
+                                   vendor_id);
+            
+            vendor_tree = proto_item_add_subtree(avp_item,vendor->ett);
+            
+            while (offset < max_offset) {
+                guint32 avp_vsa_type = tvb_get_guint8(tvb,offset++);
+                guint32 avp_vsa_len = tvb_get_guint8(tvb,offset++);
+                
+                
+                if (avp_vsa_len < 2) {
+                    proto_tree_add_text(tree, tvb, offset+1, 1,
+                                            "[VSA too short]");
+                    return;
+                }
+                
+                avp_vsa_len -= 2;
+                
+                dictionary_entry = g_hash_table_lookup(vendor->attrs_by_id,GUINT_TO_POINTER(avp_vsa_type));
+                
+                if ( !dictionary_entry ) {
+                    dictionary_entry = &no_dictionary_entry;
+                }
+                
+                avp_item = proto_tree_add_text(vendor_tree,tvb,offset-2,avp_vsa_len+2,
+                                               "VSA: l=%u t=%s(%u)",
+                                               avp_vsa_len+2, dictionary_entry->name, avp_vsa_type);
+                
+                avp_tree = proto_item_add_subtree(avp_item,dictionary_entry->ett);
 
-			/* Show this as an EAP fragment. */
-			if (tree)
-				proto_tree_add_text(avp_tree, tvb,
-				    offset, tvb_len, "EAP fragment");
+                if (show_length) {
+                    avp_len_item = proto_tree_add_uint(avp_tree,
+                                                       dictionary_entry->hf_len,
+                                                       tvb,0,0,avp_length);
+                    PROTO_ITEM_SET_GENERATED(avp_len_item);
+                }
+                
+                add_avp_to_tree(avp_tree, avp_item, pinfo, tvb, dictionary_entry, avp_vsa_len, offset);
+                
+                offset += avp_vsa_len;
+            };
+            continue;
+        }
 
-			if (eap_tvb != NULL) {
-				/*
-				 * Oops, a non-consecutive EAP-Message
-				 * attribute.
-				 */
-				proto_item_append_text(avp_item,
-				    " (non-consecutive)");
-			} else {
-				/*
-				 * RFC 2869 says, in section 5.13, describing
-				 * the EAP-Message attribute:
-				 *
-				 *    The NAS places EAP messages received
-				 *    from the authenticating peer into one
-				 *    or more EAP-Message attributes and
-				 *    forwards them to the RADIUS Server
-				 *    within an Access-Request message.
-				 *    If multiple EAP-Messages are
-				 *    contained within an Access-Request or
-				 *    Access-Challenge packet, they MUST be
-				 *    in order and they MUST be consecutive
-				 *    attributes in the Access-Request or
-				 *    Access-Challenge packet.
-				 *
-				 *		...
-				 *
-				 *    The String field contains EAP packets,
-				 *    as defined in [3].  If multiple
-				 *    EAP-Message attributes are present
-				 *    in a packet their values should be
-				 *    concatenated; this allows EAP packets
-				 *    longer than 253 octets to be passed
-				 *    by RADIUS.
-				 *
-				 * Do reassembly of EAP-Message attributes.
-				 * We just concatenate all the attributes,
-				 * and when we see either the end of the
-				 * attribute list or a non-EAP-Message
-				 * attribute, we know we're done.
-				 */
+        avp_tree = proto_item_add_subtree(avp_item,dictionary_entry->ett);
+        
+        if (show_length) {
+            avp_len_item = proto_tree_add_uint(avp_tree,
+                                               dictionary_entry->hf_len,
+                                               tvb,0,0,avp_length);
+            PROTO_ITEM_SET_GENERATED(avp_len_item);
+        }
+        
+        tvb_len = tvb_length_remaining(tvb, offset);
+        
+        if ((gint)avp_length < tvb_len)
+            tvb_len = avp_length;
+        
+        if (avp_type == RADIUS_EAP_MESSAGE_CODE) {
+            eap_seg_num++;
 
-				if (eap_buffer == NULL)
-					eap_buffer =
-					    g_malloc(eap_tot_len_captured + tvb_len);
-				else
-					eap_buffer = g_realloc(eap_buffer,
-					    eap_tot_len_captured + tvb_len);
-				tvb_memcpy(tvb, eap_buffer + eap_tot_len_captured,
-				    offset, tvb_len);
-				eap_tot_len_captured += tvb_len;
-				eap_tot_len += avp_length;
+            /* Show this as an EAP fragment. */
+            if (tree)
+                proto_tree_add_text(avp_tree, tvb, offset, tvb_len,
+                                    "EAP fragment");
 
-				if ( tvb_bytes_exist(tvb, offset + avp_length + 1, 1) ) {
-					guint8 next_type = tvb_get_guint8(tvb, offset + avp_length);
-				
-					if ( next_type != RADIUS_EAP_MESSAGE_CODE ) {
-						/* Non-EAP-Message attribute */
-						last_eap = TRUE;
-					}
-				} else {
-					/*
-					 * No more attributes, either because
-					 * we're at the end of the packet or
-					 * because we're at the end of the
-					 * captured packet data.
-					 */
-					last_eap = TRUE;
-				}
+            if (eap_tvb != NULL) {
+                /*
+                 * Oops, a non-consecutive EAP-Message
+                 * attribute.
+                 */
+                proto_item_append_text(avp_item, " (non-consecutive)");
+            } else {
+                /*
+                 * RFC 2869 says, in section 5.13, describing
+                 * the EAP-Message attribute:
+                 *
+                 *    The NAS places EAP messages received
+                 *    from the authenticating peer into one
+                 *    or more EAP-Message attributes and
+                 *    forwards them to the RADIUS Server
+                 *    within an Access-Request message.
+                 *    If multiple EAP-Messages are
+                 *    contained within an Access-Request or
+                 *    Access-Challenge packet, they MUST be
+                 *    in order and they MUST be consecutive
+                 *    attributes in the Access-Request or
+                 *    Access-Challenge packet.
+                 *
+                 *        ...
+                 *
+                 *    The String field contains EAP packets,
+                 *    as defined in [3].  If multiple
+                 *    EAP-Message attributes are present
+                 *    in a packet their values should be
+                 *    concatenated; this allows EAP packets
+                 *    longer than 253 octets to be passed
+                 *    by RADIUS.
+                 *
+                 * Do reassembly of EAP-Message attributes.
+                 * We just concatenate all the attributes,
+                 * and when we see either the end of the
+                 * attribute list or a non-EAP-Message
+                 * attribute, we know we're done.
+                 */
 
-				if (last_eap && eap_buffer) {
-					gboolean save_writable;
+                if (eap_buffer == NULL)
+                    eap_buffer = g_malloc(eap_tot_len_captured + tvb_len);
+                else
+                    eap_buffer = g_realloc(eap_buffer,
+                                           eap_tot_len_captured + tvb_len);
+                tvb_memcpy(tvb, eap_buffer + eap_tot_len_captured, offset,
+                           tvb_len);
+                eap_tot_len_captured += tvb_len;
+                eap_tot_len += avp_length;
 
-					proto_item_append_text(avp_item,
-					    " Last Segment[%u]", eap_seg_num);
+                if ( tvb_bytes_exist(tvb, offset + avp_length + 1, 1) ) {
+                    guint8 next_type = tvb_get_guint8(tvb, offset + avp_length);
+                
+                    if ( next_type != RADIUS_EAP_MESSAGE_CODE ) {
+                        /* Non-EAP-Message attribute */
+                        last_eap = TRUE;
+                    }
+                } else {
+                    /*
+                     * No more attributes, either because
+                     * we're at the end of the packet or
+                     * because we're at the end of the
+                     * captured packet data.
+                     */
+                    last_eap = TRUE;
+                }
 
-					eap_tree = proto_item_add_subtree(avp_item,ett_eap);
-				
-					eap_tvb = tvb_new_real_data(eap_buffer,
-					    eap_tot_len_captured, eap_tot_len);
-					tvb_set_free_cb(eap_tvb, g_free);
-					tvb_set_child_real_data_tvbuff(tvb,
-					    eap_tvb);
-					add_new_data_source(pinfo, eap_tvb,
-					    "Reassembled EAP");
+                if (last_eap && eap_buffer) {
+                    gboolean save_writable;
 
-					/*
-					 * Don't free this when we're done -
-					 * it's associated with a tvbuff.
-					 */
-					eap_buffer = NULL;
-	
-					/*
-					 * Set the columns non-writable,
-					 * so that the packet list shows
-					 * this as an RADIUS packet, not
-					 * as an EAP packet.
-					 */
-					save_writable =
-					    col_get_writable(pinfo->cinfo);
-					col_set_writable(pinfo->cinfo, FALSE);
+                    proto_item_append_text(avp_item, " Last Segment[%u]",
+                                           eap_seg_num);
 
-					call_dissector(eap_handle, eap_tvb,
-					    pinfo, eap_tree);
+                    eap_tree = proto_item_add_subtree(avp_item,ett_eap);
+                
+                    eap_tvb = tvb_new_real_data(eap_buffer,
+                                                eap_tot_len_captured,
+                                                eap_tot_len);
+                    tvb_set_free_cb(eap_tvb, g_free);
+                    tvb_set_child_real_data_tvbuff(tvb, eap_tvb);
+                    add_new_data_source(pinfo, eap_tvb, "Reassembled EAP");
 
-					col_set_writable(pinfo->cinfo,
-					    save_writable);
-				} else {
-					proto_item_append_text(avp_item,
-					    " Segment[%u]", eap_seg_num);
-				}
-			}
-		} else {
-			if (dictionary_entry->tagged) {
-				guint tag;
+                    /*
+                     * Don't free this when we're done -
+                     * it's associated with a tvbuff.
+                     */
+                    eap_buffer = NULL;
+    
+                    /*
+                     * Set the columns non-writable,
+                     * so that the packet list shows
+                     * this as an RADIUS packet, not
+                     * as an EAP packet.
+                     */
+                    save_writable = col_get_writable(pinfo->cinfo);
+                    col_set_writable(pinfo->cinfo, FALSE);
 
-				if (avp_length < 3) {
-					proto_tree_add_text(tree, tvb, offset,
-					    0, "AVP too short for tag");
-					offset += avp_length;
-					continue;
-				}
-				tag = tvb_get_guint8(tvb, offset);
-				if (tag <=  0x1f) {
-					proto_tree_add_uint(avp_tree,
-					    dictionary_entry->hf_tag,
-					    tvb, offset, 1, tag);
-				
-					proto_item_append_text(avp_item,
-					    " Tag=0x%.2x", tag);
-		
-					offset++;
-					avp_length--;
-				}
-			}
+                    call_dissector(eap_handle, eap_tvb, pinfo, eap_tree);
 
-			if ( dictionary_entry->dissector ) {
-				tvbuff_t* tvb_value;
-				gchar* str;
-			
-				tvb_value = tvb_new_subset(tvb, offset, tvb_len, (gint) avp_length);
-			
-				str = dictionary_entry->dissector(avp_tree,tvb_value);
-			
-				proto_item_append_text(avp_item, ": %s",str);
-			} else {
-				proto_item_append_text(avp_item, ": ");
-			
-				dictionary_entry->type(dictionary_entry,avp_tree,pinfo,tvb,offset,avp_length,avp_item);
-			}
-		}
-		
-		offset += avp_length;
-	}
+                    col_set_writable(pinfo->cinfo, save_writable);
+                } else {
+                    proto_item_append_text(avp_item, " Segment[%u]",
+                                           eap_seg_num);
+                }
+            }
+            
+            offset += avp_length;
+        } else {
+            add_avp_to_tree(avp_tree, avp_item, pinfo, tvb, dictionary_entry,
+                            avp_length, offset);
+            offset += avp_length;
+        }
+        
+    }
 
-	/*
-	 * Call the cleanup handler to free any reassembled data we haven't
-	 * attached to a tvbuff, and pop the handler.
-	 */
-	CLEANUP_CALL_AND_POP;
+    /*
+     * Call the cleanup handler to free any reassembled data we haven't
+     * attached to a tvbuff, and pop the handler.
+     */
+    CLEANUP_CALL_AND_POP;
 }
 
 static void dissect_radius(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
@@ -700,9 +854,9 @@ static void dissect_radius(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 		
 		proto_tree_add_item(radius_tree, hf_radius_authenticator, tvb, 4,AUTHENTICATOR_LENGTH,FALSE);
 	}
-	tvb_memcpy(tvb,authenticator,0,AUTHENTICATOR_LENGTH);
+	tvb_memcpy(tvb,authenticator,4,AUTHENTICATOR_LENGTH);
 
-	if (avplength > 0) {
+	if (tree && avplength > 0) {
 		/* list the attribute value pairs */
 		avptf = proto_tree_add_text(radius_tree, tvb, HDR_LENGTH,
 					    avplength, "Attribute Value Pairs");
@@ -730,6 +884,7 @@ static void register_attrs(gpointer k _U_, gpointer v, gpointer p) {
 	
 	for(i=0; abbrev[i]; i++) {
 		if(abbrev[i] == '-') abbrev[i] = '_';
+        if(abbrev[i] == '/') abbrev[i] = '_';
 	}
 	
 	hfri[0].p_id = &(a->hf);
@@ -766,10 +921,13 @@ static void register_attrs(gpointer k _U_, gpointer v, gpointer p) {
 		hfri[0].hfinfo.display = BASE_NONE;
 	} else if (a->type == radius_ipaddr) {
 		hfri[0].hfinfo.type = FT_IPv4;
-		hfri[0].hfinfo.display = BASE_DEC;
+		hfri[0].hfinfo.display = BASE_NONE;
 	} else if (a->type == radius_ipv6addr) {
 		hfri[0].hfinfo.type = FT_IPv6;
-		hfri[0].hfinfo.display = BASE_HEX;
+		hfri[0].hfinfo.display = BASE_NONE;
+	} else if (a->type == radius_ipxnet) {
+		hfri[0].hfinfo.type = FT_IPXNET;
+		hfri[0].hfinfo.display = BASE_NONE;
 	} else if (a->type == radius_date) {
 		hfri[0].hfinfo.type = FT_ABSOLUTE_TIME;
 		hfri[0].hfinfo.display = BASE_NONE;
@@ -803,13 +961,16 @@ static void register_vendors(gpointer k _U_, gpointer v, gpointer p) {
 	radius_vendor_info_t* vnd = v;
 	hfett_t* ri = p;
 	value_string vnd_vs;
-	
+	gint* ett_p = &(vnd->ett);
+    
 	vnd_vs.value = vnd->code;
 	vnd_vs.strptr = vnd->name;
 	
 	g_array_append_val(ri->vend_vs,vnd_vs);
+	g_array_append_val(ri->ett,ett_p);
 
 	g_hash_table_foreach(vnd->attrs_by_id,register_attrs,ri);
+    
 }
 
 extern void radius_register_avp_dissector(guint32 vendor_id, guint32 attribute_id, radius_avp_dissector_t radius_avp_dissector) {
@@ -825,10 +986,11 @@ extern void radius_register_avp_dissector(guint32 vendor_id, guint32 attribute_i
 		if ( ! vendor ) {
 			vendor = g_malloc(sizeof(radius_vendor_info_t));
 			
-			vendor->name = g_strdup_printf("Unknown-Vendor-%u",vendor_id);
+			vendor->name = g_strdup_printf("%s-%u",val_to_str(vendor_id, sminmpec_values, "Unknown"),vendor_id);
 			vendor->code = vendor_id;
 			vendor->attrs_by_id = g_hash_table_new(g_direct_hash,g_direct_equal);
-			
+			vendor->ett = no_vendor.ett;
+            
 			g_hash_table_insert(dict->vendors_by_id,GUINT_TO_POINTER(vendor->code),vendor);
 			g_hash_table_insert(dict->vendors_by_name,vendor->name,vendor);
 		}
@@ -859,6 +1021,19 @@ extern void radius_register_avp_dissector(guint32 vendor_id, guint32 attribute_i
 
 }
 
+static void reinit_radius(void) {
+	if ( alt_port_pref != alt_port ) {
+		
+		if (alt_port)
+			dissector_delete("udp.port", alt_port, radius_handle);
+		
+		if (alt_port_pref)
+			dissector_add("udp.port", alt_port_pref, radius_handle);
+		
+		alt_port = alt_port_pref;
+	}	
+}
+
 void
 proto_register_radius(void)
 {
@@ -887,6 +1062,18 @@ proto_register_radius(void)
 	{ "Unknown-Attribute Length","radius.Unknown_Attribute.length", FT_UINT8, BASE_DEC, NULL, 0x0,
 		"", HFILL }},
 		
+	{ &hf_radius_framed_ip_address,
+	{ "Framed-IP-Address","radius.Framed-IP-Address", FT_IPv4, BASE_NONE, NULL, 0x0,
+		"", HFILL }},
+
+	{ &hf_radius_login_ip_host,
+	{ "Login-IP-Host","radius.Login-IP-Host", FT_IPv4, BASE_NONE, NULL, 0x0,
+		"", HFILL }},
+
+	{ &hf_radius_framed_ipx_network,
+	{ "Framed-IPX-Network","radius.Framed-IPX-Network", FT_IPXNET, BASE_NONE, NULL, 0x0,
+		"", HFILL }},
+
 	{ &hf_radius_cosine_vpi,
 	{ "Cosine-VPI","radius.Cosine-Vpi", FT_UINT16, BASE_DEC, NULL, 0x0,
 		"", HFILL }},
@@ -903,6 +1090,7 @@ proto_register_radius(void)
 		&ett_radius_avp,
 		&ett_eap,
 		&(no_dictionary_entry.ett),
+        &(no_vendor.ett),
 	};
 	
 	module_t *radius_module;
@@ -920,6 +1108,8 @@ proto_register_radius(void)
 	dir = get_persconffile_path("radius", FALSE);
 	
 	if (test_for_directory(dir) != EISDIR) {
+		/* Although dir isn't a directory it may still use memory */
+		g_free(dir);
 		
 		dir = get_datafile_path("radius");
 		
@@ -935,6 +1125,8 @@ proto_register_radius(void)
 		dict_err_str = g_strdup("Could not find the radius directory");
 	}
 	
+	g_free(dir);
+    
 	if (dict_err_str) {
 		g_warning("radius: %s",dict_err_str);
 		g_free(dict_err_str);
@@ -964,16 +1156,23 @@ proto_register_radius(void)
 	g_array_free(ri.ett,FALSE);
 	g_array_free(ri.vend_vs,FALSE);
 		
-	radius_module = prefs_register_protocol(proto_radius,NULL);
+	radius_module = prefs_register_protocol(proto_radius,reinit_radius);
 	prefs_register_string_preference(radius_module,"shared_secret","Shared Secret",
-					"Shared secret used to decode User Passwords",
-					&shared_secret);
+                                     "Shared secret used to decode User Passwords",
+                                     &shared_secret);
+	prefs_register_bool_preference(radius_module,"show_length","Show AVP Lengths",
+                                     "Whether to add or not to the tree the AVP's payload length",
+                                     &show_length);
+    prefs_register_uint_preference(radius_module, "alternate_port","Alternate Port",
+                                   "An alternate UDP port to decode as RADIUS", 10, &alt_port_pref);
+
+    no_vendor.attrs_by_id = g_hash_table_new(g_direct_hash,g_direct_equal);
+    
 }
 
 void
 proto_reg_handoff_radius(void)
 {
-	dissector_handle_t radius_handle;
 	
 	eap_handle = find_dissector("eap");
 	
@@ -984,6 +1183,9 @@ proto_reg_handoff_radius(void)
 	dissector_add("udp.port", UDP_PORT_RADACCT, radius_handle);
 	dissector_add("udp.port", UDP_PORT_RADACCT_NEW, radius_handle);
 	
+	radius_register_avp_dissector(0,8,dissect_framed_ip_address);
+	radius_register_avp_dissector(0,14,dissect_login_ip_host);
+	radius_register_avp_dissector(0,23,dissect_framed_ipx_network);
 	radius_register_avp_dissector(VENDOR_COSINE,5,dissect_cosine_vpvc);
 	
 }
