@@ -1,6 +1,6 @@
 /* packet-rtcp.h
  *
- * $Id: packet-rtcp.h 12803 2004-12-21 19:28:48Z guy $
+ * $Id: packet-rtcp.h 17834 2006-04-06 13:30:45Z jake $
  *
  * Routines for RTCP dissection
  * RTCP = Real-time Transport Control Protocol
@@ -39,14 +39,13 @@ struct _rtcp_conversation_info
     guint32 setup_frame_number;
 
     /* Info used for roundtrip calculations */
-    guchar  last_received_set;
-    guint32 last_received_frame_number;
-    guint32 last_received_time_secs;
-    guint32 last_received_time_usecs;
-    guint32 last_received_ts;
+    guchar   last_received_set;
+    guint32  last_received_frame_number;
+    nstime_t last_received_timestamp;
+    guint32  last_received_ts;
 
-    /* Stored result of calculation (ms) */
-    guchar  calculated_delay_set;
+    /* Stored result of calculation */
+    guchar  lsr_matched;
     guint32 calculated_delay_used_frame;
     guint32 calculated_delay;
 };
@@ -56,4 +55,4 @@ struct _rtcp_conversation_info
 void rtcp_add_address(packet_info *pinfo,
                       address *addr, int port,
                       int other_port,
-                      gchar *setup_method, guint32 setup_frame_number);
+                      const gchar *setup_method, guint32 setup_frame_number);
