@@ -2,7 +2,7 @@
  * Routines for SMB Browser packet dissection
  * Copyright 1999, Richard Sharpe <rsharpe@ns.aus.com>
  *
- * $Id: packet-smb-browse.c 14450 2005-05-27 02:30:10Z guy $
+ * $Id: packet-smb-browse.c 15845 2005-09-17 00:31:15Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -36,7 +36,7 @@
 #include <glib.h>
 #include <ctype.h>
 #include <epan/packet.h>
-#include "smb.h"
+#include <epan/dissectors/packet-smb.h>
 
 #include "packet-smb-browse.h"
 #include "packet-dcerpc.h"
@@ -674,13 +674,12 @@ dissect_mailslot_browse(tvbuff_t *tvb, packet_info *pinfo, proto_tree *parent_tr
 		offset += 1;
 
 		/* name of computer to which to send reply */
-		computer_name = tvb_get_stringz(tvb, offset, &namelen);
+		computer_name = tvb_get_ephemeral_stringz(tvb, offset, &namelen);
 		proto_tree_add_string(tree, hf_response_computer_name,
 			tvb, offset, namelen, computer_name);
 		if (check_col(pinfo->cinfo, COL_INFO))
 			col_append_fstr(
 				pinfo->cinfo, COL_INFO, " %s", computer_name);
-		g_free(computer_name);
 		offset += namelen;
 		break;
 	}
