@@ -8,7 +8,7 @@
  * Dissection of multiple SMPP PDUs within one packet
  * provided by Chris Wilson.
  *
- * $Id: packet-smpp.c 12115 2004-09-27 22:55:15Z guy $
+ * $Id: packet-smpp.c 17739 2006-03-27 18:48:33Z etxrab $
  *
  * Refer to the AUTHORS file or the AUTHORS section in the man page
  * for contacting the author(s) of this file.
@@ -51,6 +51,7 @@
 #include <epan/packet.h>
 
 #include <epan/prefs.h>
+#include <epan/emem.h>
 #include "packet-tcp.h"
 
 /* General-purpose debug logger.
@@ -61,9 +62,8 @@
  */
 #ifdef DEBUG_smpp
 #define DebugLog(x) \
-	printf("%s:%u: ", __FILE__, __LINE__); \
-	printf x; \
-	fflush(stdout)
+	g_print("%s:%u: ", __FILE__, __LINE__); \
+	g_print x
 #else
 #define DebugLog(x) ;
 #endif
@@ -211,7 +211,7 @@ static gint ett_opt_param	= -1;
 static gint ett_dcs		= -1;
 
 /* Reassemble SMPP TCP segments */
-static gboolean reassemble_over_tcp = FALSE;
+static gboolean reassemble_over_tcp = TRUE;
 
 /*
  * Value-arrays for field-contents
@@ -692,8 +692,8 @@ static const value_string vals_dcs_cbs_language[] = {
 	{ 0x0D, "Hungarian" },
 	{ 0x0E, "Polish" },
 	{ 0x0F, "Language not specified" },
-	{ 0x10, "GSM 7-bit default alphabet - message preceeded by language indication" },
-	{ 0x11, "UCS-2 (16-bit) - message preceeded by language indication" },
+	{ 0x10, "GSM 7-bit default alphabet - message preceded by language indication" },
+	{ 0x11, "UCS-2 (16-bit) - message preceded by language indication" },
 	{ 0x20, "Czech" },
 	{ 0x21, "Hebrew" },
 	{ 0x22, "Arabic" },
@@ -803,11 +803,10 @@ smpp_handle_string_return(proto_tree *tree, tvbuff_t *tvb, int field, int *offse
 
     len = tvb_strsize(tvb, *offset);
     if (len > 1) {
-	str = (char *)tvb_get_stringz(tvb, *offset, &len);
+	str = (char *)tvb_get_ephemeral_stringz(tvb, *offset, &len);
 	proto_tree_add_string(tree, field, tvb, *offset, len, str);
     } else {
-	str = g_malloc(1 * sizeof(char));
-	str[0] = '\0';
+	str = "";
     }
     (*offset) += len;
     return str;
@@ -867,7 +866,7 @@ smpp_handle_time(proto_tree *tree, tvbuff_t *tvb,
     gint	 len;
     nstime_t	 tmptime;
 
-    strval = (char *) tvb_get_stringz(tvb, *offset, &len);
+    strval = (char *) tvb_get_ephemeral_stringz(tvb, *offset, &len);
     if (*strval)
     {
 	if (smpp_mktime(strval, &tmptime.secs, &tmptime.nsecs))
@@ -875,7 +874,6 @@ smpp_handle_time(proto_tree *tree, tvbuff_t *tvb,
 	else
 	    proto_tree_add_time(tree, field, tvb, *offset, len, &tmptime);
     }
-    g_free(strval);
     *offset += len;
 }
 
@@ -968,7 +966,7 @@ smpp_handle_tlv(proto_tree *tree, tvbuff_t *tvb, int *offset)
     guint	 length;
     guint8	 field;
     guint8	 major, minor;
-    char	 strval[BUFSIZ];
+    char	 *strval=NULL;
 
     if (tvb_reported_length_remaining(tvb, *offset) >= 4)
     {
@@ -1088,7 +1086,8 @@ smpp_handle_tlv(proto_tree *tree, tvbuff_t *tvb, int *offset)
 		field = tvb_get_guint8(tvb, *offset);
 		minor = field & 0x0F;
 		major = (field & 0xF0) >> 4;
-		sprintf(strval, "%u.%u", major, minor);
+                strval=ep_alloc(BUFSIZ);
+		g_snprintf(strval, BUFSIZ, "%u.%u", major, minor);
 		proto_tree_add_string(sub_tree, hf_smpp_SC_interface_version,
 			    	      tvb, *offset, 1, strval);
 		(*offset)++;
@@ -1204,7 +1203,7 @@ smpp_handle_tlv(proto_tree *tree, tvbuff_t *tvb, int *offset)
     }
 }
 
-static void
+void
 smpp_handle_dcs(proto_tree *tree, tvbuff_t *tvb, int *offset)
 {
     guint8	 val;
@@ -1294,15 +1293,16 @@ bind_receiver(proto_tree *tree, tvbuff_t *tvb)
     int		 offset = 0;
     guint8	 field;
     guint8	 major, minor;
-    char	 strval[BUFSIZ];
+    char	 *strval;
 
+    strval=ep_alloc(BUFSIZ);
     smpp_handle_string(tree, tvb, hf_smpp_system_id, &offset);
     smpp_handle_string(tree, tvb, hf_smpp_password, &offset);
     smpp_handle_string(tree, tvb, hf_smpp_system_type, &offset);
     field = tvb_get_guint8(tvb, offset++);
     minor = field & 0x0F;
     major = (field & 0xF0) >> 4;
-    sprintf(strval, "%u.%u", major, minor);
+    g_snprintf(strval, BUFSIZ, "%u.%u", major, minor);
     proto_tree_add_string(tree, hf_smpp_interface_version, tvb,
 			  offset - 1, 1, strval);
     smpp_handle_int1(tree, tvb, hf_smpp_addr_ton, &offset);
@@ -1342,9 +1342,9 @@ submit_sm(proto_tree *tree, tvbuff_t *tvb, packet_info *pinfo,
     int		 offset = 0;
     guint8	 flag, udhi;
     guint8	 length;
-	char *src_str = NULL;
-	char *dst_str = NULL;
-	address save_src, save_dst;
+    char *src_str = NULL;
+    char *dst_str = NULL;
+    address save_src, save_dst;
 
     smpp_handle_string_z(tree, tvb, hf_smpp_service_type, &offset, "(Default)");
     smpp_handle_int1(tree, tvb, hf_smpp_source_addr_ton, &offset);
@@ -1364,20 +1364,20 @@ submit_sm(proto_tree *tree, tvbuff_t *tvb, packet_info *pinfo,
     offset++;
     smpp_handle_int1(tree, tvb, hf_smpp_protocol_id, &offset);
     smpp_handle_int1(tree, tvb, hf_smpp_priority_flag, &offset);
-	if (tvb_get_guint8(tvb,offset)) {
-    smpp_handle_time(tree, tvb, hf_smpp_schedule_delivery_time,
-				hf_smpp_schedule_delivery_time_r, &offset);
-	} else { /* Time = NULL means Immediate delivery */
-		proto_tree_add_text(tree, tvb, offset++, 1,
-				"Scheduled delivery time: Immediate delivery");
-	}
-	if (tvb_get_guint8(tvb,offset)) {
-    smpp_handle_time(tree, tvb, hf_smpp_validity_period,
-				hf_smpp_validity_period_r, &offset);
-	} else { /* Time = NULL means SMSC default validity */
-		proto_tree_add_text(tree, tvb, offset++, 1,
-				"Validity period: SMSC default validity period");
-	}
+    if (tvb_get_guint8(tvb,offset)) {
+	smpp_handle_time(tree, tvb, hf_smpp_schedule_delivery_time,
+			 hf_smpp_schedule_delivery_time_r, &offset);
+    } else { /* Time = NULL means Immediate delivery */
+	proto_tree_add_text(tree, tvb, offset++, 1,
+			    "Scheduled delivery time: Immediate delivery");
+    }
+    if (tvb_get_guint8(tvb,offset)) {
+	smpp_handle_time(tree, tvb, hf_smpp_validity_period,
+			 hf_smpp_validity_period_r, &offset);
+    } else { /* Time = NULL means SMSC default validity */
+	proto_tree_add_text(tree, tvb, offset++, 1,
+			    "Validity period: SMSC default validity period");
+    }
     flag = tvb_get_guint8(tvb, offset);
     proto_tree_add_item(tree, hf_smpp_regdel_receipt, tvb, offset, 1, flag);
     proto_tree_add_item(tree, hf_smpp_regdel_acks, tvb, offset, 1, flag);
@@ -1407,12 +1407,10 @@ submit_sm(proto_tree *tree, tvbuff_t *tvb, packet_info *pinfo,
 	    /* Restore original addresses */
 	    COPY_ADDRESS(&(pinfo->src), &save_src);
 	    COPY_ADDRESS(&(pinfo->dst), &save_dst);
-	    /* Get rid of SMPP text string addresses */
-	    g_free(src_str);
-	    g_free(dst_str);
 	}
     	offset += length;
     }
+    /* Get rid of SMPP text string addresses */
     smpp_handle_tlv(tree, tvb, &offset);
 }
 
@@ -1692,6 +1690,9 @@ dissect_smpp(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
 	    gint pdu_real_len = tvb_length_remaining(tvb, offset);
     	    tvbuff_t *pdu_tvb;
 
+            if (pdu_len < 1)
+                THROW(ReportedBoundsError);
+
 	    if (pdu_real_len <= 0)
 		return;
 	    if (pdu_real_len > pdu_len)
@@ -1714,8 +1715,8 @@ dissect_smpp_pdu(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
     guint	 command_id;		/* SMPP command		*/
     guint	 command_status;	/* Status code		*/
     guint	 sequence_number;	/* ...of command	*/
-    gchar	*command_str;
-    gchar	*command_status_str = NULL;
+    const gchar	*command_str;
+    const gchar	*command_status_str = NULL;
     /* Set up structures needed to add the protocol subtree and manage it */
     proto_item	*ti = NULL;
     proto_tree	*smpp_tree = NULL;
@@ -2226,9 +2227,9 @@ proto_register_smpp(void)
 	    }
 	},
 	{   &hf_smpp_dlist_resp,
-	    {   "Unsuccesfull delivery list", "smpp.dlist_resp",
+	    {   "Unsuccessful delivery list", "smpp.dlist_resp",
 		FT_NONE, BASE_NONE, NULL, 0x00,
-		"The list of unsuccesfull deliveries to destinations.",
+		"The list of unsuccessful deliveries to destinations.",
 		HFILL
 	    }
 	},
@@ -2386,7 +2387,21 @@ proto_register_smpp(void)
 		HFILL
 	    }
 	},
-	{   &hf_smpp_user_message_reference,
+    {   &hf_smpp_source_subaddress,
+	    {   "Source Subaddress", "smpp.source_subaddress",
+		FT_STRING, BASE_NONE, NULL, 0x00,
+		"Source Subaddress",
+		HFILL
+	    }
+	},
+    {   &hf_smpp_dest_subaddress,
+	    {   "Destination Subaddress", "smpp.dest_subaddress",
+		FT_STRING, BASE_NONE, NULL, 0x00,
+		"Destination Subaddress",
+		HFILL
+	    }
+	},
+    {   &hf_smpp_user_message_reference,
 	    {   "Message reference", "smpp.user_message_reference",
 		FT_UINT16, BASE_HEX, NULL, 0x00,
 		"Reference to the message, assigned by the user.",
@@ -2762,5 +2777,5 @@ proto_reg_handoff_smpp(void)
 	/* Required for call_dissector() */
 	DebugLog(("Finding gsm-sms-ud subdissector\n"));
 	gsm_sms_handle = find_dissector("gsm-sms-ud");
-	g_assert (gsm_sms_handle);
+	DISSECTOR_ASSERT(gsm_sms_handle);
 }
