@@ -2,7 +2,7 @@
  * Routines for X.25 packet disassembly
  * Olivier Abad <oabad@noos.fr>
  *
- * $Id: packet-x25.c 12699 2004-12-09 19:06:59Z gerald $
+ * $Id: packet-x25.c 16956 2006-01-05 13:01:08Z jmayer $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -31,13 +31,14 @@
 #include <glib.h>
 #include <stdlib.h>
 #include <string.h>
-#include "llcsaps.h"
+#include <epan/llcsaps.h>
 #include <epan/packet.h>
 #include <epan/circuit.h>
-#include "reassemble.h"
+#include <epan/reassemble.h>
 #include <epan/prefs.h>
-#include "nlpid.h"
-#include "x264_prt_id.h"
+#include <epan/emem.h>
+#include <epan/nlpid.h>
+#include <epan/x264_prt_id.h>
 
 /*
  * Direction of packet.
@@ -215,7 +216,7 @@ static dissector_handle_t data_handle;
 
 /* Preferences */
 static gboolean payload_is_qllc_sna = FALSE;
-static gboolean reassemble_x25 = FALSE;
+static gboolean reassemble_x25 = TRUE;
 
 /* Reassembly of X.25 */
 
@@ -269,9 +270,9 @@ x25_hash_add_proto_end(guint16 vc, guint32 frame)
     close_circuit(circuit, frame);
 }
 
-static char *clear_code(unsigned char code)
+static const char *clear_code(unsigned char code)
 {
-    static char buffer[25];
+    static char *buffer;
 
     if (code == 0x00 || (code & 0x80) == 0x80)
 	return "DTE Originated";
@@ -302,14 +303,15 @@ static char *clear_code(unsigned char code)
     if (code == 0x39)
 	return "Destination Absent";
 
-    sprintf(buffer, "Unknown %02X", code);
+    buffer=ep_alloc(32);
+    g_snprintf(buffer, 32, "Unknown %02X", code);
 
     return buffer;
 }
 
-static char *clear_diag(unsigned char code)
+static const char *clear_diag(unsigned char code)
 {
-    static char buffer[25];
+    static char *buffer;
 
     if (code == 0)
 	return "No additional information";
@@ -529,14 +531,15 @@ static char *clear_diag(unsigned char code)
     if (code == 250)
 	return "Reset - user resynchronization";
 
-    sprintf(buffer, "Unknown %d", code);
+    buffer=ep_alloc(32);
+    g_snprintf(buffer, 32, "Unknown %d", code);
 
     return buffer;
 }
 
-static char *reset_code(unsigned char code)
+static const char *reset_code(unsigned char code)
 {
-    static char buffer[25];
+    static char *buffer;
 
     if (code == 0x00 || (code & 0x80) == 0x80)
 	return "DTE Originated";
@@ -557,14 +560,15 @@ static char *reset_code(unsigned char code)
     if (code == 0x1D)
 	return "Network out of order";
 
-    sprintf(buffer, "Unknown %02X", code);
+    buffer=ep_alloc(32);
+    g_snprintf(buffer, 32, "Unknown %02X", code);
 
     return buffer;
 }
 
-static char *restart_code(unsigned char code)
+static const char *restart_code(unsigned char code)
 {
-    static char buffer[25];
+    static char *buffer;
 
     if (code == 0x00 || (code & 0x80) == 0x80)
 	return "DTE Originated";
@@ -577,14 +581,15 @@ static char *restart_code(unsigned char code)
     if (code == 0x7F)
 	return "Registration/cancellation confirmed";
 
-    sprintf(buffer, "Unknown %02X", code);
+    buffer=ep_alloc(32);
+    g_snprintf(buffer, 32, "Unknown %02X", code);
 
     return buffer;
 }
 
-static char *registration_code(unsigned char code)
+static const char *registration_code(unsigned char code)
 {
-    static char buffer[25];
+    static char *buffer;
 
     if (code == 0x03)
 	return "Invalid facility request";
@@ -595,7 +600,8 @@ static char *registration_code(unsigned char code)
     if (code == 0x7F)
 	return "Registration/cancellation confirmed";
 
-    sprintf(buffer, "Unknown %02X", code);
+    buffer=ep_alloc(32);
+    g_snprintf(buffer, 32, "Unknown %02X", code);
 
     return buffer;
 }
@@ -688,10 +694,11 @@ dump_facilities(proto_tree *tree, int *offset, tvbuff_t *tvb)
 		break;
 	    case X25_FAC_THROUGHPUT:
 		if (fac_tree) {
-		    char tmpbuf[80];
+		    char *tmpbuf;
 
+		    tmpbuf=ep_alloc(80);
 		    ti = proto_tree_add_text(fac_tree, tvb, *offset, 1, "Code : %02X "
-			    "(Throughput class negociation)", fac);
+			    "(Throughput class negotiation)", fac);
 		    fac_subtree = proto_item_add_subtree(ti, ett_x25_fac_throughput);
 		    byte1 = tvb_get_guint8(tvb, *offset + 1);
 		    switch (byte1 >> 4)
@@ -705,17 +712,17 @@ dump_facilities(proto_tree *tree, int *offset, tvbuff_t *tvb)
 		    case 9:
 		    case 10:
 		    case 11:
-			sprintf(tmpbuf, "From the called DTE : %%u (%d bps)",
+			g_snprintf(tmpbuf, 80, "From the called DTE : %%u (%d bps)",
 				75*(1<<((byte1 >> 4)-3)));
 			break;
 		    case 12:
-			sprintf(tmpbuf, "From the called DTE : %%u (48000 bps)");
+			g_snprintf(tmpbuf, 80, "From the called DTE : %%u (48000 bps)");
 			break;
 		    case 13:
-			sprintf(tmpbuf, "From the called DTE : %%u (64000 bps)");
+			g_snprintf(tmpbuf, 80, "From the called DTE : %%u (64000 bps)");
 			break;
 		    default:
-			sprintf(tmpbuf, "From the called DTE : %%u (Reserved)");
+			g_snprintf(tmpbuf, 80, "From the called DTE : %%u (Reserved)");
 		    }
 		    proto_tree_add_text(fac_subtree, tvb, *offset+1, 1,
 			    decode_numeric_bitfield(byte1, 0xF0, 1*8, tmpbuf));
@@ -730,17 +737,17 @@ dump_facilities(proto_tree *tree, int *offset, tvbuff_t *tvb)
 		    case 9:
 		    case 10:
 		    case 11:
-			sprintf(tmpbuf, "From the calling DTE : %%u (%d bps)",
+			g_snprintf(tmpbuf, 80, "From the calling DTE : %%u (%d bps)",
 				75*(1<<((byte1 & 0x0F)-3)));
 			break;
 		    case 12:
-			sprintf(tmpbuf, "From the calling DTE : %%u (48000 bps)");
+			g_snprintf(tmpbuf, 80, "From the calling DTE : %%u (48000 bps)");
 			break;
 		    case 13:
-			sprintf(tmpbuf, "From the calling DTE : %%u (64000 bps)");
+			g_snprintf(tmpbuf, 80, "From the calling DTE : %%u (64000 bps)");
 			break;
 		    default:
-			sprintf(tmpbuf, "From the calling DTE : %%u (Reserved)");
+			g_snprintf(tmpbuf, 80, "From the calling DTE : %%u (Reserved)");
 		    }
 		    proto_tree_add_text(fac_subtree, tvb, *offset+1, 1,
 			    decode_numeric_bitfield(byte1, 0x0F, 1*8, tmpbuf));
@@ -789,7 +796,7 @@ dump_facilities(proto_tree *tree, int *offset, tvbuff_t *tvb)
 	    case X25_FAC_EXPRESS_DATA:
 		if (fac_tree) {
 		    ti = proto_tree_add_text(fac_tree, tvb, *offset, 1, "Code : %02X "
-			    "(Negociation of express data)", fac);
+			    "(Negotiation of express data)", fac);
 		    fac_subtree = proto_item_add_subtree(ti,
 			    ett_x25_fac_express_data);
 		    proto_tree_add_text(fac_subtree, tvb, *offset+1, 1,
@@ -825,8 +832,9 @@ dump_facilities(proto_tree *tree, int *offset, tvbuff_t *tvb)
 	    case X25_FAC_PACKET_SIZE:
 		if (fac_tree)
 		{
-		    char tmpbuf[80];
+		    char *tmpbuf;
 
+		    tmpbuf=ep_alloc(80);
 		    ti = proto_tree_add_text(fac_tree, tvb, *offset, 1, "Code : %02X "
 			    "(Packet size)", fac);
 		    fac_subtree = proto_item_add_subtree(ti, ett_x25_fac_packet_size);
@@ -834,34 +842,34 @@ dump_facilities(proto_tree *tree, int *offset, tvbuff_t *tvb)
 		    switch (byte1)
 		    {
 		    case 0x04:
-			sprintf(tmpbuf, "From the called DTE : %%u (16)");
+			g_snprintf(tmpbuf, 80, "From the called DTE : %%u (16)");
 			break;
 		    case 0x05:
-			sprintf(tmpbuf, "From the called DTE : %%u (32)");
+			g_snprintf(tmpbuf, 80, "From the called DTE : %%u (32)");
 			break;
 		    case 0x06:
-			sprintf(tmpbuf, "From the called DTE : %%u (64)");
+			g_snprintf(tmpbuf, 80, "From the called DTE : %%u (64)");
 			break;
 		    case 0x07:
-			sprintf(tmpbuf, "From the called DTE : %%u (128)");
+			g_snprintf(tmpbuf, 80, "From the called DTE : %%u (128)");
 			break;
 		    case 0x08:
-			sprintf(tmpbuf, "From the called DTE : %%u (256)");
+			g_snprintf(tmpbuf, 80, "From the called DTE : %%u (256)");
 			break;
 		    case 0x0D:
-			sprintf(tmpbuf, "From the called DTE : %%u (512)");
+			g_snprintf(tmpbuf, 80, "From the called DTE : %%u (512)");
 			break;
 		    case 0x0C:
-			sprintf(tmpbuf, "From the called DTE : %%u (1024)");
+			g_snprintf(tmpbuf, 80, "From the called DTE : %%u (1024)");
 			break;
 		    case 0x0E:
-			sprintf(tmpbuf, "From the called DTE : %%u (2048)");
+			g_snprintf(tmpbuf, 80, "From the called DTE : %%u (2048)");
 			break;
 		    case 0x0F:
-			sprintf(tmpbuf, "From the called DTE : %%u (4096)");
+			g_snprintf(tmpbuf, 80, "From the called DTE : %%u (4096)");
 			break;
 		    default:
-			sprintf(tmpbuf, "From the called DTE : %%u (Unknown)");
+			g_snprintf(tmpbuf, 80, "From the called DTE : %%u (Unknown)");
 			break;
 		    }
 		    proto_tree_add_text(fac_subtree, tvb, *offset+1, 1,
@@ -871,34 +879,34 @@ dump_facilities(proto_tree *tree, int *offset, tvbuff_t *tvb)
 		    switch (byte2)
 		    {
 		    case 0x04:
-			sprintf(tmpbuf, "From the calling DTE : %%u (16)");
+			g_snprintf(tmpbuf, 80, "From the calling DTE : %%u (16)");
 			break;
 		    case 0x05:
-			sprintf(tmpbuf, "From the calling DTE : %%u (32)");
+			g_snprintf(tmpbuf, 80, "From the calling DTE : %%u (32)");
 			break;
 		    case 0x06:
-			sprintf(tmpbuf, "From the calling DTE : %%u (64)");
+			g_snprintf(tmpbuf, 80, "From the calling DTE : %%u (64)");
 			break;
 		    case 0x07:
-			sprintf(tmpbuf, "From the calling DTE : %%u (128)");
+			g_snprintf(tmpbuf, 80, "From the calling DTE : %%u (128)");
 			break;
 		    case 0x08:
-			sprintf(tmpbuf, "From the calling DTE : %%u (256)");
+			g_snprintf(tmpbuf, 80, "From the calling DTE : %%u (256)");
 			break;
 		    case 0x0D:
-			sprintf(tmpbuf, "From the calling DTE : %%u (512)");
+			g_snprintf(tmpbuf, 80, "From the calling DTE : %%u (512)");
 			break;
 		    case 0x0C:
-			sprintf(tmpbuf, "From the calling DTE : %%u (1024)");
+			g_snprintf(tmpbuf, 80, "From the calling DTE : %%u (1024)");
 			break;
 		    case 0x0E:
-			sprintf(tmpbuf, "From the calling DTE : %%u (2048)");
+			g_snprintf(tmpbuf, 80, "From the calling DTE : %%u (2048)");
 			break;
 		    case 0x0F:
-			sprintf(tmpbuf, "From the calling DTE : %%u (4096)");
+			g_snprintf(tmpbuf, 80, "From the calling DTE : %%u (4096)");
 			break;
 		    default:
-			sprintf(tmpbuf, "From the calling DTE : %%u (Unknown)");
+			g_snprintf(tmpbuf, 80, "From the calling DTE : %%u (Unknown)");
 			break;
 		    }
 		    proto_tree_add_text(fac_subtree, tvb, *offset+2, 1,
@@ -970,8 +978,9 @@ dump_facilities(proto_tree *tree, int *offset, tvbuff_t *tvb)
 	    case X25_FAC_CALL_TRANSFER:
 		if (fac_tree) {
 		    int i;
-		    char tmpbuf[256];
+		    char *tmpbuf;
 
+		    tmpbuf=ep_alloc(258);
 		    ti = proto_tree_add_text(fac_tree, tvb, *offset, 1, "Code : %02X "
 			    "(Call redirection or deflection notification)", fac);
 		    fac_subtree = proto_item_add_subtree(ti, ett_x25_fac_call_transfer);
@@ -1018,7 +1027,7 @@ dump_facilities(proto_tree *tree, int *offset, tvbuff_t *tvb)
 		    proto_tree_add_text(fac_subtree, tvb, *offset+3, 1,
 			    "Number of semi-octets in DTE address : %u",
 			    byte3);
-		    for (i = 0; i < byte3; i++) {
+		    for (i = 0; (i<byte3)&&(i<256); i++) {
 			if (i % 2 == 0) {
 			    tmpbuf[i] = ((tvb_get_guint8(tvb, *offset+4+i/2) >> 4)
 				    & 0x0F) + '0';
@@ -1039,8 +1048,9 @@ dump_facilities(proto_tree *tree, int *offset, tvbuff_t *tvb)
 	    case X25_FAC_CALLING_ADDR_EXT:
 		if (fac_tree) {
 		    int i;
-		    char tmpbuf[256];
+		    char *tmpbuf;
 
+		    tmpbuf=ep_alloc(258);
 		    ti = proto_tree_add_text(fac_tree, tvb, *offset, 1, "Code : %02X "
 			    "(Calling address extension)", fac);
 		    fac_subtree = proto_item_add_subtree(ti,
@@ -1057,7 +1067,7 @@ dump_facilities(proto_tree *tree, int *offset, tvbuff_t *tvb)
 		    byte2 = tvb_get_guint8(tvb, *offset+2) & 0x3F;
 		    proto_tree_add_text(fac_subtree, tvb, *offset+2, 1,
 			    "Number of semi-octets in DTE address : %u", byte2);
-		    for (i = 0; i < byte2; i++) {
+		    for (i = 0; (i<byte2)&&(i<256) ; i++) {
 			if (i % 2 == 0) {
 			    tmpbuf[i] = ((tvb_get_guint8(tvb, *offset+3+i/2) >> 4)
 				    & 0x0F) + '0';
@@ -1078,8 +1088,9 @@ dump_facilities(proto_tree *tree, int *offset, tvbuff_t *tvb)
 	    case X25_FAC_CALLED_ADDR_EXT:
 		if (fac_tree) {
 		    int i;
-		    char tmpbuf[256];
+		    char *tmpbuf;
 
+		    tmpbuf=ep_alloc(258);
 		    ti = proto_tree_add_text(fac_tree, tvb, *offset, 1, "Code : %02X "
 			    "(Called address extension)", fac);
 		    fac_subtree = proto_item_add_subtree(ti,
@@ -1096,7 +1107,7 @@ dump_facilities(proto_tree *tree, int *offset, tvbuff_t *tvb)
 		    byte2 = tvb_get_guint8(tvb, *offset+2) & 0x3F;
 		    proto_tree_add_text(fac_subtree, tvb, *offset+2, 1,
 			    "Number of semi-octets in DTE address : %u", byte2);
-		    for (i = 0; i < byte2; i++) {
+		    for (i = 0; (i<byte2)&&(i<256) ; i++) {
 			if (i % 2 == 0) {
 			    tmpbuf[i] = ((tvb_get_guint8(tvb, *offset+3+i/2) >> 4)
 				    & 0x0F) + '0';
@@ -1129,8 +1140,9 @@ dump_facilities(proto_tree *tree, int *offset, tvbuff_t *tvb)
 	    case X25_FAC_CALL_DEFLECT:
 		if (fac_tree) {
 		    int i;
-		    char tmpbuf[256];
+		    char *tmpbuf;
 
+		    tmpbuf=ep_alloc(258);
 		    ti = proto_tree_add_text(fac_tree, tvb, *offset, 1, "Code : %02X "
 			    "(Call deflection selection)", fac);
 		    fac_subtree = proto_item_add_subtree(ti,
@@ -1155,7 +1167,7 @@ dump_facilities(proto_tree *tree, int *offset, tvbuff_t *tvb)
 		    proto_tree_add_text(fac_subtree, tvb, *offset+3, 1,
 			    "Number of semi-octets in the alternative DTE address : %u",
 			    byte3);
-		    for (i = 0; i < byte3; i++) {
+		    for (i = 0; (i<byte3)&&(i<256) ; i++) {
 			if (i % 2 == 0) {
 			    tmpbuf[i] = ((tvb_get_guint8(tvb, *offset+4+i/2) >> 4)
 				    & 0x0F) + '0';
@@ -1209,11 +1221,14 @@ x25_ntoa(proto_tree *tree, int *offset, tvbuff_t *tvb,
 {
     int len1, len2;
     int i;
-    char addr1[16], addr2[16];
+    char *addr1, *addr2;
     char *first, *second;
     guint8 byte;
     int localoffset;
 
+    addr1=ep_alloc(16);
+    addr2=ep_alloc(16);
+
     byte = tvb_get_guint8(tvb, *offset);
     len1 = (byte >> 0) & 0x0F;
     len2 = (byte >> 4) & 0x0F;
@@ -1291,11 +1306,14 @@ x25_toa(proto_tree *tree, int *offset, tvbuff_t *tvb,
 {
     int len1, len2;
     int i;
-    char addr1[256], addr2[256];
+    char *addr1, *addr2;
     char *first, *second;
     guint8 byte;
     int localoffset;
 
+    addr1=ep_alloc(256);
+    addr2=ep_alloc(256);
+
     len1 = tvb_get_guint8(tvb, *offset);
     if (tree) {
 	proto_tree_add_text(tree, tvb, *offset, 1,
@@ -1482,7 +1500,7 @@ dissect_x25_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
     gboolean toa;         /* TOA/NPI address format */
     guint16 bytes0_1;
     guint8 pkt_type;
-    char *short_name = NULL, *long_name = NULL;
+    const char *short_name = NULL, *long_name = NULL;
     tvbuff_t *next_tvb = NULL;
     gboolean q_bit_set = FALSE;
     gboolean m_bit_set;
@@ -2175,6 +2193,8 @@ dissect_x25_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
 	      
 		if (fd_head) {
 		    if (fd_head->next) {
+		        proto_item *frag_tree_item;
+
 		        /* This is the last packet */
 			next_tvb = tvb_new_real_data(fd_head->data, 
 						     fd_head->len,
@@ -2184,7 +2204,7 @@ dissect_x25_common(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree,
 			show_fragment_seq_tree(fd_head, 
 					       &x25_frag_items, 
 					       x25_tree, 
-					       pinfo, next_tvb);
+					       pinfo, next_tvb, &frag_tree_item);
 		    }
 	        }
 
