@@ -1,7 +1,7 @@
 /* timestamp.c
  * Routines for timestamp type setting.
  *
- * $Id: timestamp.c 11869 2004-09-01 07:07:23Z guy $
+ * $Id: timestamp.c 15534 2005-08-25 21:29:54Z ulfl $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -32,12 +32,26 @@
  * and distinguish it from a command line value */
 static ts_type timestamp_type = TS_NOT_SET;
 
-ts_type get_timestamp_setting(void)
+static int timestamp_precision = TS_PREC_AUTO_USEC;
+
+ts_type timestamp_get_type(void)
 {
 	return timestamp_type;
 }
 
-void set_timestamp_setting(ts_type ts_t)
+void timestamp_set_type(ts_type ts_t)
 {
 	timestamp_type = ts_t;
 }
+
+
+int timestamp_get_precision(void)
+{
+	return timestamp_precision;
+}
+
+void timestamp_set_precision(int tsp)
+{
+	timestamp_precision = tsp;
+}
+
