@@ -4,7 +4,7 @@
  * conversations_table   2003 Ronnie Sahlberg
  * Helper routines common to all endpoint conversations tap.
  *
- * $Id: conversations_table.c 14887 2005-07-09 11:26:46Z ulfl $
+ * $Id: conversations_table.c 17446 2006-03-02 20:50:33Z etxrab $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -30,6 +30,7 @@
 #endif
 
 #include <string.h>
+#include <ctype.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <math.h>
@@ -38,6 +39,7 @@
 
 #include <epan/packet_info.h>
 #include <epan/to_str.h>
+#include <epan/address.h>
 #include <epan/addr_resolv.h>
 #include <epan/tap.h>
 
@@ -53,7 +55,7 @@
 #include "gtk/color_dlg.h"
 #include "gtkglobals.h"
 #include "main.h"
-#include "ui_util.h"
+#include "gui_utils.h"
 #include "dlg_utils.h"
 #include "help_dlg.h"
 
@@ -61,7 +63,15 @@
 #define GTK_MENU_FUNC(a) ((GtkItemFactoryCallback)(a))
 
 #define NUM_COLS 10
+#define CONV_PTR_KEY "conversations-pointer"
 
+#define CMP_INT(i1, i2)	\
+	if ((i1) > (i2))	\
+		return 1;	\
+	else if ((i1) < (i2))	\
+		return -1;	\
+	else			\
+		return 0;
 
 /* convert a port number into a string */
 static char *
@@ -78,7 +88,8 @@ ct_port_to_str(int port_type, guint32 port)
 
 	switch(port_type){
 	case PT_TCP:
-	case PT_UDP:
+    case PT_UDP:
+    case PT_NCP:
 		g_snprintf(strp, 11, "%d", port);
 		return strp;
 	}
@@ -100,7 +111,7 @@ ct_port_to_str(int port_type, guint32 port)
    of protocols,   either ethernet, tokenring, fddi etc so we must be more
    specific there  thats why we need specific_addr_type
 */
-static char *
+static const char *
 ct_get_filter_name(address *addr, int specific_addr_type, int port_type, int name_type)
 {
 	switch(name_type){
@@ -212,6 +223,8 @@ ct_get_filter_name(address *addr, int specific_addr_type, int port_type, int nam
 			return "tcp.srcport";
 		case PT_UDP:
 			return "udp.srcport";
+        case PT_NCP:
+            return "ncp.connection";
                 default:
                         ;
 		}
@@ -222,6 +235,8 @@ ct_get_filter_name(address *addr, int specific_addr_type, int port_type, int nam
 			return "tcp.dstport";
 		case PT_UDP:
 			return "udp.dstport";
+        case PT_NCP:
+            return "ncp.connection";
                 default:
                         ;
 		}
@@ -232,6 +247,8 @@ ct_get_filter_name(address *addr, int specific_addr_type, int port_type, int nam
 			return "tcp.port";
 		case PT_UDP:
 			return "udp.port";
+        case PT_NCP:
+            return "ncp.connection";
                 default:
                         ;
 		}
@@ -256,7 +273,7 @@ reset_ct_table_data(conversations_table *ct)
 {
     guint32 i;
     char title[256];
-	
+
 	/* Allow clist to update */
 	gtk_clist_thaw(ct->table);
 
@@ -310,41 +327,50 @@ ct_win_destroy_cb(GtkWindow *win _U_, gpointer data)
 static gint
 ct_sort_column(GtkCList *clist, gconstpointer ptr1, gconstpointer ptr2)
 {
-	char *text1 = NULL;
-	char *text2 = NULL;
-	guint64 i1, i2;
+	guint32 idx1, idx2;
+	conversations_table *ct = OBJECT_GET_DATA(clist, CONV_PTR_KEY);
+	conversation_t *conv1 = NULL;
+	conversation_t *conv2 = NULL;
 
 	const GtkCListRow *row1 = ptr1;
 	const GtkCListRow *row2 = ptr2;
 
-	text1 = GTK_CELL_TEXT (row1->cell[clist->sort_column])->text;
-	text2 = GTK_CELL_TEXT (row2->cell[clist->sort_column])->text;
+	idx1 = GPOINTER_TO_INT(row1->data);
+	idx2 = GPOINTER_TO_INT(row2->data);
+
+	if (!ct || idx1 >= ct->num_conversations || idx2 >= ct->num_conversations)
+		return 0;
+
+	conv1 = &ct->conversations[idx1];
+	conv2 = &ct->conversations[idx2];
 
 	switch(clist->sort_column){
-	case 0:
-	case 2:
-		return strcmp (text1, text2);
-	case 1:
-	case 3:
-	case 4:
-	case 5:
-	case 6:
-	case 7:
-	case 8:
-	case 9:
-		sscanf(text1, "%" PRIu64, &i1);
-		sscanf(text2, "%" PRIu64, &i2);
-        /* XXX - this might cause trouble because of overflow problems */
-        /* XXX - is this correct anyway? Subtracting two unsigned values will still be an unsigned value, which will never become negative */
-		return (gint) (i1-i2);
+	case 0: /* Source address */
+		return(CMP_ADDRESS(&conv1->src_address, &conv2->src_address));
+	case 2: /* Destination address */
+		return(CMP_ADDRESS(&conv1->dst_address, &conv2->dst_address));
+	case 1: /* Source port */
+		CMP_INT(conv1->src_port, conv2->src_port);
+	case 3: /* Destination port */
+		CMP_INT(conv1->dst_port, conv2->dst_port);
+	case 4: /* Packets */
+		CMP_INT(conv1->tx_frames+conv1->rx_frames,
+			conv2->tx_frames+conv2->rx_frames);
+        case 5: /* Bytes */
+		CMP_INT(conv1->tx_bytes+conv1->rx_bytes,
+			conv2->tx_bytes+conv2->rx_bytes);
+        case 6: /* Packets A->B */
+		CMP_INT(conv1->tx_frames, conv2->tx_frames);
+        case 7: /* Bytes A->B */
+		CMP_INT(conv1->tx_bytes, conv2->tx_bytes);
+        case 8: /* Packets A<-B */
+		CMP_INT(conv1->rx_frames, conv2->rx_frames);
+        case 9: /* Bytes A<-B */
+		CMP_INT(conv1->rx_bytes, conv2->rx_bytes);
+	default:
+		g_assert_not_reached();
 	}
-	g_assert_not_reached();
-	
-	/* Allow clist to redraw */
-	
-	gtk_clist_thaw(clist);
-	gtk_clist_freeze(clist);
-	
+
 	return 0;
 }
 
@@ -369,8 +395,8 @@ ct_click_column_cb(GtkCList *clist, gint column, gpointer data)
 			gtk_widget_show(col_arrows[column].ascend_pm);
 		}
 	} else {
-		clist->sort_type = GTK_SORT_DESCENDING;
-		gtk_widget_show(col_arrows[column].descend_pm);
+		clist->sort_type = GTK_SORT_ASCENDING;
+		gtk_widget_show(col_arrows[column].ascend_pm);
 		gtk_clist_set_sort_column(clist, column);
 	}
 
@@ -1097,9 +1123,9 @@ draw_ct_table_data(conversations_table *ct)
         gtk_clist_set_text(ct->table, j, 9, str);
 
     }
-	
+
     draw_ct_table_addresses(ct);
-	
+
     gtk_clist_sort(ct->table);
 
     /* Allow table to redraw */
@@ -1117,44 +1143,44 @@ draw_ct_table_data_cb(void *arg)
 static void
 copy_as_csv_cb(GtkWindow *win _U_, gpointer data)
 {
-   guint32         i,j;                 
-   gchar           *table_entry;                      
-   gchar           *CSV_str;         
-   GtkClipboard    *cb;  
-   
+   guint32         i,j;
+   gchar           *table_entry;
+   GtkClipboard    *cb;
+   GString         *CSV_str = g_string_new("");
+
    conversations_table *talkers=(conversations_table *)data;
-   
-   CSV_str=g_new(gchar,(80*(talkers->num_conversations+1))); /* 80 chars * num rows */
-   strcpy(CSV_str,"");                                   /* initialize string   */
+
    /* Add the column headers to the CSV data */
    for(i=0;i<talkers->num_columns;i++){                  /* all columns         */
     if((i==1 || i==3) && !talkers->has_ports) continue;  /* Don't add the port column if it's empty */
-     strcat(CSV_str,talkers->default_titles[i]);         /* add the column heading to the CSV string */
-     strcat(CSV_str,",");
+     g_string_append(CSV_str,talkers->default_titles[i]);/* add the column heading to the CSV string */
+    if(i!=talkers->num_columns-1)
+     g_string_append(CSV_str,",");
    }
-   strcat(CSV_str,"\n");                                 /* new row */
- 
+   g_string_append(CSV_str,"\n");                        /* new row */
+
    /* Add the column values to the CSV data */
-   for(i=0;i<talkers->num_conversations;i++){                /* all rows            */
+   for(i=0;i<talkers->num_conversations;i++){            /* all rows            */
     for(j=0;j<talkers->num_columns;j++){                 /* all columns         */
      if((j==1 || j==3) && !talkers->has_ports) continue; /* Don't add the port column if it's empty */
      gtk_clist_get_text(talkers->table,i,j,&table_entry);/* copy table item into string */
-     strcat(CSV_str,table_entry);                        /* add the table entry to the CSV string */
-     strcat(CSV_str,",");
-    } 
-    strcat(CSV_str,"\n");                                /* new row */  
+     g_string_append(CSV_str,table_entry);               /* add the table entry to the CSV string */
+    if(j!=talkers->num_columns-1)
+     g_string_append(CSV_str,",");
+    }
+    g_string_append(CSV_str,"\n");                       /* new row */
    }
 
    /* Now that we have the CSV data, copy it into the default clipboard */
-   cb = gtk_clipboard_get(GDK_SELECTION_CLIPBOARD);     /* Get the default clipboard */
-   gtk_clipboard_set_text(cb, CSV_str, -1);             /* Copy the CSV data into the clipboard */
-   g_free(CSV_str);                                     /* Free the memory */
-} 
+   cb = gtk_clipboard_get(GDK_SELECTION_CLIPBOARD);      /* Get the default clipboard */
+   gtk_clipboard_set_text(cb, CSV_str->str, -1);         /* Copy the CSV data into the clipboard */
+   g_string_free(CSV_str, TRUE);                         /* Free the memory */
+}
 #endif
 
 
 static gboolean
-init_ct_table_page(conversations_table *conversations, GtkWidget *vbox, gboolean hide_ports, char *table_name, char *tap_name, char *filter, tap_packet_cb packet_func)
+init_ct_table_page(conversations_table *conversations, GtkWidget *vbox, gboolean hide_ports, const char *table_name, const char *tap_name, const char *filter, tap_packet_cb packet_func)
 {
     int i;
     column_arrows *col_arrows;
@@ -1166,13 +1192,13 @@ init_ct_table_page(conversations_table *conversations, GtkWidget *vbox, gboolean
 #if (GTK_MAJOR_VERSION >= 2)
     GtkWidget *copy_bt;
     GtkTooltips *tooltips = gtk_tooltips_new();
-#endif           
+#endif
 
 
     conversations->page_lb=NULL;
     conversations->resolve_names=TRUE;
-    conversations->has_ports=!hide_ports;   
-    conversations->num_columns=NUM_COLS;   
+    conversations->has_ports=!hide_ports;
+    conversations->num_columns=NUM_COLS;
     conversations->default_titles[0]="Address A",
     conversations->default_titles[1]="Port A";
     conversations->default_titles[2]="Address B";
@@ -1183,6 +1209,10 @@ init_ct_table_page(conversations_table *conversations, GtkWidget *vbox, gboolean
     conversations->default_titles[7]="Bytes A->B";
     conversations->default_titles[8]="Packets A<-B";
     conversations->default_titles[9]="Bytes A<-B";
+    if (strcmp(table_name, "NCP")==0) {
+        conversations->default_titles[1]="Connection A";
+        conversations->default_titles[3]="Connection B";
+    }
 
     g_snprintf(title, 255, "%s Conversations", table_name);
     label=gtk_label_new(title);
@@ -1193,6 +1223,7 @@ init_ct_table_page(conversations_table *conversations, GtkWidget *vbox, gboolean
     gtk_box_pack_start(GTK_BOX(vbox), conversations->scrolled_window, TRUE, TRUE, 0);
 
     conversations->table=(GtkCList *)gtk_clist_new(NUM_COLS);
+    OBJECT_SET_DATA(conversations->table, CONV_PTR_KEY, conversations);
 
     col_arrows = (column_arrows *) g_malloc(sizeof(column_arrows) * NUM_COLS);
     win_style = gtk_widget_get_style(conversations->scrolled_window);
@@ -1209,7 +1240,7 @@ init_ct_table_page(conversations_table *conversations, GtkWidget *vbox, gboolean
         gtk_table_attach(GTK_TABLE(col_arrows[i].table), col_arrows[i].descend_pm, 1, 2, 0, 1, GTK_SHRINK, GTK_SHRINK, 0, 0);
         /* make total frames be the default sort order */
         if (i == 4) {
-            gtk_widget_show(col_arrows[i].descend_pm);
+            gtk_widget_show(col_arrows[i].ascend_pm);
         }
         gtk_clist_set_column_widget(GTK_CLIST(conversations->table), i, col_arrows[i].table);
         gtk_widget_show(col_arrows[i].table);
@@ -1218,7 +1249,7 @@ init_ct_table_page(conversations_table *conversations, GtkWidget *vbox, gboolean
 
     gtk_clist_set_compare_func(conversations->table, ct_sort_column);
     gtk_clist_set_sort_column(conversations->table, 4);
-    gtk_clist_set_sort_type(conversations->table, GTK_SORT_DESCENDING);
+    gtk_clist_set_sort_type(conversations->table, GTK_SORT_ASCENDING);
 
 
     gtk_clist_set_column_auto_resize(conversations->table, 0, TRUE);
@@ -1254,11 +1285,11 @@ init_ct_table_page(conversations_table *conversations, GtkWidget *vbox, gboolean
     /* XXX - maybe we want to have a "Copy as CSV" stock button here? */
     /*copy_bt = gtk_button_new_with_label ("Copy content to clipboard as CSV");*/
     copy_bt = BUTTON_NEW_FROM_STOCK(GTK_STOCK_COPY);
-    gtk_tooltips_set_tip(tooltips, copy_bt, 
+    gtk_tooltips_set_tip(tooltips, copy_bt,
         "Copy all statistical values of this page to the clipboard in CSV (Comma Seperated Values) format.", NULL);
-    SIGNAL_CONNECT(copy_bt, "clicked", copy_as_csv_cb,(gpointer *) conversations);    
-    gtk_box_pack_start(GTK_BOX(vbox), copy_bt, FALSE, FALSE, 0); 
-#endif                 
+    SIGNAL_CONNECT(copy_bt, "clicked", copy_as_csv_cb,(gpointer *) conversations);
+    gtk_box_pack_start(GTK_BOX(vbox), copy_bt, FALSE, FALSE, 0);
+#endif
 
     /* register the tap and rerun the taps on the packet list */
     error_string=register_tap_listener(tap_name, conversations, filter, reset_ct_table_data_cb, packet_func, draw_ct_table_data_cb);
@@ -1273,7 +1304,7 @@ init_ct_table_page(conversations_table *conversations, GtkWidget *vbox, gboolean
 
 
 void
-init_conversation_table(gboolean hide_ports, char *table_name, char *tap_name, char *filter, tap_packet_cb packet_func)
+init_conversation_table(gboolean hide_ports, const char *table_name, const char *tap_name, const char *filter, tap_packet_cb packet_func)
 {
     conversations_table *conversations;
     char title[256];
@@ -1323,9 +1354,9 @@ init_conversation_table(gboolean hide_ports, char *table_name, char *tap_name, c
     gtk_widget_show_all(conversations->win);
     window_present(conversations->win);
 
-    cf_retap_packets(&cfile);
+    cf_retap_packets(&cfile, FALSE);
+
 
-	
     /* Keep clist frozen to cause modifications to the clist (inserts, appends, others that are extremely slow
 	   in GTK2) to not be drawn, allow refreshes to occur at strategic points for performance */
   	gtk_clist_freeze(conversations->table);
@@ -1352,7 +1383,7 @@ ct_win_destroy_notebook_cb(GtkWindow *win _U_, gpointer data)
 
 
 static conversations_table *
-init_ct_notebook_page_cb(gboolean hide_ports, char *table_name, char *tap_name, char *filter, tap_packet_cb packet_func)
+init_ct_notebook_page_cb(gboolean hide_ports, const char *table_name, const char *tap_name, const char *filter, tap_packet_cb packet_func)
 {
     gboolean ret;
     GtkWidget *page_vbox;
@@ -1378,9 +1409,9 @@ init_ct_notebook_page_cb(gboolean hide_ports, char *table_name, char *tap_name,
 
 typedef struct {
     gboolean hide_ports;       /* hide TCP / UDP port columns */
-    char *table_name;          /* GUI output name */
-    char *tap_name;            /* internal name */
-    char *filter;              /* display filter string (unused) */
+    const char *table_name;    /* GUI output name */
+    const char *tap_name;      /* internal name */
+    const char *filter;        /* display filter string (unused) */
     tap_packet_cb packet_func; /* function to be called for new incoming packets */
 } register_ct_t;
 
@@ -1388,7 +1419,7 @@ typedef struct {
 static GSList *registered_ct_tables = NULL;
 
 void
-register_conversation_table(gboolean hide_ports, char *table_name, char *tap_name, char *filter, tap_packet_cb packet_func)
+register_conversation_table(gboolean hide_ports, const char *table_name, const char *tap_name, const char *filter, tap_packet_cb packet_func)
 {
     register_ct_t *table;
 
@@ -1511,7 +1542,7 @@ init_conversation_notebook_cb(GtkWidget *w _U_, gpointer d _U_)
     gtk_widget_show_all(win);
     window_present(win);
 
-    cf_retap_packets(&cfile);
+    cf_retap_packets(&cfile, FALSE);
 
     /* after retapping, redraw table */
     for (page=1; page<=GPOINTER_TO_INT(pages[0]); page++) {
