@@ -7,7 +7,7 @@
  * Copyright 2000, Jeffrey C. Foster <jfoste@woodward.com> and
  * Guy Harris <guy@alum.mit.edu>
  *
- * $Id: dfilter_expr_dlg.c 12852 2004-12-29 01:08:20Z guy $
+ * $Id: dfilter_expr_dlg.c 17555 2006-03-09 19:38:51Z gerald $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -49,7 +49,7 @@
 
 #include "globals.h"
 #include "main.h"
-#include "ui_util.h"
+#include "gui_utils.h"
 #include "simple_dialog.h"
 #include "dlg_utils.h"
 #include "proto_dlg.h"
@@ -77,18 +77,18 @@ typedef struct protocol_data {
 
 static void show_relations(GtkWidget *relation_list, ftenum_t ftype);
 static gboolean relation_is_presence_test(const char *string);
-static void add_relation_list(GtkWidget *relation_list, char *relation, gboolean sensitive);
+static void add_relation_list(GtkWidget *relation_list, const char *relation, gboolean sensitive);
 static void build_boolean_values(GtkWidget *value_list_scrolled_win,
                                  GtkWidget *value_list,
                                  const true_false_string *values);
 static void build_enum_values(GtkWidget *value_list_scrolled_win,
                               GtkWidget *value_list,
                               const value_string *values);
-static void add_value_list_item(GtkWidget *value_list, gchar *string,
-                                gpointer data);
+static void add_value_list_item(GtkWidget *value_list, const gchar *string,
+                                const gpointer data);
 static void display_value_fields(header_field_info *hfinfo,
                                  gboolean is_comparison, GtkWidget *value_label,
-                                 GtkWidget *value_entry, 
+                                 GtkWidget *value_entry,
                                  GtkWidget *value_list_label, GtkWidget *value_list,
                                  GtkWidget *value_list_scrolled_win,
                                  GtkWidget *range_label,
@@ -297,7 +297,7 @@ relation_is_presence_test(const char *string)
 }
 
 static void
-add_relation_list(GtkWidget *relation_list, char *relation, gboolean sensitive)
+add_relation_list(GtkWidget *relation_list, const char *relation, gboolean sensitive)
 {
 #if GTK_MAJOR_VERSION < 2
     GtkWidget *label, *item;
@@ -431,7 +431,7 @@ build_boolean_values(GtkWidget *value_list_scrolled_win, GtkWidget *value_list,
      */
     if (values == NULL)
         values = &true_false;
-    add_value_list_item(value_list, values->true_string, (gpointer)values);
+    add_value_list_item(value_list, values->true_string, (const gpointer) values);
     add_value_list_item(value_list, values->false_string, NULL);
 
     /*
@@ -488,7 +488,7 @@ build_enum_values(GtkWidget *value_list_scrolled_win _U_, GtkWidget *value_list,
      */
     while (values->strptr != NULL) {
         add_value_list_item(value_list, values->strptr,
-                            (gpointer)values);
+                            (const gpointer) values);
         values++;
     }
 
@@ -503,7 +503,7 @@ build_enum_values(GtkWidget *value_list_scrolled_win _U_, GtkWidget *value_list,
 }
 
 static void
-add_value_list_item(GtkWidget *value_list, gchar *string, gpointer data)
+add_value_list_item(GtkWidget *value_list, const gchar *string, const gpointer data)
 {
 #if GTK_MAJOR_VERSION < 2
     GtkWidget *label, *item;
@@ -731,7 +731,7 @@ value_list_sel_cb(GtkTreeSelection *sel, gpointer value_entry_arg)
 }
 
 static void
-dfilter_report_bad_value(char *format, ...)
+dfilter_report_bad_value(const char *format, ...)
 {
 	char error_msg_buf[1024];
 	va_list args;
@@ -787,12 +787,8 @@ dfilter_expr_dlg_accept_cb(GtkWidget *w, gpointer filter_te_arg)
     gtk_label_get(GTK_LABEL(item_label), &item_str);
 #else
     if (gtk_tree_selection_get_selected(gtk_tree_view_get_selection(GTK_TREE_VIEW(relation_list)),
-                                        &model, &iter))
+                                        &model, &iter)) {
         gtk_tree_model_get(model, &iter, 0, &item_str, -1);
-    else {
-    	/* XXX - the relation list is in GTK_SELECTION_BROWSE mode; how
-    	   can this ever be null? */
-        item_str = NULL;
     }
 #endif
 
@@ -1078,7 +1074,7 @@ dfilter_expr_dlg_new(GtkWidget *filter_te)
 #if GTK_MAJOR_VERSION < 2
     int len;
     void *cookie;
-    gchar *name;
+    const gchar *name;
     GHashTable *proto_array;
     GtkCTreeNode *protocol_node, *item_node;
 #else
@@ -1112,7 +1108,7 @@ dfilter_expr_dlg_new(GtkWidget *filter_te)
 
     tree_scrolled_win = scrolled_window_new(NULL, NULL);
 #if GTK_MAJOR_VERSION >= 2
-    gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(tree_scrolled_win), 
+    gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(tree_scrolled_win),
                                    GTK_SHADOW_IN);
 #endif
     gtk_box_pack_start(GTK_BOX(field_vb), tree_scrolled_win, TRUE, TRUE, 0);
@@ -1164,7 +1160,7 @@ dfilter_expr_dlg_new(GtkWidget *filter_te)
     gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(relation_list_scrolled_win),
                                    GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
 #if GTK_MAJOR_VERSION >= 2
-    gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(relation_list_scrolled_win), 
+    gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(relation_list_scrolled_win),
                                    GTK_SHADOW_IN);
 #endif
 
@@ -1326,7 +1322,7 @@ dfilter_expr_dlg_new(GtkWidget *filter_te)
         name = proto_get_protocol_short_name(protocol); /* name, short_name or filter name ? */
         protocol_node = gtk_ctree_insert_node(GTK_CTREE(field_tree),
                                               NULL, NULL,
-                                              &name, 5,
+                                              (gchar **) &name, 5,
                                               NULL, NULL, NULL, NULL,
                                               FALSE, FALSE);
         gtk_ctree_node_set_row_data(GTK_CTREE(field_tree), protocol_node,
@@ -1396,7 +1392,7 @@ dfilter_expr_dlg_new(GtkWidget *filter_te)
 
     GtkTreeIter iter, child_iter;
     void *cookie, *cookie2;
-    gchar *name;
+    const gchar *name;
 
     for (i = proto_get_first_protocol(&cookie); i != -1;
 	 i = proto_get_next_protocol(&cookie)) {
