@@ -1,7 +1,7 @@
 /* filter_dlg.c
  * Dialog boxes for (display and capture) filter editing
  *
- * $Id: filter_dlg.c 13890 2005-03-24 03:58:38Z guy $
+ * $Id: filter_dlg.c 16926 2005-12-31 17:09:27Z ulfl $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -36,7 +36,7 @@
 #include "gtk/main.h"
 #include "filter_dlg.h"
 #include "dlg_utils.h"
-#include "ui_util.h"
+#include "gui_utils.h"
 #include "simple_dialog.h"
 #include "dfilter_expr_dlg.h"
 #include "compat_macros.h"
@@ -105,7 +105,8 @@ capture_filter_construct_cb(GtkWidget *w, gpointer user_data _U_)
 	static construct_args_t args = {
 		"Ethereal: Capture Filter",
 		FALSE,
-		FALSE
+		FALSE,
+        FALSE
 	};
 
 	/* Has a filter dialog box already been opened for that button? */
@@ -194,7 +195,8 @@ cfilter_dialog_cb(GtkWidget *w _U_)
 	static construct_args_t args = {
 		"Ethereal: Capture Filter",
 		FALSE,
-		FALSE
+		FALSE,
+        FALSE
 	};
 
 	/* Has a filter dialog box already been opened for editing
@@ -222,7 +224,8 @@ dfilter_dialog_cb(GtkWidget *w _U_)
 	static construct_args_t args = {
 		"Ethereal: Display Filter",
 		TRUE,
-		TRUE
+		TRUE,
+        FALSE
 	};
 
     display_filter_construct_cb(OBJECT_GET_DATA(top_level, E_FILT_BT_PTR_KEY), &args);
@@ -362,6 +365,12 @@ filter_dialog_new(GtkWidget *button, GtkWidget *parent_filter_te,
 	gtk_window_set_default_size(GTK_WINDOW(main_w), 400, 400);
     OBJECT_SET_DATA(main_w, E_FILT_CONSTRUCT_ARGS_KEY, construct_args);
 
+    if(construct_args->modal_and_transient) {
+        GdkWindow*  parent = gtk_widget_get_parent_window(parent_filter_te);
+        gtk_window_set_transient_for(GTK_WINDOW(main_w), GTK_WINDOW(parent));
+        gtk_window_set_modal(GTK_WINDOW(main_w), TRUE);
+    }
+
     main_vb = gtk_vbox_new(FALSE, 0);
     gtk_container_border_width(GTK_CONTAINER(main_vb), 5);
     gtk_container_add(GTK_CONTAINER(main_w), main_vb);
@@ -813,7 +822,7 @@ filter_dlg_save_cb(GtkWidget *save_bt _U_, gpointer data)
 	char *pf_dir_path;
 	char *f_path;
 	int f_save_errno;
-	char *filter_type;
+	const char *filter_type;
 
 	/* Create the directory that holds personal configuration files,
 	   if necessary.  */
