@@ -6,7 +6,7 @@
  *
  * Modified from summary_dlg.c
  *
- * $Id: gsm_map_summary.c 12128 2004-09-29 00:06:36Z guy $
+ * $Id: gsm_map_summary.c 15501 2005-08-21 19:24:11Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -38,10 +38,13 @@
 #include "epan/packet_info.h"
 #include "epan/epan.h"
 #include "epan/value_string.h"
-#include "tap_menu.h"
+#include "../stat_menu.h"
+#include "gui_stat_menu.h"
+#include "globals.h"
+#include "file.h"
 #include "summary.h"
 #include "dlg_utils.h"
-#include "ui_util.h"
+#include "gui_utils.h"
 #include "compat_macros.h"
 #include <epan/tap.h>
 
@@ -62,7 +65,7 @@ add_string_to_box(gchar *str, GtkWidget *box)
 }
 
 
-void
+static void
 gsm_map_stat_gtk_sum_cb(GtkWidget *w _U_, gpointer d _U_)
 {
   summary_tally summary;
@@ -80,7 +83,7 @@ gsm_map_stat_gtk_sum_cb(GtkWidget *w _U_, gpointer d _U_)
   double	tot_invokes_size, tot_rr_size;
 
   /* initialize the tally */
-  summary_fill_in(&summary);
+  summary_fill_in(&cfile, &summary);
 
   /* initial compututations */
   seconds = summary.stop_time - summary.start_time;
@@ -103,7 +106,7 @@ gsm_map_stat_gtk_sum_cb(GtkWidget *w _U_, gpointer d _U_)
   gtk_widget_show(file_box);
 
   /* filename */
-  g_snprintf(string_buff, SUM_STR_MAX, "Name: %s", summary.filename);
+  g_snprintf(string_buff, SUM_STR_MAX, "Name: %s", ((summary.filename) ? summary.filename : "None"));
   add_string_to_box(string_buff, file_box);
 
   /* length */
@@ -170,7 +173,10 @@ gsm_map_stat_gtk_sum_cb(GtkWidget *w _U_, gpointer d _U_)
   add_string_to_box(string_buff, invoke_box);
 
   /* Total number of invokes per second */
-  g_snprintf(string_buff, SUM_STR_MAX, "Total number of Invokes per second: %.2f", tot_invokes/seconds);
+  if (seconds)
+	g_snprintf(string_buff, SUM_STR_MAX, "Total number of Invokes per second: %.2f", tot_invokes/seconds);
+  else
+	g_snprintf(string_buff, SUM_STR_MAX, "Total number of Invokes per second: N/A");
   add_string_to_box(string_buff, invoke_box);
 
   /* Total size of invokes */
@@ -178,11 +184,17 @@ gsm_map_stat_gtk_sum_cb(GtkWidget *w _U_, gpointer d _U_)
   add_string_to_box(string_buff, invoke_box);
 
   /* Average size of invokes */
-  g_snprintf(string_buff, SUM_STR_MAX, "Average number of bytes per Invoke: %.2f", tot_invokes_size/tot_invokes);
+  if (tot_invokes)
+	g_snprintf(string_buff, SUM_STR_MAX, "Average number of bytes per Invoke: %.2f", tot_invokes_size/tot_invokes);
+  else
+	g_snprintf(string_buff, SUM_STR_MAX, "Average number of bytes per Invoke: N/A");
   add_string_to_box(string_buff, invoke_box);
 
   /* Average size of invokes per second */
-  g_snprintf(string_buff, SUM_STR_MAX, "Average number of bytes per second: %.2f", tot_invokes_size/seconds);
+  if (seconds)
+	g_snprintf(string_buff, SUM_STR_MAX, "Average number of bytes per second: %.2f", tot_invokes_size/seconds);
+  else
+	g_snprintf(string_buff, SUM_STR_MAX, "Average number of bytes per second: N/A");
   add_string_to_box(string_buff, invoke_box);
 
   /* Return Results frame */
@@ -199,7 +211,10 @@ gsm_map_stat_gtk_sum_cb(GtkWidget *w _U_, gpointer d _U_)
   add_string_to_box(string_buff, rr_box);
 
   /* Total number of return results per second */
-  g_snprintf(string_buff, SUM_STR_MAX, "Total number of Return Results per second: %.2f", tot_rr/seconds);
+  if (seconds)
+	g_snprintf(string_buff, SUM_STR_MAX, "Total number of Return Results per second: %.2f", tot_rr/seconds);
+  else
+	g_snprintf(string_buff, SUM_STR_MAX, "Total number of Return Results per second: N/A");
   add_string_to_box(string_buff, rr_box);
 
   /* Total size of return results */
@@ -207,11 +222,17 @@ gsm_map_stat_gtk_sum_cb(GtkWidget *w _U_, gpointer d _U_)
   add_string_to_box(string_buff, rr_box);
 
   /* Average size of return results */
-  g_snprintf(string_buff, SUM_STR_MAX, "Average number of bytes per Return Result: %.2f", tot_rr_size/tot_rr);
+  if (tot_rr)
+	g_snprintf(string_buff, SUM_STR_MAX, "Average number of bytes per Return Result: %.2f", tot_rr_size/tot_rr);
+  else
+	g_snprintf(string_buff, SUM_STR_MAX, "Average number of bytes per Return Result: N/A");
   add_string_to_box(string_buff, rr_box);
 
   /* Average size of return results per second */
-  g_snprintf(string_buff, SUM_STR_MAX, "Average number of bytes per second: %.2f", tot_rr_size/seconds);
+  if (seconds)
+	g_snprintf(string_buff, SUM_STR_MAX, "Average number of bytes per second: %.2f", tot_rr_size/seconds);
+  else
+	g_snprintf(string_buff, SUM_STR_MAX, "Average number of bytes per second: N/A");
   add_string_to_box(string_buff, rr_box);
 
   /* Totals frame */
@@ -227,19 +248,28 @@ gsm_map_stat_gtk_sum_cb(GtkWidget *w _U_, gpointer d _U_)
   g_snprintf(string_buff, SUM_STR_MAX, "Total number of GSM MAP messages: %u", tot_invokes + tot_rr);
   add_string_to_box(string_buff, tot_box);
 
-  g_snprintf(string_buff, SUM_STR_MAX, "Total number of GSM MAP messages per second: %.2f",
-    (tot_invokes + tot_rr)/seconds);
+  if (seconds)
+	g_snprintf(string_buff, SUM_STR_MAX, "Total number of GSM MAP messages per second: %.2f",
+		(tot_invokes + tot_rr)/seconds);
+  else
+	g_snprintf(string_buff, SUM_STR_MAX, "Total number of GSM MAP messages per second: N/A");
   add_string_to_box(string_buff, tot_box);
 
   g_snprintf(string_buff, SUM_STR_MAX, "Total number of bytes for GSM MAP messages: %.0f", tot_invokes_size + tot_rr_size);
   add_string_to_box(string_buff, tot_box);
 
-  g_snprintf(string_buff, SUM_STR_MAX, "Average number of bytes per GSM MAP messages: %.2f",
-    (tot_invokes_size + tot_rr_size)/(tot_invokes + tot_rr));
+  if (tot_invokes + tot_rr)
+	g_snprintf(string_buff, SUM_STR_MAX, "Average number of bytes per GSM MAP messages: %.2f",
+		(tot_invokes_size + tot_rr_size)/(tot_invokes + tot_rr));
+  else
+	g_snprintf(string_buff, SUM_STR_MAX, "Average number of bytes per GSM MAP messages: N/A");
   add_string_to_box(string_buff, tot_box);
 
-  g_snprintf(string_buff, SUM_STR_MAX, "Average number of bytes second: %.2f",
-    (tot_invokes_size + tot_rr_size)/seconds);
+  if (seconds)
+	g_snprintf(string_buff, SUM_STR_MAX, "Average number of bytes second: %.2f",
+		(tot_invokes_size + tot_rr_size)/seconds);
+  else
+	  g_snprintf(string_buff, SUM_STR_MAX, "Average number of bytes second: N/A");
   add_string_to_box(string_buff, tot_box);
 
 
@@ -261,6 +291,6 @@ gsm_map_stat_gtk_sum_cb(GtkWidget *w _U_, gpointer d _U_)
 void
 register_tap_listener_gtkgsm_map_summary(void)
 {
-    register_tap_menu_item("GSM/MAP Summary",  REGISTER_TAP_GROUP_NONE,
+    register_stat_menu_item("GSM/MAP Summary",  REGISTER_STAT_GROUP_TELEPHONY,
         gsm_map_stat_gtk_sum_cb, NULL, NULL, NULL);
 }
