@@ -1,7 +1,7 @@
 /* hostlist_ipx.c   2004 Ian Schorr
  * modified from endpoint_talkers_ipx.c   2003 Ronnie Sahlberg
  *
- * $Id: hostlist_ipx.c 12128 2004-09-29 00:06:36Z guy $
+ * $Id: hostlist_ipx.c 17252 2006-02-11 13:05:24Z lego $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -33,7 +33,9 @@
 #include <gtk/gtk.h>
 #include <string.h>
 #include "epan/packet.h"
-#include "tap_menu.h"
+#include <epan/stat_cmd_args.h>
+#include "../stat_menu.h"
+#include "gui_stat_menu.h"
 #include <epan/tap.h>
 #include "../register.h"
 #include "hostlist_table.h"
@@ -41,10 +43,10 @@
 
 
 static int
-ipx_hostlist_packet(void *pit, packet_info *pinfo, epan_dissect_t *edt _U_, void *vip)
+ipx_hostlist_packet(void *pit, packet_info *pinfo, epan_dissect_t *edt _U_, const void *vip)
 {
 	hostlist_table *hosts=(hostlist_table *)pit;
-	ipxhdr_t *ipxh=vip;
+	const ipxhdr_t *ipxh=vip;
 
 	/* Take two "add" passes per packet, adding for each direction, ensures that all
 	packets are counted properly (even if address is sending to itself) 
@@ -58,9 +60,9 @@ ipx_hostlist_packet(void *pit, packet_info *pinfo, epan_dissect_t *edt _U_, void
 
 
 static void
-gtk_ipx_hostlist_init(char *optarg)
+gtk_ipx_hostlist_init(const char *optarg, void* userdata _U_)
 {
-	char *filter=NULL;
+	const char *filter=NULL;
 
 	if(!strncmp(optarg,"hosts,ipx,",10)){
 		filter=optarg+10;
@@ -68,7 +70,7 @@ gtk_ipx_hostlist_init(char *optarg)
 		filter=NULL;
 	}
 
-	init_hostlist_table(TRUE, "IPX Hosts", "ipx", filter, (void *)ipx_hostlist_packet);
+	init_hostlist_table(TRUE, "IPX Hosts", "ipx", filter, ipx_hostlist_packet);
 
 }
 
@@ -76,18 +78,17 @@ gtk_ipx_hostlist_init(char *optarg)
 static void
 gtk_ipx_hostlist_cb(GtkWidget *w _U_, gpointer d _U_)
 {
-	gtk_ipx_hostlist_init("hosts,ipx");
+	gtk_ipx_hostlist_init("hosts,ipx",NULL);
 }
 
 
 void
 register_tap_listener_ipx_hostlist(void)
 {
-	register_ethereal_tap("hosts,ipx", gtk_ipx_hostlist_init);
+	register_stat_cmd_arg("hosts,ipx", gtk_ipx_hostlist_init,NULL);
 
-	register_tap_menu_item("IPX", REGISTER_TAP_GROUP_ENDPOINT_LIST,
+	register_stat_menu_item("IPX", REGISTER_STAT_GROUP_ENDPOINT_LIST,
 	    gtk_ipx_hostlist_cb, NULL, NULL, NULL);
 
-	register_hostlist_table(TRUE, "IPX", "ipx", NULL /*filter*/, (void *)ipx_hostlist_packet);
+	register_hostlist_table(TRUE, "IPX", "ipx", NULL /*filter*/, ipx_hostlist_packet);
 }
-
