@@ -1,7 +1,7 @@
 /* packet_list.c
  * packet list related functions   2002 Olivier Abad
  *
- * $Id: packet_list.c 14826 2005-06-30 22:46:19Z ulfl $
+ * $Id: packet_list.c 17251 2006-02-11 11:21:38Z guy $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -35,7 +35,7 @@
 #include "color.h"
 #include "color_filters.h"
 #include "../ui_util.h"
-#include "ui_util.h"
+#include "gui_utils.h"
 #include "main.h"
 #include "menu.h"
 #include "colors.h"
@@ -121,13 +121,13 @@ GtkWidget *packet_list;
    a lower time stamp than any frame with a non-reference time;
    if both packets' times are reference times, we compare the
    times of the packets. */
-#define COMPARE_TS(secs, usecs) \
+#define COMPARE_TS(ts) \
 		((fdata1->flags.ref_time && !fdata2->flags.ref_time) ? -1 : \
 		 (!fdata1->flags.ref_time && fdata2->flags.ref_time) ? 1 : \
-		 (fdata1->secs < fdata2->secs) ? -1 : \
-		 (fdata1->secs > fdata2->secs) ? 1 : \
-		 (fdata1->usecs < fdata2->usecs) ? -1 :\
-		 (fdata1->usecs > fdata2->usecs) ? 1 : \
+		 (fdata1->ts.secs < fdata2->ts.secs) ? -1 : \
+		 (fdata1->ts.secs > fdata2->ts.secs) ? 1 : \
+		 (fdata1->ts.nsecs < fdata2->ts.nsecs) ? -1 :\
+		 (fdata1->ts.nsecs > fdata2->ts.nsecs) ? 1 : \
 		 COMPARE_FRAME_NUM())
 static gint
 packet_list_compare(EthCList *clist, gconstpointer  ptr1, gconstpointer  ptr2)
@@ -158,29 +158,32 @@ packet_list_compare(EthCList *clist, gconstpointer  ptr1, gconstpointer  ptr2)
     return COMPARE_FRAME_NUM();
 
   case COL_CLS_TIME:
-    switch (get_timestamp_setting()) {
+    switch (timestamp_get_type()) {
 
     case TS_ABSOLUTE:
     case TS_ABSOLUTE_WITH_DATE:
-      return COMPARE_TS(abs_secs, abs_usecs);
+      return COMPARE_TS(abs_ts);
 
     case TS_RELATIVE:
-      return COMPARE_TS(rel_secs, rel_usecs);
+      return COMPARE_TS(rel_ts);
 
     case TS_DELTA:
-      return COMPARE_TS(del_secs, del_usecs);
+      return COMPARE_TS(del_ts);
+
+    case TS_NOT_SET:
+      return 0;
     }
     return 0;
 
   case COL_ABS_TIME:
   case COL_ABS_DATE_TIME:
-    return COMPARE_TS(abs_secs, abs_usecs);
+    return COMPARE_TS(abs_ts);
 
   case COL_REL_TIME:
-    return COMPARE_TS(rel_secs, rel_usecs);
+    return COMPARE_TS(rel_ts);
 
   case COL_DELTA_TIME:
-    return COMPARE_TS(del_secs, del_usecs);
+    return COMPARE_TS(del_ts);
 
   case COL_PACKET_LENGTH:
     return COMPARE_NUM(pkt_len);
@@ -310,12 +313,12 @@ set_frame_mark(gboolean set, frame_data *frame, gint row) {
 }
 
 /* call this after last set_frame_mark is done */
-void mark_frames_ready(void) {
-  file_set_save_marked_sensitive();
+static void mark_frames_ready(void) {
+  file_save_update_dynamics();
   packets_bar_update();
 }
 
-void mark_frame_cb(GtkWidget *w _U_, gpointer data _U_) {
+void packet_list_mark_frame_cb(GtkWidget *w _U_, gpointer data _U_) {
   if (cfile.current_frame) {
     /* XXX hum, should better have a "cfile->current_row" here ... */
     set_frame_mark(!cfile.current_frame->flags.marked,
@@ -338,7 +341,7 @@ static void mark_all_frames(gboolean set) {
   mark_frames_ready();
 }
 
-void update_marked_frames(void) {
+void packet_list_update_marked_frames(void) {
   frame_data *fdata;
 
   if (cfile.plist == NULL) return;
@@ -354,11 +357,11 @@ void update_marked_frames(void) {
   mark_frames_ready();
 }
 
-void mark_all_frames_cb(GtkWidget *w _U_, gpointer data _U_) {
+void packet_list_mark_all_frames_cb(GtkWidget *w _U_, gpointer data _U_) {
   mark_all_frames(TRUE);
 }
 
-void unmark_all_frames_cb(GtkWidget *w _U_, gpointer data _U_) {
+void packet_list_unmark_all_frames_cb(GtkWidget *w _U_, gpointer data _U_) {
   mark_all_frames(FALSE);
 }
 
@@ -416,7 +419,7 @@ packet_list_button_pressed_cb(GtkWidget *w, GdkEvent *event, gpointer data _U_)
 
 /* Set the selection mode of the packet list window. */
 void
-set_plist_sel_browse(gboolean val)
+packet_list_set_sel_browse(gboolean val)
 {
         GtkSelectionMode new_mode;
         /* initialize with a mode we don't use, so that the mode == new_mode
@@ -454,7 +457,7 @@ set_plist_sel_browse(gboolean val)
 
 /* Set the font of the packet list window. */
 void
-set_plist_font(FONT_TYPE *font)
+packet_list_set_font(FONT_TYPE *font)
 {
 	int i;
 	gint col_width;
@@ -516,8 +519,8 @@ packet_list_new(e_prefs *prefs)
     /* Column titles are filled in below */
     gtk_container_add(GTK_CONTAINER(pkt_scrollw), packet_list);
 
-    set_plist_sel_browse(prefs->gui_plist_sel_browse);
-    set_plist_font(user_font_get_regular());
+    packet_list_set_sel_browse(prefs->gui_plist_sel_browse);
+    packet_list_set_font(user_font_get_regular());
     gtk_widget_set_name(packet_list, "packet list");
     SIGNAL_CONNECT(packet_list, "select-row", packet_list_select_cb, NULL);
     SIGNAL_CONNECT(packet_list, "unselect-row", packet_list_unselect_cb, NULL);
@@ -558,10 +561,10 @@ packet_list_set_column_titles(void)
     win_style = gtk_widget_get_style(top_level);
     ascend_pm = gdk_pixmap_create_from_xpm_d(top_level->window, &ascend_bm,
                                              &win_style->bg[GTK_STATE_NORMAL],
-                                             (gchar **)clist_ascend_xpm);
+                                             (gchar **) clist_ascend_xpm);
     descend_pm = gdk_pixmap_create_from_xpm_d(top_level->window, &descend_bm,
                                               &win_style->bg[GTK_STATE_NORMAL],
-                                              (gchar **)clist_descend_xpm);
+                                              (gchar **) clist_descend_xpm);
 
     col_arrows = (column_arrows *) g_malloc(sizeof(column_arrows) *
                                             cfile.cinfo.num_cols);
@@ -606,63 +609,69 @@ packet_list_freeze(void)
     eth_clist_freeze(ETH_CLIST(packet_list));
 }
 
-void
+static void
 packet_list_resize_columns(void) {
     int         i;
     int         progbar_nextstep;
     int         progbar_quantum;
-    gboolean    stop_flag;
-    GTimeVal    start_time;
-    float       prog_val;
+    gboolean    progbar_stop_flag;
+    GTimeVal    progbar_start_time;
+    float       progbar_val;
     progdlg_t  *progbar = NULL;
     gchar       status_str[100];
 
-
+    /* Update the progress bar when it gets to this value. */
     progbar_nextstep = 0;
     /* When we reach the value that triggers a progress bar update,
        bump that value by this amount. */
     progbar_quantum = cfile.cinfo.num_cols/N_PROGBAR_UPDATES;
+    /* Progress so far. */
+    progbar_val = 0.0;
 
-    stop_flag = FALSE;
-    g_get_current_time(&start_time);
+    progbar_stop_flag = FALSE;
+    g_get_current_time(&progbar_start_time);
 
 
     main_window_update();
 
     for (i = 0; i < cfile.cinfo.num_cols; i++) {
+      /* Create the progress bar if necessary.
+         We check on every iteration of the loop, so that it takes no
+         longer than the standard time to create it (otherwise, for a
+         large file, we might take considerably longer than that standard
+         time in order to get to the next progress bar step). */
+      if (progbar == NULL)
+         progbar = delayed_create_progress_dlg("Resizing", "Resize Columns", 
+           TRUE, &progbar_stop_flag, &progbar_start_time, progbar_val);
+
       if (i >= progbar_nextstep) {
         /* let's not divide by zero. I should never be started
          * with count == 0, so let's assert that
          */
         g_assert(cfile.cinfo.num_cols > 0);
 
-        prog_val = (gfloat) i / cfile.cinfo.num_cols;
-
-        /* Create the progress bar if necessary */
-        if (progbar == NULL)
-           progbar = delayed_create_progress_dlg("Resizing", "Resize Columns", 
-             &stop_flag, &start_time, prog_val);
+        progbar_val = (gfloat) i / cfile.cinfo.num_cols;
 
         if (progbar != NULL) {
           g_snprintf(status_str, sizeof(status_str),
                      "%u of %u columns (%s)", i+1, cfile.cinfo.num_cols, cfile.cinfo.col_title[i]);
-          update_progress_dlg(progbar, prog_val, status_str);
+          update_progress_dlg(progbar, progbar_val, status_str);
         }
 
         progbar_nextstep += progbar_quantum;
       }
 
-      if (stop_flag) {
+      if (progbar_stop_flag) {
         /* Well, the user decided to abort the resizing... */
         break;
       }
 
-        /* auto resize the current column */
-        eth_clist_set_column_auto_resize(ETH_CLIST(packet_list), i, TRUE);
+      /* auto resize the current column */
+      eth_clist_set_column_auto_resize(ETH_CLIST(packet_list), i, TRUE);
 
-        /* the current column should be resizeable by the user again */
-        /* (will turn off auto resize again) */
-        eth_clist_set_column_resizeable(ETH_CLIST(packet_list), i, TRUE);
+      /* the current column should be resizeable by the user again */
+      /* (will turn off auto resize again) */
+      eth_clist_set_column_resizeable(ETH_CLIST(packet_list), i, TRUE);
     }
 
     /* We're done resizing the columns; destroy the progress bar if it
@@ -702,7 +711,7 @@ packet_list_append(const gchar *text[], gpointer data)
 {
     gint row;
 
-    row = eth_clist_append(ETH_CLIST(packet_list), (gchar **)text);
+    row = eth_clist_append(ETH_CLIST(packet_list), (gchar **) text);
     eth_clist_set_row_data(ETH_CLIST(packet_list), row, data);
     return row;
 }
@@ -765,14 +774,49 @@ packet_list_get_row_data(gint row)
     return eth_clist_get_row_data(ETH_CLIST(packet_list), row);
 }
 
+
+/* get the first fully visible row number, given row MUST be visible */
+static gint
+packet_list_first_full_visible_row(gint row) {
+
+	g_assert(eth_clist_row_is_visible(ETH_CLIST(packet_list), row) ==
+        GTK_VISIBILITY_FULL);
+
+	while(eth_clist_row_is_visible(ETH_CLIST(packet_list), row) ==
+        GTK_VISIBILITY_FULL) {
+		row--;
+	}
+
+	return ++row;
+}
+
+/* get the last fully visible row number, given row MUST be visible */
+static gint
+packet_list_last_full_visible_row(gint row) {
+
+	g_assert(eth_clist_row_is_visible(ETH_CLIST(packet_list), row) ==
+        GTK_VISIBILITY_FULL);
+
+	while(eth_clist_row_is_visible(ETH_CLIST(packet_list), row) ==
+        GTK_VISIBILITY_FULL) {
+		row++;
+	}
+
+	return --row;
+}
+
 /* Set the selected row and the focus row of the packet list to the specified
  * row, and make it visible if it's not currently visible. */
 void
 packet_list_set_selected_row(gint row)
 {
-    if (eth_clist_row_is_visible(ETH_CLIST(packet_list), row) !=
-        GTK_VISIBILITY_FULL)
-        eth_clist_moveto(ETH_CLIST(packet_list), row, -1, 0.0, 0.0);
+	gint visible_rows;
+	gint first_row;
+	gboolean full_visible;
+
+
+	full_visible = eth_clist_row_is_visible(ETH_CLIST(packet_list), row) ==
+        GTK_VISIBILITY_FULL;
 
     /* XXX - why is there no "eth_clist_set_focus_row()", so that we
      * can make the row for the frame we found the focus row?
@@ -782,6 +826,35 @@ packet_list_set_selected_row(gint row)
     ETH_CLIST(packet_list)->focus_row = row;
 
     eth_clist_select_row(ETH_CLIST(packet_list), row, -1);
+
+    if (!full_visible) {
+
+        eth_clist_freeze(ETH_CLIST(packet_list));
+
+        eth_clist_moveto(ETH_CLIST(packet_list), row, -1, 0.0, 0.0);
+
+		/* even after move still invisible (happens with empty list) -> give up */
+		if(eth_clist_row_is_visible(ETH_CLIST(packet_list), row) !=
+			GTK_VISIBILITY_FULL) {
+			return;
+		}
+
+		/* The now selected row will be the first visible row in the list.
+		 * This is inconvenient, as the user is usually interested in some 
+		 * packets *before* the currently selected one too.
+		 *
+		 * Try to adjust the visible rows, so the currently selected row will 
+		 * be shown around the first third of the list screen.
+		 * 
+		 * (This won't even do any harm if the current row is the first or the 
+		 * last in the list) */
+		visible_rows = packet_list_last_full_visible_row(row) - packet_list_first_full_visible_row(row);
+		first_row = row - visible_rows / 3;
+
+		eth_clist_moveto(ETH_CLIST(packet_list), first_row >= 0 ? first_row : 0, -1, 0.0, 0.0);
+
+		eth_clist_thaw(ETH_CLIST(packet_list));
+	}
 }
 
 /* Return the column number that the clist is currently sorted by */
