@@ -1,7 +1,7 @@
 /* rtp_stream_dlg.c
  * RTP streams summary addition for ethereal
  *
- * $Id: rtp_stream_dlg.c 12636 2004-12-01 11:54:41Z guy $
+ * $Id: rtp_stream_dlg.c 15844 2005-09-17 00:02:31Z guy $
  *
  * Copyright 2003, Alcatel Business Systems
  * By Lars Ruoff <lars.ruoff@gmx.net>
@@ -36,24 +36,25 @@
 #include "globals.h"
 #include "epan/filesystem.h"
 
-#include "tap_menu.h"
+#include "../stat_menu.h"
+#include "gui_stat_menu.h"
 #include "dlg_utils.h"
-#include "ui_util.h"
+#include "gui_utils.h"
 #include "compat_macros.h"
 #include "gtkglobals.h"
 
 #include "image/clist_ascend.xpm"
 #include "image/clist_descend.xpm"
 
-#include "rtp_pt.h"
+#include <epan/rtp_pt.h>
 
 #include <epan/address.h>
 
 #include <string.h>
+#include <locale.h>
+#include <epan/addr_resolv.h>
 
 
-typedef const guint8 * ip_addr_p;
-
 static const gchar FWD_LABEL_TEXT[] = "Select a forward stream with left mouse button";
 static const gchar REV_LABEL_TEXT[] = "Select a reverse stream with SHIFT + left mouse button";
 
@@ -74,36 +75,60 @@ static GList *last_list = NULL;
 
 static guint32 streams_nb = 0;     /* number of displayed streams */
 
+#define NUM_COLS 12
+static const gchar *titles[NUM_COLS] =  {"Src IP addr", "Src port",  "Dest IP addr", "Dest port", "SSRC", "Payload", "Packets", "Lost", "Max Delta (ms)", "Max Jitter (ms)", "Mean Jitter (ms)", "Pb?"};
+
 /****************************************************************************/
 /* append a line to clist */
 static void add_to_clist(rtp_stream_info_t* strinfo)
 {
 	gchar label_text[256];
 	gint added_row;
-	gchar *data[8];
-	gchar field[8][30];
+	gchar *data[NUM_COLS];
+	guint32 expected;
+	gint32 lost;
+	double perc;
+	int i;
+	char *savelocale;
 
-	data[0]=&field[0][0];
-	data[1]=&field[1][0];
-	data[2]=&field[2][0];
-	data[3]=&field[3][0];
-	data[4]=&field[4][0];
-	data[5]=&field[5][0];
-	data[6]=&field[6][0];
-	data[7]=&field[7][0];
+	/* save the current locale */
+	savelocale = setlocale(LC_NUMERIC, NULL);
+	/* switch to "C" locale to avoid problems with localized decimal separators
+		in g_snprintf("%f") functions */
+	setlocale(LC_NUMERIC, "C");
 
-	g_snprintf(field[0], 20, "%s", address_to_str_w_none(&(strinfo->src_addr)));
-	g_snprintf(field[1], 20, "%u", strinfo->src_port);
-	g_snprintf(field[2], 20, "%s", address_to_str_w_none(&(strinfo->dest_addr)));
-	g_snprintf(field[3], 20, "%u", strinfo->dest_port);
-	g_snprintf(field[4], 20, "%u", strinfo->ssrc);
-	g_snprintf(field[5], 30, "%s", val_to_str(strinfo->pt, rtp_payload_type_vals,
+	data[0] = g_strdup(get_addr_name(&(strinfo->src_addr)));
+	data[1] = g_strdup_printf("%u", strinfo->src_port);
+	data[2] = g_strdup(get_addr_name(&(strinfo->dest_addr)));
+	data[3] = g_strdup_printf("%u", strinfo->dest_port);
+	data[4] = g_strdup_printf("%u", strinfo->ssrc);
+	data[5] = g_strdup(val_to_str(strinfo->pt, rtp_payload_type_vals,
 		"Unknown (%u)"));
-	g_snprintf(field[6], 20, "%u", strinfo->npackets);
-	/* XXX: Comment field is not used for the moment */
-/*	g_snprintf(field[7], 20, "%s", "");*/
+	data[6] = g_strdup_printf("%u", strinfo->npackets);
+
+	expected = (strinfo->rtp_stats.stop_seq_nr + strinfo->rtp_stats.cycles*65536)
+		- strinfo->rtp_stats.start_seq_nr + 1;
+	lost = expected - strinfo->rtp_stats.total_nr;
+	if (expected){
+		perc = (double)(lost*100)/(double)expected;
+	} else {
+		perc = 0;
+	}
+	data[7] = g_strdup_printf("%d (%.1f%%)", lost, perc);
+	data[8] = g_strdup_printf("%.2f", strinfo->rtp_stats.max_delta*1000);
+	data[9] = g_strdup_printf("%.2f", strinfo->rtp_stats.max_jitter*1000);
+	data[10] = g_strdup_printf("%.2f", strinfo->rtp_stats.mean_jitter*1000);
+	if (strinfo->problem)
+		data[11] = g_strdup("X");
+	else
+		data[11] = g_strdup("");
+
+	/* restore previous locale setting */
+	setlocale(LC_NUMERIC, savelocale);
 
 	added_row = gtk_clist_append(GTK_CLIST(clist), data);
+	for (i = 0; i < NUM_COLS; i++)
+		g_free(data[i]);
 
 	/* set data pointer of last row to point to user data for that row */
 	gtk_clist_set_row_data(GTK_CLIST(clist), added_row, strinfo);
@@ -189,7 +214,7 @@ rtpstream_on_unselect                  (GtkButton       *button _U_,
 
 
 /****************************************************************************/
-gint rtp_stream_info_cmp_reverse(gconstpointer aa, gconstpointer bb)
+static gint rtp_stream_info_cmp_reverse(gconstpointer aa, gconstpointer bb)
 {
 	const struct _rtp_stream_info* a = aa;
 	const struct _rtp_stream_info* b = bb;
@@ -263,7 +288,7 @@ rtpstream_on_goto                      (GtkButton       *button _U_,
 {
 	if (selected_stream_fwd)
 	{
-		goto_frame(&cfile, selected_stream_fwd->first_frame_num);
+		cf_goto_frame(&cfile, selected_stream_fwd->first_frame_num);
 	}
 }
 */
@@ -348,10 +373,10 @@ rtpstream_on_filter                    (GtkButton       *button _U_,
 		filter_string_fwd = g_strdup_printf(
 			"(ip%s.src==%s && udp.srcport==%u && ip%s.dst==%s && udp.dstport==%u && rtp.ssrc==%u)",
 			ip_version,
-			address_to_str_w_none(&(selected_stream_fwd->src_addr)),
+			address_to_str(&(selected_stream_fwd->src_addr)),
 			selected_stream_fwd->src_port,
 			ip_version,
-			address_to_str_w_none(&(selected_stream_fwd->dest_addr)),
+			address_to_str(&(selected_stream_fwd->dest_addr)),
 			selected_stream_fwd->dest_port,
 			selected_stream_fwd->ssrc);
         filter_string = filter_string_fwd;
@@ -368,10 +393,10 @@ rtpstream_on_filter                    (GtkButton       *button _U_,
 		filter_string_rev = g_strdup_printf(
 			"(ip%s.src==%s && udp.srcport==%u && ip%s.dst==%s && udp.dstport==%u && rtp.ssrc==%u)",
 			ip_version,
-			address_to_str_w_none(&(selected_stream_rev->src_addr)),
+			address_to_str(&(selected_stream_rev->src_addr)),
 			selected_stream_rev->src_port,
 			ip_version,
-			address_to_str_w_none(&(selected_stream_rev->dest_addr)),
+			address_to_str(&(selected_stream_rev->dest_addr)),
 			selected_stream_rev->dest_port,
 			selected_stream_rev->ssrc);
 
@@ -395,6 +420,41 @@ rtpstream_on_filter                    (GtkButton       *button _U_,
 }
 
 
+/****************************************************************************/
+#if (GTK_MAJOR_VERSION >= 2)
+static void
+rtpstream_on_copy_as_csv(GtkWindow *win _U_, gpointer data _U_)
+{
+	int             i,j;
+	gchar           *table_entry;
+	GString         *CSV_str;
+	GtkClipboard    *cb;
+
+	CSV_str = g_string_sized_new(240*(GTK_CLIST(clist)->rows+1));
+	/* Add the column headers to the CSV data */
+	for (j=0; j<NUM_COLS; j++) {
+		g_string_append(CSV_str, titles[j]);
+		g_string_append(CSV_str, ",");
+	}
+	g_string_append(CSV_str,"\n");
+
+	/* Add the column values to the CSV data */
+	for (i=0; i<GTK_CLIST(clist)->rows; i++) {
+		for (j=0; j<NUM_COLS; j++) {
+			gtk_clist_get_text(GTK_CLIST(clist),i,j,&table_entry);
+			g_string_append(CSV_str,table_entry);
+			g_string_append(CSV_str,",");
+		} 
+		g_string_append(CSV_str,"\n");
+	}
+
+	/* Now that we have the CSV data, copy it into the default clipboard */
+	cb = gtk_clipboard_get(GDK_SELECTION_CLIPBOARD);
+	gtk_clipboard_set_text(cb, CSV_str->str, CSV_str->len);
+	g_string_free(CSV_str, TRUE);
+}
+#endif
+
 /****************************************************************************/
 static void
 rtpstream_on_analyse                   (GtkButton       *button _U_,
@@ -410,6 +470,11 @@ rtpstream_on_analyse                   (GtkButton       *button _U_,
 	address ip_dst_rev;
 	guint16 port_dst_rev = 0;
 	guint32 ssrc_rev = 0;
+
+	if (!(selected_stream_fwd || selected_stream_rev))
+	{
+		return;
+	}
 	
 	SET_ADDRESS(&ip_src_fwd,AT_NONE,0,NULL);
 	SET_ADDRESS(&ip_dst_fwd,AT_NONE,0,NULL);
@@ -463,9 +528,9 @@ rtpstream_on_select_row(GtkCList *clist,
 	if (event==NULL || event->state & GDK_SHIFT_MASK) {
 		selected_stream_rev = gtk_clist_get_row_data(GTK_CLIST(clist), row);
 		g_snprintf(label_text, 80, "Reverse: %s:%u -> %s:%u, SSRC=%u",
-			address_to_str_w_none(&(selected_stream_rev->src_addr)),
+			get_addr_name(&(selected_stream_rev->src_addr)),
 			selected_stream_rev->src_port,
-			address_to_str_w_none(&(selected_stream_rev->dest_addr)),
+			get_addr_name(&(selected_stream_rev->dest_addr)),
 			selected_stream_rev->dest_port,
 			selected_stream_rev->ssrc
 		);
@@ -474,9 +539,9 @@ rtpstream_on_select_row(GtkCList *clist,
 	else {
 		selected_stream_fwd = gtk_clist_get_row_data(GTK_CLIST(clist), row);
 		g_snprintf(label_text, 80, "Forward: %s:%u -> %s:%u, SSRC=%u",
-			address_to_str_w_none(&(selected_stream_fwd->src_addr)),
+			get_addr_name(&(selected_stream_fwd->src_addr)),
 			selected_stream_fwd->src_port,
-			address_to_str_w_none(&(selected_stream_fwd->dest_addr)),
+			get_addr_name(&(selected_stream_fwd->dest_addr)),
 			selected_stream_fwd->dest_port,
 			selected_stream_fwd->ssrc
 		);
@@ -493,8 +558,6 @@ rtpstream_on_select_row(GtkCList *clist,
 
 
 /****************************************************************************/
-#define NUM_COLS 7
-
 typedef struct column_arrows {
 	GtkWidget *table;
 	GtkWidget *ascend_pm;
@@ -553,12 +616,16 @@ rtpstream_sort_column(GtkCList *clist, gconstpointer ptr1, gconstpointer ptr2)
 	case 0:
 	case 2:
 	case 5:
-	case 7:
+	case 11:
 		return strcmp (text1, text2);
 	case 1:
 	case 3:
 	case 4:
 	case 6:
+	case 7:
+	case 8:
+	case 9:
+	case 10:
 		i1=atoi(text1);
 		i2=atoi(text2);
 		return i1-i2;
@@ -574,157 +641,179 @@ rtpstream_sort_column(GtkCList *clist, gconstpointer ptr1, gconstpointer ptr2)
 
 static void rtpstream_dlg_create (void)
 {
-	GtkWidget *rtpstream_dlg_w;
-	GtkWidget *main_vb;
-	GtkWidget *scrolledwindow;
-	GtkWidget *hbuttonbox;
-/*	GtkWidget *bt_goto;*/
-	GtkWidget *bt_unselect;
-	GtkWidget *bt_findrev;
-	GtkWidget *bt_save;
-	GtkWidget *bt_mark;
-	GtkWidget *bt_filter;
-	GtkWidget *bt_analyze;
-	GtkWidget *bt_close;
+    GtkWidget *rtpstream_dlg_w;
+    GtkWidget *main_vb;
+    GtkWidget *scrolledwindow;
+    GtkWidget *hbuttonbox;
+/*    GtkWidget *bt_goto;*/
+    GtkWidget *bt_unselect;
+    GtkWidget *bt_findrev;
+    GtkWidget *bt_save;
+    GtkWidget *bt_mark;
+    GtkWidget *bt_filter;
+    GtkWidget *bt_analyze;
+    GtkWidget *bt_close;
+#if (GTK_MAJOR_VERSION >= 2)
+    GtkWidget *bt_copy;
+#endif           
     GtkTooltips *tooltips = gtk_tooltips_new();
 
-	gchar *titles[8] =  {"Src IP addr", "Src port",  "Dest IP addr", "Dest port", "SSRC", "Payload", "Packets", "Comment"};
-	column_arrows *col_arrows;
-	GtkWidget *column_lb;
-	int i;
+    column_arrows *col_arrows;
+    GtkWidget *column_lb;
+    int i;
 
-	rtpstream_dlg_w = dlg_window_new("Ethereal: RTP Streams");
-	gtk_window_set_default_size(GTK_WINDOW(rtpstream_dlg_w), 620, 200);
+    rtpstream_dlg_w = dlg_window_new("Ethereal: RTP Streams");
+    gtk_window_set_default_size(GTK_WINDOW(rtpstream_dlg_w), 620, 200);
 
-	main_vb = gtk_vbox_new (FALSE, 0);
-	gtk_container_add(GTK_CONTAINER(rtpstream_dlg_w), main_vb);
-	gtk_container_set_border_width (GTK_CONTAINER (main_vb), 12);
+    main_vb = gtk_vbox_new (FALSE, 0);
+    gtk_container_add(GTK_CONTAINER(rtpstream_dlg_w), main_vb);
+    gtk_container_set_border_width (GTK_CONTAINER (main_vb), 12);
 
-	top_label = gtk_label_new ("Detected 0 RTP streams. Choose one for forward and reverse direction for analysis");
-	gtk_box_pack_start (GTK_BOX (main_vb), top_label, FALSE, FALSE, 8);
+    top_label = gtk_label_new ("Detected 0 RTP streams. Choose one for forward and reverse direction for analysis");
+    gtk_box_pack_start (GTK_BOX (main_vb), top_label, FALSE, FALSE, 8);
 
-	scrolledwindow = scrolled_window_new (NULL, NULL);
-	gtk_box_pack_start (GTK_BOX (main_vb), scrolledwindow, TRUE, TRUE, 0);
+    scrolledwindow = scrolled_window_new (NULL, NULL);
+    gtk_box_pack_start (GTK_BOX (main_vb), scrolledwindow, TRUE, TRUE, 0);
 
-	clist = gtk_clist_new (NUM_COLS);
-	gtk_container_add (GTK_CONTAINER (scrolledwindow), clist);
+    clist = gtk_clist_new (NUM_COLS);
+    gtk_container_add (GTK_CONTAINER (scrolledwindow), clist);
 
-	gtk_clist_set_column_width (GTK_CLIST (clist), 0, 100);
-	gtk_clist_set_column_width (GTK_CLIST (clist), 1, 50);
-	gtk_clist_set_column_width (GTK_CLIST (clist), 2, 100);
-	gtk_clist_set_column_width (GTK_CLIST (clist), 3, 50);
-	gtk_clist_set_column_width (GTK_CLIST (clist), 4, 80);
-	gtk_clist_set_column_width (GTK_CLIST (clist), 5, 118);
-	gtk_clist_set_column_width (GTK_CLIST (clist), 6, 40);
-/*	gtk_clist_set_column_width (GTK_CLIST (clist), 7, 51);*/
+    gtk_clist_set_column_width (GTK_CLIST (clist), 0, 88);
+    gtk_clist_set_column_width (GTK_CLIST (clist), 1, 44);
+    gtk_clist_set_column_width (GTK_CLIST (clist), 2, 88);
+    gtk_clist_set_column_width (GTK_CLIST (clist), 3, 44);
+    gtk_clist_set_column_width (GTK_CLIST (clist), 4, 64);
+    gtk_clist_set_column_width (GTK_CLIST (clist), 5, 96);
+    gtk_clist_set_column_width (GTK_CLIST (clist), 6, 50);
+    gtk_clist_set_column_width (GTK_CLIST (clist), 7, 50);
+    gtk_clist_set_column_width (GTK_CLIST (clist), 8, 80);
+    gtk_clist_set_column_width (GTK_CLIST (clist), 9, 80);
+    gtk_clist_set_column_width (GTK_CLIST (clist), 10, 80);
+    gtk_clist_set_column_width (GTK_CLIST (clist), 11, 40);
 
-	gtk_clist_set_column_justification(GTK_CLIST(clist), 0, GTK_JUSTIFY_CENTER);
-	gtk_clist_set_column_justification(GTK_CLIST(clist), 1, GTK_JUSTIFY_CENTER);
-	gtk_clist_set_column_justification(GTK_CLIST(clist), 2, GTK_JUSTIFY_CENTER);
-	gtk_clist_set_column_justification(GTK_CLIST(clist), 3, GTK_JUSTIFY_CENTER);
-	gtk_clist_set_column_justification(GTK_CLIST(clist), 4, GTK_JUSTIFY_CENTER);
-	gtk_clist_set_column_justification(GTK_CLIST(clist), 5, GTK_JUSTIFY_LEFT);
-	gtk_clist_set_column_justification(GTK_CLIST(clist), 6, GTK_JUSTIFY_RIGHT);
-/*	gtk_clist_set_column_justification(GTK_CLIST(clist), 7, GTK_JUSTIFY_CENTER);*/
+    gtk_clist_set_column_justification(GTK_CLIST(clist), 0, GTK_JUSTIFY_CENTER);
+    gtk_clist_set_column_justification(GTK_CLIST(clist), 1, GTK_JUSTIFY_CENTER);
+    gtk_clist_set_column_justification(GTK_CLIST(clist), 2, GTK_JUSTIFY_CENTER);
+    gtk_clist_set_column_justification(GTK_CLIST(clist), 3, GTK_JUSTIFY_CENTER);
+    gtk_clist_set_column_justification(GTK_CLIST(clist), 4, GTK_JUSTIFY_CENTER);
+    gtk_clist_set_column_justification(GTK_CLIST(clist), 5, GTK_JUSTIFY_LEFT);
+    gtk_clist_set_column_justification(GTK_CLIST(clist), 6, GTK_JUSTIFY_RIGHT);
+    gtk_clist_set_column_justification(GTK_CLIST(clist), 7, GTK_JUSTIFY_RIGHT);
+    gtk_clist_set_column_justification(GTK_CLIST(clist), 8, GTK_JUSTIFY_RIGHT);
+    gtk_clist_set_column_justification(GTK_CLIST(clist), 9, GTK_JUSTIFY_RIGHT);
+    gtk_clist_set_column_justification(GTK_CLIST(clist), 10, GTK_JUSTIFY_RIGHT);
+    gtk_clist_set_column_justification(GTK_CLIST(clist), 11, GTK_JUSTIFY_LEFT);
 
-	gtk_clist_column_titles_show (GTK_CLIST (clist));
+    gtk_clist_column_titles_show (GTK_CLIST (clist));
 
-	gtk_clist_set_compare_func(GTK_CLIST(clist), rtpstream_sort_column);
-	gtk_clist_set_sort_column(GTK_CLIST(clist), 0);
-	gtk_clist_set_sort_type(GTK_CLIST(clist), GTK_SORT_ASCENDING);
+    gtk_clist_set_compare_func(GTK_CLIST(clist), rtpstream_sort_column);
+    gtk_clist_set_sort_column(GTK_CLIST(clist), 0);
+    gtk_clist_set_sort_type(GTK_CLIST(clist), GTK_SORT_ASCENDING);
 
-	gtk_widget_show(rtpstream_dlg_w);
+    gtk_widget_show(rtpstream_dlg_w);
 
-	/* sort by column feature */
-	col_arrows = (column_arrows *) g_malloc(sizeof(column_arrows) * NUM_COLS);
+    /* sort by column feature */
+    col_arrows = (column_arrows *) g_malloc(sizeof(column_arrows) * NUM_COLS);
 
-	for (i=0; i<NUM_COLS; i++) {
-		col_arrows[i].table = gtk_table_new(2, 2, FALSE);
-		gtk_table_set_col_spacings(GTK_TABLE(col_arrows[i].table), 5);
-		column_lb = gtk_label_new(titles[i]);
-		gtk_table_attach(GTK_TABLE(col_arrows[i].table), column_lb, 0, 1, 0, 2, GTK_SHRINK, GTK_SHRINK, 0, 0);
-		gtk_widget_show(column_lb);
+    for (i=0; i<NUM_COLS; i++) {
+        col_arrows[i].table = gtk_table_new(2, 2, FALSE);
+        gtk_table_set_col_spacings(GTK_TABLE(col_arrows[i].table), 5);
+        column_lb = gtk_label_new(titles[i]);
+        gtk_table_attach(GTK_TABLE(col_arrows[i].table), column_lb, 0, 1, 0, 2, GTK_SHRINK, GTK_SHRINK, 0, 0);
+        gtk_widget_show(column_lb);
 
-		col_arrows[i].ascend_pm = xpm_to_widget(clist_ascend_xpm);
-		gtk_table_attach(GTK_TABLE(col_arrows[i].table), col_arrows[i].ascend_pm, 1, 2, 1, 2, GTK_SHRINK, GTK_SHRINK, 0, 0);
-		col_arrows[i].descend_pm = xpm_to_widget(clist_descend_xpm);
-		gtk_table_attach(GTK_TABLE(col_arrows[i].table), col_arrows[i].descend_pm, 1, 2, 0, 1, GTK_SHRINK, GTK_SHRINK, 0, 0);
-		/* make src-ip be the default sort order */
-		if (i == 0) {
-			gtk_widget_show(col_arrows[i].ascend_pm);
-		}
-		gtk_clist_set_column_widget(GTK_CLIST(clist), i, col_arrows[i].table);
-		gtk_widget_show(col_arrows[i].table);
-	}
+        col_arrows[i].ascend_pm = xpm_to_widget(clist_ascend_xpm);
+        gtk_table_attach(GTK_TABLE(col_arrows[i].table), col_arrows[i].ascend_pm, 1, 2, 1, 2, GTK_SHRINK, GTK_SHRINK, 0, 0);
+        col_arrows[i].descend_pm = xpm_to_widget(clist_descend_xpm);
+        gtk_table_attach(GTK_TABLE(col_arrows[i].table), col_arrows[i].descend_pm, 1, 2, 0, 1, GTK_SHRINK, GTK_SHRINK, 0, 0);
+        /* make src-ip be the default sort order */
+        if (i == 0) {
+            gtk_widget_show(col_arrows[i].ascend_pm);
+        }
+        gtk_clist_set_column_widget(GTK_CLIST(clist), i, col_arrows[i].table);
+        gtk_widget_show(col_arrows[i].table);
+    }
 
-	SIGNAL_CONNECT(clist, "click-column", rtpstream_click_column_cb, col_arrows);
+    SIGNAL_CONNECT(clist, "click-column", rtpstream_click_column_cb, col_arrows);
 
-	label_fwd = gtk_label_new (FWD_LABEL_TEXT);
-	gtk_box_pack_start (GTK_BOX (main_vb), label_fwd, FALSE, FALSE, 0);
+    label_fwd = gtk_label_new (FWD_LABEL_TEXT);
+    gtk_box_pack_start (GTK_BOX (main_vb), label_fwd, FALSE, FALSE, 0);
 
-	label_rev = gtk_label_new (REV_LABEL_TEXT);
-	gtk_box_pack_start (GTK_BOX (main_vb), label_rev, FALSE, FALSE, 0);
+    label_rev = gtk_label_new (REV_LABEL_TEXT);
+    gtk_box_pack_start (GTK_BOX (main_vb), label_rev, FALSE, FALSE, 0);
 
     /* button row */
-	hbuttonbox = gtk_hbutton_box_new ();
-	gtk_box_pack_start (GTK_BOX (main_vb), hbuttonbox, FALSE, FALSE, 0);
-	gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox), GTK_BUTTONBOX_END);
-	gtk_button_box_set_spacing (GTK_BUTTON_BOX (hbuttonbox), 0);
+    hbuttonbox = gtk_hbutton_box_new ();
+    gtk_box_pack_start (GTK_BOX (main_vb), hbuttonbox, FALSE, FALSE, 0);
+    gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox), GTK_BUTTONBOX_END);
+    gtk_button_box_set_spacing (GTK_BUTTON_BOX (hbuttonbox), 0);
 
-	bt_unselect = gtk_button_new_with_label ("Unselect");
-	gtk_container_add (GTK_CONTAINER (hbuttonbox), bt_unselect);
+    bt_unselect = gtk_button_new_with_label ("Unselect");
+    gtk_container_add (GTK_CONTAINER (hbuttonbox), bt_unselect);
     gtk_tooltips_set_tip (tooltips, bt_unselect, "Undo stream selection", NULL);
 
-	bt_findrev = gtk_button_new_with_label ("Find Reverse");
-	gtk_container_add (GTK_CONTAINER (hbuttonbox), bt_findrev);
+    bt_findrev = gtk_button_new_with_label ("Find Reverse");
+    gtk_container_add (GTK_CONTAINER (hbuttonbox), bt_findrev);
     gtk_tooltips_set_tip (tooltips, bt_findrev, "Find the reverse stream matching the selected forward stream", NULL);
 /*
-	bt_goto = BUTTON_NEW_FROM_STOCK(GTK_STOCK_JUMP_TO);
-	gtk_container_add (GTK_CONTAINER (hbuttonbox), bt_goto);
+    bt_goto = BUTTON_NEW_FROM_STOCK(GTK_STOCK_JUMP_TO);
+    gtk_container_add (GTK_CONTAINER (hbuttonbox), bt_goto);
 */
-	bt_save = BUTTON_NEW_FROM_STOCK(GTK_STOCK_SAVE_AS);
-	gtk_container_add (GTK_CONTAINER (hbuttonbox), bt_save);
+    bt_save = BUTTON_NEW_FROM_STOCK(GTK_STOCK_SAVE_AS);
+    gtk_container_add (GTK_CONTAINER (hbuttonbox), bt_save);
     gtk_tooltips_set_tip (tooltips, bt_save, "Save stream payload in rtpdump format", NULL);
 
-	bt_mark = gtk_button_new_with_label ("Mark packets");
-	gtk_container_add (GTK_CONTAINER (hbuttonbox), bt_mark);
+    bt_mark = gtk_button_new_with_label ("Mark Packets");
+    gtk_container_add (GTK_CONTAINER (hbuttonbox), bt_mark);
     gtk_tooltips_set_tip (tooltips, bt_mark, "Mark packets of the selected stream(s)", NULL);
 
-	bt_filter = gtk_button_new_with_label ("Prepare filter");
-	gtk_container_add (GTK_CONTAINER (hbuttonbox), bt_filter);
+    bt_filter = gtk_button_new_with_label ("Prepare Filter");
+    gtk_container_add (GTK_CONTAINER (hbuttonbox), bt_filter);
     gtk_tooltips_set_tip (tooltips, bt_filter, "Prepare a display filter of the selected stream(s)", NULL);
 
-	bt_analyze = gtk_button_new_with_label ("Analyze");
-	gtk_container_add (GTK_CONTAINER (hbuttonbox), bt_analyze);
+#if (GTK_MAJOR_VERSION >= 2)
+    /* XXX - maybe we want to have a "Copy as CSV" stock button here? */
+    /*bt_copy = gtk_button_new_with_label ("Copy content to clipboard as CSV");*/
+    bt_copy = BUTTON_NEW_FROM_STOCK(GTK_STOCK_COPY);
+    gtk_container_add (GTK_CONTAINER (hbuttonbox), bt_copy);
+    gtk_tooltips_set_tip(tooltips, bt_copy, 
+        "Copy all statistical values of this page to the clipboard in CSV (Comma Seperated Values) format.", NULL);
+#endif                 
+
+    bt_analyze = gtk_button_new_with_label ("Analyze");
+    gtk_container_add (GTK_CONTAINER (hbuttonbox), bt_analyze);
     gtk_tooltips_set_tip (tooltips, bt_analyze, "Open an analyze window of the selected stream(s)", NULL);
 
-	bt_close = BUTTON_NEW_FROM_STOCK(GTK_STOCK_CLOSE);
-	gtk_container_add (GTK_CONTAINER (hbuttonbox), bt_close);
+    bt_close = BUTTON_NEW_FROM_STOCK(GTK_STOCK_CLOSE);
+    gtk_container_add (GTK_CONTAINER (hbuttonbox), bt_close);
     gtk_tooltips_set_tip (tooltips, bt_close, "Close this dialog", NULL);
-	GTK_WIDGET_SET_FLAGS(bt_close, GTK_CAN_DEFAULT);
+    GTK_WIDGET_SET_FLAGS(bt_close, GTK_CAN_DEFAULT);
 
-	SIGNAL_CONNECT(clist, "select_row", rtpstream_on_select_row, NULL);
-	SIGNAL_CONNECT(bt_unselect, "clicked", rtpstream_on_unselect, NULL);
-	SIGNAL_CONNECT(bt_findrev, "clicked", rtpstream_on_findrev, NULL);
+    SIGNAL_CONNECT(clist, "select_row", rtpstream_on_select_row, NULL);
+    SIGNAL_CONNECT(bt_unselect, "clicked", rtpstream_on_unselect, NULL);
+    SIGNAL_CONNECT(bt_findrev, "clicked", rtpstream_on_findrev, NULL);
 /*
-	SIGNAL_CONNECT(bt_goto, "clicked", rtpstream_on_goto, NULL);
+    SIGNAL_CONNECT(bt_goto, "clicked", rtpstream_on_goto, NULL);
 */
-	SIGNAL_CONNECT(bt_save, "clicked", rtpstream_on_save, NULL);
-	SIGNAL_CONNECT(bt_mark, "clicked", rtpstream_on_mark, NULL);
-	SIGNAL_CONNECT(bt_filter, "clicked", rtpstream_on_filter, NULL);
-	SIGNAL_CONNECT(bt_analyze, "clicked", rtpstream_on_analyse, NULL);
+    SIGNAL_CONNECT(bt_save, "clicked", rtpstream_on_save, NULL);
+    SIGNAL_CONNECT(bt_mark, "clicked", rtpstream_on_mark, NULL);
+    SIGNAL_CONNECT(bt_filter, "clicked", rtpstream_on_filter, NULL);
+#if (GTK_MAJOR_VERSION >= 2)
+    SIGNAL_CONNECT(bt_copy, "clicked", rtpstream_on_copy_as_csv, NULL);
+#endif
+    SIGNAL_CONNECT(bt_analyze, "clicked", rtpstream_on_analyse, NULL);
 
-	window_set_cancel_button(rtpstream_dlg_w, bt_close, window_cancel_button_cb);
+    window_set_cancel_button(rtpstream_dlg_w, bt_close, window_cancel_button_cb);
 
-	SIGNAL_CONNECT(rtpstream_dlg_w, "delete_event", window_delete_event_cb, NULL);
-	SIGNAL_CONNECT(rtpstream_dlg_w, "destroy", rtpstream_on_destroy, NULL);
+    SIGNAL_CONNECT(rtpstream_dlg_w, "delete_event", window_delete_event_cb, NULL);
+    SIGNAL_CONNECT(rtpstream_dlg_w, "destroy", rtpstream_on_destroy, NULL);
 
-	gtk_widget_show_all(rtpstream_dlg_w);
-	window_present(rtpstream_dlg_w);
+    gtk_widget_show_all(rtpstream_dlg_w);
+    window_present(rtpstream_dlg_w);
 
-	rtpstream_on_unselect(NULL, NULL);
+    rtpstream_on_unselect(NULL, NULL);
 
-	rtp_stream_dlg = rtpstream_dlg_w;
+    rtp_stream_dlg = rtpstream_dlg_w;
 }
 
 
@@ -778,7 +867,7 @@ void rtpstream_dlg_show(GList *list)
 
 /****************************************************************************/
 /* entry point when called via the GTK menu */
-void rtpstream_launch(GtkWidget *w _U_, gpointer data _U_)
+static void rtpstream_launch(GtkWidget *w _U_, gpointer data _U_)
 {
 	/* Register the tap listener */
 	register_tap_listener_rtp_stream();
@@ -796,6 +885,6 @@ void rtpstream_launch(GtkWidget *w _U_, gpointer data _U_)
 void
 register_tap_listener_rtp_stream_dlg(void)
 {
-	register_tap_menu_item("RTP/Show All Streams...", REGISTER_TAP_GROUP_NONE,
+	register_stat_menu_item("RTP/Show All Streams", REGISTER_STAT_GROUP_TELEPHONY,
 	    rtpstream_launch, NULL, NULL, NULL);
 }
