@@ -1,7 +1,7 @@
-/* 
+/*
  * Copyright 2004, Irene Ruengeler <i.ruengeler [AT] fh-muenster.de>
  *
- * $Id: sctp_graph_dlg.c 14252 2005-05-01 13:51:07Z tuexen $
+ * $Id: sctp_graph_dlg.c 17778 2006-03-30 22:17:42Z tuexen $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -21,13 +21,14 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
- 
+
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
 
 
 #include <gtk/gtk.h>
+#include <gdk/gdk.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <math.h>
@@ -35,7 +36,6 @@
 #include "globals.h"
 #include "epan/filesystem.h"
 #include "../color.h"
-#include "tap_menu.h"
 #include "dlg_utils.h"
 #include "ui_util.h"
 #include "main.h"
@@ -61,8 +61,7 @@
 #define BOTTOM_BORDER 50
 
 #define SUB_32(a, b)	a-b
-#define MINI(a,b) (a<b)?a:b
-#define MAXI(a,b) (a>b)?a:b
+#define POINT_SIZE	3
 
 struct chunk_header {
 	guint8  type;
@@ -108,6 +107,8 @@ struct gaps {
 };
 
 
+static gboolean label_set = FALSE;
+static guint32 max_tsn=0, min_tsn=0;
 static void sctp_graph_set_title(struct sctp_udata *u_data);
 static void create_draw_area(GtkWidget *box, struct sctp_udata *u_data);
 
@@ -118,7 +119,8 @@ static void draw_sack_graph(struct sctp_udata *u_data)
 	GList *list=NULL, *tlist;
 	guint16 gap_start=0, gap_end=0, i, j, nr;
 	guint8 type;
-	guint32 tsnumber, max_tsn=0, min_tsn=0;
+	guint32 tsnumber;
+	gint xvalue, yvalue;
 	GdkColor red_color = {0, 65535, 0, 0};
 	GdkColor green_color = {0, 0, 65535, 0};
 	GdkGC *red_gc, *green_gc;
@@ -198,39 +200,44 @@ static void draw_sack_graph(struct sctp_udata *u_data)
 				sack_header =(struct sack_chunk_header *)tlist->data;
 				nr=ntohs(sack_header->nr_of_gaps);
 				tsnumber = ntohl(sack_header->cum_tsn_ack);
-				if (nr>0)
-				{
-					gap = (struct gaps *)(sack_header->tsns+(4*(nr-1)));
-					gap_start=ntohs(gap->start);
-					gap_end = ntohs(gap->end);
-					max_num=gap_end+tsnumber;
-				}
-				else
-					max_num=tsnumber;
 
 				if (sack->secs>=u_data->io->x1_tmp_sec)
 				{
 					if (nr>0)
 					{
+						gap = (struct gaps *)(&(sack_header->tsns));
 						for(i=0;i<nr; i++)
 						{
+							gap_start=ntohs(gap->start);
+							gap_end = ntohs(gap->end);
+							max_num=gap_end+tsnumber;
 							for (j=gap_start; j<=gap_end; j++)
 							{
-								diff=sack->secs*1000000+sack->usecs-u_data->io->min_x;
-								gdk_draw_arc(u_data->io->pixmap,green_gc,TRUE,
-								             (guint32)(LEFT_BORDER+u_data->io->offset+u_data->io->x_interval*diff),
-								             (guint32)(u_data->io->pixmap_height-BOTTOM_BORDER-u_data->io->offset-((SUB_32(j+tsnumber,min_tsn))*u_data->io->y_interval)),
-								             3, 3,0, (64*360) );
+								diff = sack->secs*1000000+sack->usecs-u_data->io->min_x;
+								xvalue = (guint32)(LEFT_BORDER+u_data->io->offset+u_data->io->x_interval*diff);
+								yvalue = (guint32)(u_data->io->pixmap_height-BOTTOM_BORDER-POINT_SIZE-u_data->io->offset-((SUB_32(j+tsnumber,min_tsn))*u_data->io->y_interval));
+								if (xvalue >= LEFT_BORDER+u_data->io->offset &&
+								    xvalue <= u_data->io->pixmap_width-RIGHT_BORDER+u_data->io->offset &&
+								    yvalue >= TOP_BORDER-u_data->io->offset &&
+								    yvalue <= u_data->io->pixmap_height-BOTTOM_BORDER-u_data->io->offset)
+									gdk_draw_arc(u_data->io->pixmap,green_gc,TRUE,
+								             	    xvalue,
+								                    yvalue,
+								             	    POINT_SIZE, POINT_SIZE,0, (64*360) );
 							}
+							if (i < nr-1)
+								gap++;
 						}
 					}
+					else
+						max_num=tsnumber;
 					if (tsnumber>=min_tsn)
 					{
 						diff=sack->secs*1000000+sack->usecs-u_data->io->min_x;
 						gdk_draw_arc(u_data->io->pixmap,red_gc,TRUE,
 						             (guint32)(LEFT_BORDER+u_data->io->offset+u_data->io->x_interval*diff),
-						             (guint32)(u_data->io->pixmap_height-BOTTOM_BORDER-u_data->io->offset-((SUB_32(tsnumber,min_tsn))*u_data->io->y_interval)),
-						             3, 3,0, (64*360) );
+						             (guint32)(u_data->io->pixmap_height-BOTTOM_BORDER-POINT_SIZE-u_data->io->offset-((SUB_32(tsnumber,min_tsn))*u_data->io->y_interval)),
+						             POINT_SIZE, POINT_SIZE,0, (64*360) );
 					}
 				}
 			tlist = g_list_next(tlist);
@@ -250,7 +257,7 @@ static void draw_tsn_graph(struct sctp_udata *u_data)
 	tsn_t *tsn;
 	GList *list=NULL, *tlist;
 	guint8 type;
-	guint32 tsnumber=0, min_tsn=0, max_tsn=0;
+	guint32 tsnumber=0;
 	guint32 min_secs=0, diff;
 
 	if (u_data->dir==1)
@@ -297,8 +304,8 @@ static void draw_tsn_graph(struct sctp_udata *u_data)
 					diff=tsn->secs*1000000+tsn->usecs-u_data->io->min_x;
 					gdk_draw_arc(u_data->io->pixmap,u_data->io->draw_area->style->black_gc,TRUE,
 					             (guint32)(LEFT_BORDER+u_data->io->offset+u_data->io->x_interval*diff),
-					             (guint32)(u_data->io->pixmap_height-BOTTOM_BORDER-u_data->io->offset-((SUB_32(tsnumber,min_tsn))*u_data->io->y_interval)),
-					             3, 3, 0, (64*360));
+					             (guint32)(u_data->io->pixmap_height-BOTTOM_BORDER-POINT_SIZE-u_data->io->offset-((SUB_32(tsnumber,min_tsn))*u_data->io->y_interval)),
+					             POINT_SIZE, POINT_SIZE, 0, (64*360));
 			}
 			tlist = g_list_next(tlist);
 		}
@@ -310,7 +317,8 @@ static void draw_tsn_graph(struct sctp_udata *u_data)
 static void sctp_graph_draw(struct sctp_udata *u_data)
 {
 	int length, lwidth, j, b;
-	guint32 label_width, label_height, distance=5, i, e, sec, w, start, a;
+	guint32  distance=5, i, e, sec, w, start, a;
+	gint label_width, label_height;
 	char label_string[15];
 	gfloat dis;
 
@@ -376,7 +384,12 @@ static void sctp_graph_draw(struct sctp_udata *u_data)
 	gdk_draw_line(u_data->io->pixmap,u_data->io->draw_area->style->black_gc,u_data->io->pixmap_width-RIGHT_BORDER+u_data->io->offset, u_data->io->pixmap_height-BOTTOM_BORDER, u_data->io->pixmap_width-RIGHT_BORDER+u_data->io->offset-5, u_data->io->pixmap_height-BOTTOM_BORDER+5);
 	u_data->io->axis_width=u_data->io->pixmap_width-LEFT_BORDER-RIGHT_BORDER-u_data->io->offset;
 
-	u_data->io->x_interval = (float)((u_data->io->axis_width*1.0)/u_data->io->tmp_width);
+	/* try to avoid dividing by zero */
+	if(u_data->io->tmp_width>0){
+		u_data->io->x_interval = (float)((u_data->io->axis_width*1.0)/u_data->io->tmp_width);
+	} else {
+		u_data->io->x_interval = (float)(u_data->io->axis_width);
+	}
 
 	e=0;
 	if (u_data->io->x_interval<1)
@@ -418,13 +431,13 @@ static void sctp_graph_draw(struct sctp_udata *u_data)
 	if (u_data->io->offset!=0)
 	{
 		g_snprintf(label_string, 15, "%u", u_data->io->x1_tmp_sec);
-		
+
 #if GTK_MAJOR_VERSION < 2
 		lwidth=gdk_string_width(font, label_string);
-		                        gdk_draw_string(u_data->io->pixmap,font,u_data->io->draw_area->style->black_gc,
-		                        LEFT_BORDER-10,
-		                        u_data->io->pixmap_height-BOTTOM_BORDER+20,
-		                        label_string);
+		gdk_draw_string(u_data->io->pixmap,font,u_data->io->draw_area->style->black_gc,
+		                LEFT_BORDER-10,
+		                u_data->io->pixmap_height-BOTTOM_BORDER+20,
+		                label_string);
 #else
 		memcpy(label_string,(gchar *)g_locale_to_utf8(label_string, -1 , NULL, NULL, NULL), 15);
 		pango_layout_set_text(layout, label_string, -1);
@@ -585,7 +598,7 @@ static void sctp_graph_draw(struct sctp_udata *u_data)
 						lwidth=gdk_string_width(font, label_string);
 						gdk_draw_string(u_data->io->pixmap,font,u_data->io->draw_area->style->black_gc,
 						                LEFT_BORDER-length-lwidth-5,
-						                (guint32)(u_data->io->pixmap_height-BOTTOM_BORDER-u_data->io->offset-(i-u_data->io->min_y)*u_data->io->y_interval-3),
+						                (guint32)(u_data->io->pixmap_height-BOTTOM_BORDER-u_data->io->offset-(i-u_data->io->min_y)*u_data->io->y_interval-POINT_SIZE),
 						                label_string);
 #else
 						memcpy(label_string,(gchar *)g_locale_to_utf8(label_string, -1 , NULL, NULL, NULL), 15);
@@ -593,7 +606,7 @@ static void sctp_graph_draw(struct sctp_udata *u_data)
 						pango_layout_get_pixel_size(layout, &lwidth, NULL);
 						gdk_draw_layout(u_data->io->pixmap,u_data->io->draw_area->style->black_gc,
 						                LEFT_BORDER-length-lwidth-5,
-						                (guint32)(u_data->io->pixmap_height-BOTTOM_BORDER-u_data->io->offset-(i-u_data->io->min_y)*u_data->io->y_interval-3),
+						                (guint32)(u_data->io->pixmap_height-BOTTOM_BORDER-u_data->io->offset-(i-u_data->io->min_y)*u_data->io->y_interval-POINT_SIZE),
 						                layout);
 #endif
 				}
@@ -622,7 +635,7 @@ sctp_graph_t *ios;
 			draw_sack_graph(u_data);
 			draw_tsn_graph(u_data);
 			break;
-		case 1: 
+		case 1:
 			draw_tsn_graph(u_data);
 			break;
 		case 2:
@@ -707,7 +720,6 @@ configure_event(GtkWidget *widget, GdkEventConfigure *event _U_, struct sctp_uda
 			widget->allocation.width,
 			widget->allocation.height);
 	sctp_graph_redraw(u_data);
-	sctp_graph_redraw(u_data);
 	return TRUE;
 }
 
@@ -737,7 +749,7 @@ on_zoomin_bt (GtkWidget *widget _U_, struct sctp_udata *u_data)
 {
 	sctp_min_max_t *tmp_minmax;
 
-	if (u_data->io->rectangle==TRUE)
+	if (u_data->io->rectangle_present==TRUE)
 	{
 		tmp_minmax = g_malloc(sizeof(sctp_min_max_t));
 
@@ -757,10 +769,41 @@ on_zoomin_bt (GtkWidget *widget _U_, struct sctp_udata *u_data)
 		u_data->io->length = g_slist_length(u_data->assoc->min_max);
 		u_data->io->tmp=TRUE;
 		u_data->io->rectangle=FALSE;
+		u_data->io->rectangle_present=FALSE;
 		sctp_graph_redraw(u_data);
 	}
 }
 
+static void
+zoomin_bt (struct sctp_udata *u_data)
+{
+	sctp_min_max_t *tmp_minmax;
+
+	tmp_minmax = g_malloc(sizeof(sctp_min_max_t));
+
+	u_data->io->tmp_min_tsn1=u_data->io->y1_tmp+u_data->io->min_y;
+	u_data->io->tmp_max_tsn1=u_data->io->y2_tmp+1+u_data->io->min_y;
+	u_data->io->tmp_min_tsn2=u_data->io->tmp_min_tsn1;
+	u_data->io->tmp_max_tsn2=u_data->io->tmp_max_tsn1;
+	tmp_minmax->tmp_min_secs=u_data->io->x1_tmp_sec;
+	tmp_minmax->tmp_min_usecs=	u_data->io->x1_tmp_usec;
+	tmp_minmax->tmp_max_secs=	u_data->io->x2_tmp_sec;
+	tmp_minmax->tmp_max_usecs=	u_data->io->x2_tmp_usec;
+	tmp_minmax->tmp_min_tsn1=u_data->io->tmp_min_tsn1;
+	tmp_minmax->tmp_max_tsn1=u_data->io->tmp_max_tsn1;
+	tmp_minmax->tmp_min_tsn2=u_data->io->tmp_min_tsn2;
+	tmp_minmax->tmp_max_tsn2=u_data->io->tmp_max_tsn2;
+	u_data->assoc->min_max = g_slist_prepend(u_data->assoc->min_max, tmp_minmax);
+	u_data->io->length = g_slist_length(u_data->assoc->min_max);
+	u_data->io->tmp=TRUE;
+	u_data->io->rectangle=FALSE;
+	u_data->io->rectangle_present=FALSE;
+	sctp_graph_redraw(u_data);
+	
+}
+
+
+
 static void
 on_zoomout_bt (GtkWidget *widget _U_, struct sctp_udata *u_data)
 {
@@ -816,7 +859,6 @@ on_zoomout_bt (GtkWidget *widget _U_, struct sctp_udata *u_data)
 	sctp_graph_redraw(u_data);
 }
 
-
 static gint
 on_button_press (GtkWidget *widget _U_, GdkEventButton *event, struct sctp_udata *u_data)
 {
@@ -826,8 +868,8 @@ on_button_press (GtkWidget *widget _U_, GdkEventButton *event, struct sctp_udata
 	{
 		gdk_draw_rectangle(u_data->io->pixmap,u_data->io->draw_area->style->white_gc,
 		                   FALSE,
-		                   (gint)floor(MINI(u_data->io->x_old,u_data->io->x_new)),
-		                   (gint)floor(MINI(u_data->io->y_old,u_data->io->y_new)),
+		                   (gint)floor(MIN(u_data->io->x_old,u_data->io->x_new)),
+		                   (gint)floor(MIN(u_data->io->y_old,u_data->io->y_new)),
 		                   (gint)floor(abs((long)(u_data->io->x_new-u_data->io->x_old))),
 		                   (gint)floor(abs((long)(u_data->io->y_new-u_data->io->y_old))));
 		ios=(sctp_graph_t *)OBJECT_GET_DATA(u_data->io->draw_area, "sctp_graph_t");
@@ -847,8 +889,8 @@ on_button_press (GtkWidget *widget _U_, GdkEventButton *event, struct sctp_udata
 	}
 	u_data->io->x_old=event->x;
 	u_data->io->y_old=event->y;
-	if (u_data->io->y_old>u_data->io->pixmap_height-BOTTOM_BORDER-u_data->io->offset)
-		u_data->io->y_old=u_data->io->pixmap_height-BOTTOM_BORDER-u_data->io->offset;
+	if (u_data->io->y_old>u_data->io->pixmap_height-BOTTOM_BORDER-u_data->io->offset-POINT_SIZE)
+		u_data->io->y_old=u_data->io->pixmap_height-BOTTOM_BORDER-u_data->io->offset-POINT_SIZE;
 	if (u_data->io->x_old<LEFT_BORDER+u_data->io->offset)
 		u_data->io->x_old=LEFT_BORDER+u_data->io->offset;
 	u_data->io->rectangle=FALSE;
@@ -861,8 +903,32 @@ static gint
 on_button_release (GtkWidget *widget _U_, GdkEventButton *event, struct sctp_udata *u_data)
 {
 	sctp_graph_t *ios;
-	guint32 helpx;
-	guint32 helpy, x1_tmp, x2_tmp;
+	guint32 helpx, helpy, x1_tmp, x2_tmp,  y_value;
+	gint label_width, label_height;
+	gdouble x_value, position;
+	gint lwidth;
+	char label_string[30];
+	GdkGC *text_color;
+	#if GTK_MAJOR_VERSION < 2
+		GdkFont *font;
+#else
+		PangoLayout  *layout;
+#endif
+
+#if GTK_MAJOR_VERSION < 2
+		font = u_data->io->draw_area->style->font;
+#endif
+
+#if GTK_MAJOR_VERSION < 2
+		label_width=gdk_string_width(font, label_string);
+		label_height=gdk_string_height(font, label_string);
+#else
+		g_snprintf(label_string, 15, "%d", 0);
+		memcpy(label_string,(gchar *)g_locale_to_utf8(label_string, -1 , NULL, NULL, NULL), 15);
+		layout = gtk_widget_create_pango_layout(u_data->io->draw_area, label_string);
+		pango_layout_get_pixel_size(layout, &label_width, &label_height);
+
+#endif
 
 	if (event->y>u_data->io->pixmap_height-BOTTOM_BORDER-u_data->io->offset)
 		event->y = u_data->io->pixmap_height-BOTTOM_BORDER-u_data->io->offset;
@@ -870,11 +936,15 @@ on_button_release (GtkWidget *widget _U_, GdkEventButton *event, struct sctp_uda
 		event->x = LEFT_BORDER+u_data->io->offset;
 	if (abs((long)(event->x-u_data->io->x_old))>10 || abs((long)(event->y-u_data->io->y_old))>10)
 	{
+		u_data->io->rect_x_min = (gint)floor(MIN(u_data->io->x_old,event->x));
+		u_data->io->rect_x_max = (gint)ceil(MAX(u_data->io->x_old,event->x));
+		u_data->io->rect_y_min = (gint)floor(MIN(u_data->io->y_old,event->y));
+		u_data->io->rect_y_max = (gint)ceil(MAX(u_data->io->y_old,event->y))+POINT_SIZE;
 		gdk_draw_rectangle(u_data->io->pixmap,u_data->io->draw_area->style->black_gc,
-		                   FALSE,
-		                   (gint)floor(MINI(u_data->io->x_old,event->x)), (gint)floor(MINI(u_data->io->y_old,event->y)),
-		                   (gint)abs((long)(event->x-u_data->io->x_old)),
-		                   (gint)abs((long)(event->y-u_data->io->y_old)));
+ 		                   FALSE,
+ 		                   u_data->io->rect_x_min, u_data->io->rect_y_min,
+ 		                   u_data->io->rect_x_max - u_data->io->rect_x_min,
+						   u_data->io->rect_y_max - u_data->io->rect_y_min);
 		ios=(sctp_graph_t *)OBJECT_GET_DATA(u_data->io->draw_area, "sctp_graph_t");
 
 		if(!ios){
@@ -891,26 +961,97 @@ on_button_release (GtkWidget *widget _U_, GdkEventButton *event, struct sctp_uda
 
 		x1_tmp=(unsigned int)floor(u_data->io->min_x+((u_data->io->x_old-LEFT_BORDER-u_data->io->offset)*u_data->io->tmp_width/u_data->io->axis_width));
 		x2_tmp=(unsigned int)floor(u_data->io->min_x+((event->x-LEFT_BORDER-u_data->io->offset)*u_data->io->tmp_width/u_data->io->axis_width));
-		helpx=MINI(x1_tmp, x2_tmp);
+		helpx=MIN(x1_tmp, x2_tmp);
 		if (helpx==x2_tmp)
 		{
 			x2_tmp=x1_tmp;
 			x1_tmp=helpx;
 		}
+		u_data->io->x1_akt_sec = u_data->io->x1_tmp_sec;
+		u_data->io->x1_akt_usec = u_data->io->x1_tmp_usec;
+		u_data->io->x2_akt_sec = u_data->io->x2_tmp_sec;
+		u_data->io->x2_akt_usec = u_data->io->x2_tmp_usec;
 		u_data->io->x1_tmp_sec=(guint32)x1_tmp/1000000;
 		u_data->io->x1_tmp_usec=x1_tmp%1000000;
 		u_data->io->x2_tmp_sec=(guint32)x2_tmp/1000000;
 		u_data->io->x2_tmp_usec=x2_tmp%1000000;
-		u_data->io->y1_tmp=(guint32)((u_data->io->pixmap_height-BOTTOM_BORDER-u_data->io->y_old)/u_data->io->y_interval);
-		u_data->io->y2_tmp=(guint32)((u_data->io->pixmap_height-BOTTOM_BORDER-event->y)/u_data->io->y_interval);
-		helpy = MINI(u_data->io->y1_tmp, u_data->io->y2_tmp);
-		u_data->io->y2_tmp = MAXI(u_data->io->y1_tmp, u_data->io->y2_tmp);
+		u_data->io->y1_tmp=(guint32)((u_data->io->pixmap_height-BOTTOM_BORDER-u_data->io->offset-u_data->io->y_old)/u_data->io->y_interval);
+		u_data->io->y2_tmp=(guint32)((u_data->io->pixmap_height-BOTTOM_BORDER-u_data->io->offset-event->y)/u_data->io->y_interval);
+		helpy = MIN(u_data->io->y1_tmp, u_data->io->y2_tmp);
+		u_data->io->y2_tmp = MAX(u_data->io->y1_tmp, u_data->io->y2_tmp);
 		u_data->io->y1_tmp = helpy;
 		u_data->io->x_new=event->x;
 		u_data->io->y_new=event->y;
 		u_data->io->rectangle=TRUE;
+		u_data->io->rectangle_present=TRUE;
 	}
+	else
+	{
+		if (u_data->io->rectangle_present==TRUE)
+		{
+			u_data->io->rectangle_present=FALSE;
+			if (event->x >= u_data->io->rect_x_min && event->x <= u_data->io->rect_x_max && 
+			     event->y >= u_data->io->rect_y_min && event->y <= u_data->io->rect_y_max)
+				zoomin_bt(u_data);
+			else
+			{
+				u_data->io->x1_tmp_sec = u_data->io->x1_akt_sec;
+				u_data->io->x1_tmp_usec = u_data->io->x1_akt_usec;
+				u_data->io->x2_tmp_sec = u_data->io->x2_akt_sec;
+				u_data->io->x2_tmp_usec = u_data->io->x2_akt_usec;
+				sctp_graph_redraw(u_data);
+			}
+		}
+		else if (label_set)
+		{
+			label_set = FALSE;
+			sctp_graph_redraw(u_data);
+		}
+		else
+		{
+			x_value = ((event->x-LEFT_BORDER-u_data->io->offset) * ((u_data->io->x2_tmp_sec+u_data->io->x2_tmp_usec/1000000.0)-(u_data->io->x1_tmp_sec+u_data->io->x1_tmp_usec/1000000.0)) / (u_data->io->pixmap_width-LEFT_BORDER-u_data->io->offset))+u_data->io->x1_tmp_sec+u_data->io->x1_tmp_usec/1000000.0;
+			y_value = floor((u_data->io->pixmap_height-BOTTOM_BORDER-u_data->io->offset-event->y) * (max_tsn - min_tsn) / (u_data->io->pixmap_height-BOTTOM_BORDER-u_data->io->offset)) + min_tsn;
+			text_color = u_data->io->draw_area->style->black_gc;
+			g_snprintf(label_string, 30, "(%.6lf, %u)", x_value, y_value);
+			label_set = TRUE;
 
+			gdk_draw_line(u_data->io->pixmap,text_color, event->x-2, event->y, event->x+2, event->y);
+			gdk_draw_line(u_data->io->pixmap,text_color, event->x, event->y-2, event->x, event->y+2);
+			if (event->x+150>=u_data->io->pixmap_width)
+				position = event->x - 150;
+			else
+				position = event->x + 5;
+
+#if GTK_MAJOR_VERSION < 2
+			lwidth=gdk_string_width(font, label_string);
+		                            gdk_draw_string(u_data->io->pixmap,font,text_color,
+		                            position,
+		                            event->y-10,
+		                            label_string);
+#else
+			memcpy(label_string,(gchar *)g_locale_to_utf8(label_string, -1 , NULL, NULL, NULL), 15);
+			pango_layout_set_text(layout, label_string, -1);
+			pango_layout_get_pixel_size(layout, &lwidth, NULL);
+
+			gdk_draw_layout(u_data->io->pixmap,text_color,
+							position,
+							event->y-10,
+							layout);
+	#endif
+			ios=(sctp_graph_t *)OBJECT_GET_DATA(u_data->io->draw_area, "sctp_graph_t");
+
+			if(!ios){
+				exit(10);
+			}
+			gdk_draw_pixmap(u_data->io->draw_area->window,
+		                    u_data->io->draw_area->style->fg_gc[GTK_WIDGET_STATE(u_data->io->draw_area)],
+		                    ios->pixmap,
+		                    0, 0,
+		                    0, 0,
+		                    u_data->io->draw_area->allocation.width,
+		                    u_data->io->draw_area->allocation.height);
+		}
+	}
 	return TRUE;
 }
 
@@ -983,8 +1124,7 @@ static void init_sctp_graph_window(struct sctp_udata *u_data)
 	gtk_signal_connect(GTK_OBJECT(u_data->io->draw_area),"button_press_event",(GtkSignalFunc)on_button_press, u_data);
 	gtk_signal_connect(GTK_OBJECT(u_data->io->draw_area),"button_release_event",(GtkSignalFunc)on_button_release, u_data);
 	gtk_widget_set_events(u_data->io->draw_area, GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_EXPOSURE_MASK);
-	/* dlg_set_cancel(u_data->io->window, bt_close); */
-	
+
 	gtk_widget_show(u_data->io->window);
 }
 
