@@ -1,7 +1,7 @@
 /* 
  * Copyright 2004, Irene Ruengeler <i.ruengeler [AT] fh-muenster.de>
  *
- * $Id: sctp_stat.c 14391 2005-05-18 05:19:22Z etxrab $
+ * $Id: sctp_stat.c 17947 2006-04-21 20:42:26Z gerald $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -32,7 +32,6 @@
 #include "globals.h"
 #include "epan/packet_info.h"   /* Needed for packet_info */
 #include <epan/tap.h>           /* Needed for register_tap_listener */
-#include "tap_menu.h"
 #include "dlg_utils.h"
 #include "compat_macros.h"
 #include "register.h"
@@ -138,7 +137,7 @@ void free_first(gpointer data, gpointer user_data _U_)
 	g_free(data);
 }
 
-void tsn_free(gpointer data, gpointer user_data _U_)
+static void tsn_free(gpointer data, gpointer user_data _U_)
 {
 	tsn_t *tsn;
 
@@ -292,17 +291,17 @@ static sctp_assoc_info_t *calc_checksum(struct _sctp_info *check_data, sctp_asso
 }
 
 
-gint sctp_assoc_vtag_cmp(gconstpointer aa, gconstpointer bb)
+static gint sctp_assoc_vtag_cmp(gconstpointer aa, gconstpointer bb)
 {
 
 	const struct _sctp_assoc_info* a = aa;
 	const struct _sctp_assoc_info* b = bb;
 
 	if (a == b)
-		return(0);
+		return(FORWARD_STREAM);
 
 	if (a == NULL || b == NULL)
-		return(1);
+		return(ASSOC_NOT_FOUND);
 
 	/* assoc known*/
 	if ((a->port1 == b->port1) &&
@@ -314,12 +313,14 @@ gint sctp_assoc_vtag_cmp(gconstpointer aa, gconstpointer bb)
 
 	if ((a->port1 == b->port2) &&
 	    (a->port2 == b->port1) &&
-	    (a->verification_tag1 == b->verification_tag2))
+	    (a->verification_tag1 == b->verification_tag2) &&
+	    (a->verification_tag1 != 0))
 		return(BACKWARD_STREAM);
 		
 	if ((a->port1 == b->port2) &&
 	    (a->port2 == b->port1) &&
-	    (a->verification_tag2 == b->verification_tag1))
+	    (a->verification_tag2 == b->verification_tag1) &&
+	    (a->verification_tag2 != 0))
 		return(BACKWARD_STREAM);
 
 	/*forward stream verifivation tag can be added*/
@@ -347,8 +348,8 @@ gint sctp_assoc_vtag_cmp(gconstpointer aa, gconstpointer bb)
 	return(ASSOC_NOT_FOUND);
 }
 
-
-gint sctp_assoc_address_cmp(gconstpointer aa, gconstpointer bb)
+/*
+static gint sctp_assoc_address_cmp(gconstpointer aa, gconstpointer bb)
 {
 	GList *srclist, *dstlist;
 	const struct _sctp_tmp_info* a = aa;
@@ -571,13 +572,13 @@ gint sctp_assoc_address_cmp(gconstpointer aa, gconstpointer bb)
 	g_free(dst);
 	return ASSOC_NOT_FOUND;
 }
+*/
 
 
 
 
 
-
-sctp_assoc_info_t * find_assoc(sctp_tmp_info_t * needle)
+static sctp_assoc_info_t * find_assoc(sctp_tmp_info_t * needle)
 {
 	sctp_allassocs_info_t *assoc_info;
 	sctp_assoc_info_t *info = NULL;
@@ -585,7 +586,7 @@ sctp_assoc_info_t * find_assoc(sctp_tmp_info_t * needle)
 	guint8 cmp;
 
 	assoc_info = &sctp_tapinfo_struct;
-	if ((list = g_list_first(assoc_info->assoc_info_list))!=NULL)
+	if ((list = g_list_last(assoc_info->assoc_info_list))!=NULL)
 	{
 		while (list)
 		{
@@ -649,13 +650,13 @@ sctp_assoc_info_t * find_assoc(sctp_tmp_info_t * needle)
 				return info;
 			}
 
-			list = g_list_next(list);
+			list = g_list_previous(list);
 		}
 	}
 	return NULL;
 }
 
-sctp_assoc_info_t * add_chunk_count(address * vadd, sctp_assoc_info_t * info, guint32 direction, guint32 type)
+static sctp_assoc_info_t * add_chunk_count(address * vadd, sctp_assoc_info_t * info, guint32 direction, guint32 type)
 {
 	GList *list;
 	address *v=NULL;
@@ -700,7 +701,7 @@ sctp_assoc_info_t * add_chunk_count(address * vadd, sctp_assoc_info_t * info, gu
 	return info;
 }
 
-sctp_assoc_info_t * add_address(address * vadd, sctp_assoc_info_t *info, guint8 direction)
+static sctp_assoc_info_t * add_address(address * vadd, sctp_assoc_info_t *info, guint8 direction)
 {
 	GList *list;
 	address *v=NULL;
@@ -791,7 +792,7 @@ packet(void *tapdata _U_, packet_info *pinfo , epan_dissect_t *edt _U_ , const v
 		tmp_info.src.len  = 16;
 	}
 	
-	addr = malloc(tmp_info.src.len);
+	addr = g_malloc(tmp_info.src.len);
 	memcpy(addr, sctp_info->ip_src.data, tmp_info.src.len);
 	tmp_info.src.data = addr;
 
@@ -808,7 +809,7 @@ packet(void *tapdata _U_, packet_info *pinfo , epan_dissect_t *edt _U_ , const v
 		tmp_info.dst.len  = 16;
 	}
 	
-	addr = malloc(tmp_info.dst.len);
+	addr = g_malloc(tmp_info.dst.len);
 	memcpy(addr, sctp_info->ip_dst.data, tmp_info.dst.len);
 	tmp_info.dst.data = addr;
 
@@ -839,12 +840,12 @@ packet(void *tapdata _U_, packet_info *pinfo , epan_dissect_t *edt _U_ , const v
 			memset(info, 0, sizeof(sctp_assoc_info_t));
 			info->src.type = tmp_info.src.type;
 			info->src.len  = tmp_info.src.len;
-			addr = malloc(tmp_info.dst.len);
+			addr = g_malloc(tmp_info.dst.len);
 			memcpy(addr,(tmp_info.src.data), tmp_info.src.len);
 			info->src.data = addr;
 			info->dst.type = tmp_info.dst.type;
 			info->dst.len  = tmp_info.dst.len;
-			addr = malloc(tmp_info.dst.len);
+			addr = g_malloc(tmp_info.dst.len);
 			memcpy(addr, (tmp_info.dst.data), tmp_info.dst.len);
 			info->dst.data = addr;
 			info->port1 = tmp_info.port1;
@@ -904,22 +905,22 @@ packet(void *tapdata _U_, packet_info *pinfo , epan_dissect_t *edt _U_ , const v
 				sack->tsns = NULL;
 				sack->src.type=tsn->src.type = tmp_info.src.type;
 				sack->src.len=tsn->src.len   = tmp_info.src.len;
-				addr = malloc(tmp_info.src.len);
+				addr = g_malloc(tmp_info.src.len);
 				memcpy(addr, tmp_info.src.data, tmp_info.src.len);
 				tsn->src.data = addr;
-				addr = malloc(tmp_info.src.len);
+				addr = g_malloc(tmp_info.src.len);
 				memcpy(addr, tmp_info.src.data, tmp_info.src.len);
 				sack->src.data = addr;
 				sack->dst.type = tsn->dst.type = tmp_info.dst.type;
 				sack->dst.len  =tsn->dst.len   = tmp_info.dst.len;
-				addr = malloc(tmp_info.dst.len);
+				addr = g_malloc(tmp_info.dst.len);
 				memcpy(addr, tmp_info.dst.data, tmp_info.dst.len);
 				tsn->dst.data = addr;
-				addr = malloc(tmp_info.dst.len);
+				addr = g_malloc(tmp_info.dst.len);
 				memcpy(addr, tmp_info.dst.data, tmp_info.dst.len);
 				sack->dst.data = addr;
-				sack->secs=tsn->secs   = (guint32)pinfo->fd->rel_secs;
-				sack->usecs=tsn->usecs = (guint32)pinfo->fd->rel_usecs;
+				sack->secs=tsn->secs   = (guint32)pinfo->fd->rel_ts.secs;
+				sack->usecs=tsn->usecs = (guint32)pinfo->fd->rel_ts.nsecs/1000;
 				if (((tvb_get_guint8(sctp_info->tvb[0],0)) == SCTP_DATA_CHUNK_ID) ||
 					((tvb_get_guint8(sctp_info->tvb[0],0)) == SCTP_SACK_CHUNK_ID))
 				{
@@ -956,7 +957,7 @@ packet(void *tapdata _U_, packet_info *pinfo , epan_dissect_t *edt _U_ , const v
 						store = g_malloc(sizeof (address));
 						store->type = AT_IPv4;;
 						store->len  = 4;
-						store->data = malloc(4);
+						store->data = g_malloc(4);
 						tvb_memcpy(sctp_info->tvb[chunk_number], (guint8 *)(store->data),IPV4_ADDRESS_OFFSET, 4);
 						info = add_address(store, info, 1);
 					}
@@ -965,7 +966,7 @@ packet(void *tapdata _U_, packet_info *pinfo , epan_dissect_t *edt _U_ , const v
 						store = g_malloc(sizeof (address));
 						store->type = AT_IPv6;;
 						store->len  = 16;
-						store->data = malloc(16);
+						store->data = g_malloc(16);
 						tvb_memcpy(sctp_info->tvb[chunk_number], (guint8 *)(store->data),IPV6_ADDRESS_OFFSET, IPV6_ADDRESS_LENGTH);	
 						info = add_address(store, info, 1);
 					}
@@ -1070,14 +1071,14 @@ packet(void *tapdata _U_, packet_info *pinfo , epan_dissect_t *edt _U_ , const v
 				store = g_malloc(sizeof (address));
 				store->type = tmp_info.src.type;
 				store->len  = tmp_info.src.len;
-				addr = malloc(tmp_info.src.len);
+				addr = g_malloc(tmp_info.src.len);
 				memcpy(addr,(tmp_info.src.data),tmp_info.src.len);
 				store->data = addr;
 				info  = add_address(store, info, 1);
 				store = g_malloc(sizeof (address));
 				store->type = tmp_info.dst.type;
 				store->len  = tmp_info.dst.len;
-				addr = malloc(tmp_info.dst.len);
+				addr = g_malloc(tmp_info.dst.len);
 				memcpy(addr,(tmp_info.dst.data),tmp_info.dst.len);
 				store->data = addr;
 				info = add_address(store, info, 2);
@@ -1118,22 +1119,22 @@ packet(void *tapdata _U_, packet_info *pinfo , epan_dissect_t *edt _U_ , const v
 			sack->tsns = NULL;
 			sack->src.type = tsn->src.type = tmp_info.src.type;
 			sack->src.len  = tsn->src.len = tmp_info.src.len;
-			addr = malloc(tmp_info.src.len);
+			addr = g_malloc(tmp_info.src.len);
 			memcpy(addr, tmp_info.src.data, tmp_info.src.len);
 			tsn->src.data = addr;
-			addr = malloc(tmp_info.src.len);
+			addr = g_malloc(tmp_info.src.len);
 			memcpy(addr, tmp_info.src.data, tmp_info.src.len);
 			sack->src.data = addr;
 			sack->dst.type = tsn->dst.type = tmp_info.dst.type;
 			sack->dst.len  = tsn->dst.len = tmp_info.dst.len;
-			addr = malloc(tmp_info.dst.len);
+			addr = g_malloc(tmp_info.dst.len);
 			memcpy(addr, tmp_info.dst.data, tmp_info.dst.len);
 			tsn->dst.data = addr;			
-			addr = malloc(tmp_info.dst.len);
+			addr = g_malloc(tmp_info.dst.len);
 			memcpy(addr, tmp_info.dst.data, tmp_info.dst.len);
 			sack->dst.data = addr;
-			sack->secs=tsn->secs = (guint32)pinfo->fd->rel_secs;
-			sack->usecs=tsn->usecs = (guint32)pinfo->fd->rel_usecs;
+			sack->secs=tsn->secs = (guint32)pinfo->fd->rel_ts.secs;
+			sack->usecs=tsn->usecs = (guint32)pinfo->fd->rel_ts.nsecs/1000;
 			if (((tvb_get_guint8(sctp_info->tvb[0],0)) == SCTP_DATA_CHUNK_ID) ||
 			((tvb_get_guint8(sctp_info->tvb[0],0)) == SCTP_SACK_CHUNK_ID))
 			{
@@ -1160,7 +1161,7 @@ packet(void *tapdata _U_, packet_info *pinfo , epan_dissect_t *edt _U_ , const v
 		store = g_malloc(sizeof (address));
 		store->type = tmp_info.src.type;
 		store->len  = tmp_info.src.len;
-		addr = malloc(tmp_info.src.len);
+		addr = g_malloc(tmp_info.src.len);
 		memcpy(addr,(tmp_info.src.data),tmp_info.src.len);
 		store->data = addr;
 			
@@ -1172,7 +1173,7 @@ packet(void *tapdata _U_, packet_info *pinfo , epan_dissect_t *edt _U_ , const v
 		store = g_malloc(sizeof (address));
 		store->type = tmp_info.dst.type;
 		store->len  = tmp_info.dst.len;
-		addr = malloc(tmp_info.dst.len);
+		addr = g_malloc(tmp_info.dst.len);
 		memcpy(addr,(tmp_info.dst.data),tmp_info.dst.len);
 		store->data = addr;
 		
@@ -1222,7 +1223,7 @@ packet(void *tapdata _U_, packet_info *pinfo , epan_dissect_t *edt _U_ , const v
 						store = g_malloc(sizeof (address));
 						store->type = AT_IPv4;;
 						store->len  = 4;
-						store->data = malloc(4);
+						store->data = g_malloc(4);
 						tvb_memcpy(sctp_info->tvb[chunk_number], (guint8 *)(store->data),IPV4_ADDRESS_OFFSET, 4);
 						info = add_address(store, info, info->direction);
 					}
@@ -1231,7 +1232,7 @@ packet(void *tapdata _U_, packet_info *pinfo , epan_dissect_t *edt _U_ , const v
 						store = g_malloc(sizeof (address));
 						store->type = AT_IPv6;;
 						store->len  = 16;
-						store->data = malloc(16);
+						store->data = g_malloc(16);
 						tvb_memcpy(sctp_info->tvb[chunk_number], (guint8 *)(store->data),IPV6_ADDRESS_OFFSET, IPV6_ADDRESS_LENGTH);
 						info = add_address(store, info, info->direction);
 					}
@@ -1444,12 +1445,6 @@ const sctp_allassocs_info_t* sctp_stat_get_info(void)
 }
 
 
-static void
-gtk_sctpstat_init(char *dummy _U_)
-{
-
-}
-
 static void sctp_update(void *dummy _U_)
 {
 	if (get_stat_dlg()!=NULL)
@@ -1463,7 +1458,6 @@ register_tap_listener_sctp_stat(void)
 
 	if (!sctp_tapinfo_struct.is_registered)
 	{
-		register_tap_listener_cmd_arg("sctp",gtk_sctpstat_init);
 		if ((error_string = register_tap_listener("sctp", &sctp_tapinfo_struct, NULL, reset, packet, sctp_update))) {
 			simple_dialog(ESD_TYPE_ERROR, ESD_BTN_OK, error_string->str);
 			g_string_free(error_string, TRUE);
