@@ -3,7 +3,7 @@
  * Helper routines common to all service response time statistics
  * tap.
  *
- * $Id: service_response_time_table.c 14795 2005-06-27 08:13:41Z etxrab $
+ * $Id: service_response_time_table.c 17139 2006-02-01 06:09:55Z etxrab $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -44,7 +44,7 @@
 #include "color.h"
 #include "gtk/color_dlg.h"
 #include "main.h"
-#include "ui_util.h"
+#include "gui_utils.h"
 #include "gtkglobals.h"
 
 
@@ -336,7 +336,7 @@ init_srt_table(srt_stat_table *rst, int num_procs, GtkWidget *vbox, const char *
 	GdkPixmap *ascend_pm, *descend_pm;
 	GtkStyle *win_style;
 	GtkWidget *column_lb;
-	char *default_titles[] = { "Index", "Procedure", "Calls", "Min SRT", "Max SRT", "Avg SRT" };
+	const char *default_titles[] = { "Index", "Procedure", "Calls", "Min SRT", "Max SRT", "Avg SRT" };
 
 
 	if(filter_string){
@@ -408,13 +408,13 @@ init_srt_table(srt_stat_table *rst, int num_procs, GtkWidget *vbox, const char *
 	rst->num_procs=num_procs;
 	rst->procedures=g_malloc(sizeof(srt_procedure_t)*num_procs);
 	for(i=0;i<num_procs;i++){
-		rst->procedures[i].num=0;
-		rst->procedures[i].min.secs=0;
-		rst->procedures[i].min.nsecs=0;
-		rst->procedures[i].max.secs=0;
-		rst->procedures[i].max.nsecs=0;
-		rst->procedures[i].tot.secs=0;
-		rst->procedures[i].tot.nsecs=0;
+		rst->procedures[i].stats.num=0;
+		rst->procedures[i].stats.min.secs=0;
+		rst->procedures[i].stats.min.nsecs=0;
+		rst->procedures[i].stats.max.secs=0;
+		rst->procedures[i].stats.max.nsecs=0;
+		rst->procedures[i].stats.tot.secs=0;
+		rst->procedures[i].stats.tot.nsecs=0;
 		for(j=0;j<6;j++){
 			rst->procedures[i].entries[j]=NULL;
 		}
@@ -429,8 +429,6 @@ init_srt_table(srt_stat_table *rst, int num_procs, GtkWidget *vbox, const char *
 void
 init_srt_table_row(srt_stat_table *rst, int index, const char *procedure)
 {
-	char str[10];
-
 	/* we have discovered a new procedure. Extend the table accordingly */
 	if(index>=rst->num_procs){
 		int old_num_procs=rst->num_procs;
@@ -438,20 +436,19 @@ init_srt_table_row(srt_stat_table *rst, int index, const char *procedure)
 		rst->num_procs=index+1;
 		rst->procedures=g_realloc(rst->procedures, sizeof(srt_procedure_t)*(rst->num_procs));
 		for(i=old_num_procs;i<rst->num_procs;i++){
-			rst->procedures[i].num=0;
-			rst->procedures[i].min.secs=0;
-			rst->procedures[i].min.nsecs=0;
-			rst->procedures[i].max.secs=0;
-			rst->procedures[i].max.nsecs=0;
-			rst->procedures[i].tot.secs=0;
-			rst->procedures[i].tot.nsecs=0;
+			rst->procedures[i].stats.num=0;
+			rst->procedures[i].stats.min.secs=0;
+			rst->procedures[i].stats.min.nsecs=0;
+			rst->procedures[i].stats.max.secs=0;
+			rst->procedures[i].stats.max.nsecs=0;
+			rst->procedures[i].stats.tot.secs=0;
+			rst->procedures[i].stats.tot.nsecs=0;
 			for(j=0;j<6;j++){
 				rst->procedures[i].entries[j]=NULL;
 			}
 		}
 	}
-	g_snprintf(str, 10, "%d",index);
-	rst->procedures[index].entries[0]=g_strdup(str);
+	rst->procedures[index].entries[0]=g_strdup_printf("%d", index);
 
 	rst->procedures[index].entries[1]=g_strdup(procedure);
 
@@ -459,106 +456,82 @@ init_srt_table_row(srt_stat_table *rst, int index, const char *procedure)
 	rst->procedures[index].entries[3]=g_strdup("0");
 	rst->procedures[index].entries[4]=g_strdup("0");
 	rst->procedures[index].entries[5]=g_strdup("0");
-
-	gtk_clist_insert(rst->table, index, rst->procedures[index].entries);
-	gtk_clist_set_row_data(rst->table, index, (gpointer) index);
 }
 
 void
 add_srt_table_data(srt_stat_table *rst, int index, const nstime_t *req_time, packet_info *pinfo)
 {
 	srt_procedure_t *rp;
-	nstime_t delta;
+	nstime_t t, delta;
+	gint row;
 
 	rp=&rst->procedures[index];
 
+	/*
+	 * If the count of calls for this procedure is currently zero, it's
+	 * going to become non-zero, so add a row for it (we don't want
+	 * rows for procedures that have no calls - especially if the
+	 * procedure has no calls because the index doesn't correspond
+	 * to a procedure, but is an unused/reserved value).
+	 *
+	 * (Yes, this means that the rows aren't in order by anything
+	 * interesting.  That's why we have the table sorted by a column.)
+	 */
+	if (rp->stats.num==0){
+		row=gtk_clist_append(rst->table, rst->procedures[index].entries);
+		gtk_clist_set_row_data(rst->table, row, (gpointer) index);
+	}
+
 	/* calculate time delta between request and reply */
-	delta.secs=pinfo->fd->abs_secs-req_time->secs;
-	delta.nsecs=pinfo->fd->abs_usecs*1000-req_time->nsecs;
-	if(delta.nsecs<0){
-		delta.nsecs+=1000000000;
-		delta.secs--;
-	}
+	t=pinfo->fd->abs_ts;
+	nstime_delta(&delta, &t, req_time);
 
-	if(rp->num==0){
-		rp->max.secs=delta.secs;
-		rp->max.nsecs=delta.nsecs;
-	}
-
-	if(rp->num==0){
-		rp->min.secs=delta.secs;
-		rp->min.nsecs=delta.nsecs;
-	}
-
-	if( (delta.secs<rp->min.secs)
-	||( (delta.secs==rp->min.secs)
-	  &&(delta.nsecs<rp->min.nsecs) ) ){
-		rp->min.secs=delta.secs;
-		rp->min.nsecs=delta.nsecs;
-	}
-
-	if( (delta.secs>rp->max.secs)
-	||( (delta.secs==rp->max.secs)
-	  &&(delta.nsecs>rp->max.nsecs) ) ){
-		rp->max.secs=delta.secs;
-		rp->max.nsecs=delta.nsecs;
-	}
-
-	rp->tot.secs += delta.secs;
-	rp->tot.nsecs += delta.nsecs;
-	if(rp->tot.nsecs>1000000000){
-		rp->tot.nsecs-=1000000000;
-		rp->tot.secs++;
-	}
-
-	rp->num++;
+	time_stat_update(&rp->stats, &delta, pinfo);
 }
 
 void
 draw_srt_table_data(srt_stat_table *rst)
 {
 	int i,j;
-#ifdef G_HAVE_UINT64
 	guint64 td;
-#else
-	guint32 td;
-#endif
-	char str[256], *strp;
+	char *strp;
 
 	for(i=0;i<rst->num_procs;i++){
-		/* scale it to units of 10us.*/
-		/* for long captures with a large tot time, this can overflow on 32bit */
-		td=(int)rst->procedures[i].tot.secs;
-		td=td*100000+(int)rst->procedures[i].tot.nsecs/10000;
-		if(rst->procedures[i].num){
-			td/=rst->procedures[i].num;
-		} else {
-			td=0;
+		/* ignore procedures with no calls (they don't have CList rows) */
+		if(rst->procedures[i].stats.num==0){
+			continue;
 		}
 
+		/* scale it to units of 10us.*/
+		/* for long captures with a large tot time, this can overflow on 32bit */
+		td=(int)rst->procedures[i].stats.tot.secs;
+		td=td*100000+(int)rst->procedures[i].stats.tot.nsecs/10000;
+		td/=rst->procedures[i].stats.num;
+
 		j=gtk_clist_find_row_from_data(rst->table, (gpointer)i);
-		g_snprintf(str, 256, "%d", rst->procedures[i].num);
-		strp=g_strdup(str);
+		strp=g_strdup_printf("%d", rst->procedures[i].stats.num);
 		gtk_clist_set_text(rst->table, j, 2, strp);
 		g_free(rst->procedures[i].entries[2]);
 		rst->procedures[i].entries[2]=strp;
 
 
-		g_snprintf(str, 256, "%3d.%05d", (int)rst->procedures[i].min.secs,rst->procedures[i].min.nsecs/10000);
-		strp=g_strdup(str);
+		strp=g_strdup_printf("%3d.%05d",
+		    (int)rst->procedures[i].stats.min.secs,
+		    rst->procedures[i].stats.min.nsecs/10000);
 		gtk_clist_set_text(rst->table, j, 3, strp);
 		g_free(rst->procedures[i].entries[3]);
 		rst->procedures[i].entries[3]=strp;
 
 
-		g_snprintf(str, 256, "%3d.%05d", (int)rst->procedures[i].max.secs,rst->procedures[i].max.nsecs/10000);
-		strp=g_strdup(str);
+		strp=g_strdup_printf("%3d.%05d",
+		    (int)rst->procedures[i].stats.max.secs,
+		    rst->procedures[i].stats.max.nsecs/10000);
 		gtk_clist_set_text(rst->table, j, 4, strp);
 		g_free(rst->procedures[i].entries[4]);
 		rst->procedures[i].entries[4]=strp;
 
-		g_snprintf(str, 256, "%3d.%05d", td/100000, td%100000);
-		strp=g_strdup(str);
+		strp=g_strdup_printf("%3" PRId64 ".%05" PRId64,
+		    td/100000, td%100000);
 		gtk_clist_set_text(rst->table, j, 5, strp);
 		g_free(rst->procedures[i].entries[5]);
 		rst->procedures[i].entries[5]=strp;
@@ -574,14 +547,9 @@ reset_srt_table_data(srt_stat_table *rst)
 	int i;
 
 	for(i=0;i<rst->num_procs;i++){
-		rst->procedures[i].num=0;
-		rst->procedures[i].min.secs=0;
-		rst->procedures[i].min.nsecs=0;
-		rst->procedures[i].max.secs=0;
-		rst->procedures[i].max.nsecs=0;
-		rst->procedures[i].tot.secs=0;
-		rst->procedures[i].tot.nsecs=0;
+		time_stat_init(&rst->procedures[i].stats);
 	}
+	gtk_clist_clear(rst->table);
 }
 
 void
