@@ -2,7 +2,7 @@
  * Routines for docsis dissection
  * Copyright 2002, Anand V. Narwani <anand[AT]narwani.org>
  *
- * $Id: packet-docsis.c 12593 2004-11-24 09:13:52Z guy $
+ * $Id: packet-docsis.c 17138 2006-02-01 05:44:20Z etxrab $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -49,8 +49,6 @@
 # include "config.h"
 #endif
 
-#include "plugins/plugin_api.h"
-#include "plugins/plugin_api_defs.h"
 #include "moduleinfo.h"
 
 #include <stdio.h>
@@ -196,7 +194,7 @@ dissect_ehdr (tvbuff_t * tvb, proto_tree * tree, gboolean isfrag)
 {
   proto_tree *ehdr_tree;
   proto_item *it;
-  guint8 ehdrlen;
+  gint ehdrlen;
   int pos;
   guint8 type;
   guint8 len;
@@ -209,7 +207,7 @@ dissect_ehdr (tvbuff_t * tvb, proto_tree * tree, gboolean isfrag)
 
   it = proto_tree_add_text (tree, tvb, pos, ehdrlen, "Extended Header");
   ehdr_tree = proto_item_add_subtree (it, ett_ehdr);
-  while (pos < (int)(ehdrlen + 4))
+  while (pos < ehdrlen + 4)
     {
       type = (tvb_get_guint8 (tvb, pos) & 0xF0);
       len = (tvb_get_guint8 (tvb, pos) & 0x0F);
@@ -324,13 +322,14 @@ dissect_docsis (tvbuff_t * tvb, packet_info * pinfo, proto_tree * tree)
   guint8 fctype;
   guint8 fcparm;
   guint8 ehdron;
-  guint8 mac_parm;
-  guint8 hdrlen;
+  gint mac_parm;
+  gint hdrlen;
   guint16 len_sid;
   tvbuff_t *next_tvb, *mgt_tvb;
   gint pdulen, captured_length;
-  guint16 framelen;
+  gint framelen;
   gboolean isfrag = FALSE;
+  gint oldconcatlen;
 
 /* Set up structures needed to add the protocol subtree and manage it */
   proto_item *ti;
@@ -338,8 +337,8 @@ dissect_docsis (tvbuff_t * tvb, packet_info * pinfo, proto_tree * tree)
 /* concatlen and concatpos are declared static to allow for recursive calls to
  * the dissect_docsis routine when dissecting Concatenated frames
  */
-  static guint16 concatlen;
-  static guint16 concatpos;
+  static gint concatlen;
+  static gint concatpos;
 
 /* Extract important fields */
   fc = tvb_get_guint8 (tvb, 0);	/* Frame Control Byte */
@@ -565,8 +564,11 @@ dissect_docsis (tvbuff_t * tvb, packet_info * pinfo, proto_tree * tree)
 	   * docsis frames are dissected. */
 	  while (concatlen > 0)
 	    {
+	      oldconcatlen = concatlen;
 	      next_tvb = tvb_new_subset (tvb, concatpos, -1, concatlen);
 	      call_dissector (docsis_handle, next_tvb, pinfo, tree);
+	      if (oldconcatlen <= concatlen)
+		THROW(ReportedBoundsError);
 	    }
 	  concatlen = 0;
 	  concatpos = 0;
@@ -698,9 +700,9 @@ proto_register_docsis (void)
       "Version", HFILL}
      },
     {&hf_docsis_ehdr_phsi,
-     {"Payload Header Supression Index", "docsis.ehdr.phsi",
+     {"Payload Header Suppression Index", "docsis.ehdr.phsi",
       FT_UINT8, BASE_DEC, NULL, 0x0,
-      "Payload Header Supression Index", HFILL}
+      "Payload Header Suppression Index", HFILL}
      },
     {&hf_docsis_ehdr_qind,
      {"Queue Indicator", "docsis.ehdr.qind",
@@ -773,46 +775,8 @@ proto_reg_handoff_docsis (void)
 #ifndef ENABLE_STATIC
 
 G_MODULE_EXPORT void
-plugin_reg_handoff (void)
+plugin_register (void)
 {
-  proto_reg_handoff_docsis ();
-  proto_reg_handoff_docsis_bpkmattr ();
-  proto_reg_handoff_docsis_dsarsp ();
-  proto_reg_handoff_docsis_mgmt ();
-  proto_reg_handoff_docsis_rngreq ();
-  proto_reg_handoff_docsis_rngrsp ();
-  proto_reg_handoff_docsis_bpkmreq ();
-  proto_reg_handoff_docsis_dscack ();
-  proto_reg_handoff_docsis_map ();
-  proto_reg_handoff_docsis_tlv ();
-  proto_reg_handoff_docsis_bpkmrsp ();
-  proto_reg_handoff_docsis_dscreq ();
-  proto_reg_handoff_docsis_regack ();
-  proto_reg_handoff_docsis_uccreq ();
-  proto_reg_handoff_docsis_dscrsp ();
-  proto_reg_handoff_docsis_regreq ();
-  proto_reg_handoff_docsis_uccrsp ();
-  proto_reg_handoff_docsis_dsaack ();
-  proto_reg_handoff_docsis_dsdreq ();
-  proto_reg_handoff_docsis_regrsp ();
-  proto_reg_handoff_docsis_ucd ();
-  proto_reg_handoff_docsis_type29ucd ();
-  proto_reg_handoff_docsis_dsareq ();
-  proto_reg_handoff_docsis_dsdrsp ();
-  proto_reg_handoff_docsis_vsif ();
-  proto_reg_handoff_docsis_intrngreq ();
-
-}
-
-G_MODULE_EXPORT void
-plugin_init (plugin_address_table_t * pat
-#ifndef PLUGINS_NEED_ADDRESS_TABLE
-	     _U_
-#endif
-  )
-{
-  /* initialise the table of pointers needed in Win32 DLLs */
-  plugin_address_table_init (pat);
   /* register the new protocol, protocol fields, and subtrees */
   if (proto_docsis == -1)
     {				/* execute protocol initialization only once */
@@ -837,6 +801,10 @@ plugin_init (plugin_address_table_t * pat
       proto_register_docsis_regrsp ();
       proto_register_docsis_ucd ();
       proto_register_docsis_type29ucd ();
+      proto_register_docsis_dcd ();
+      proto_register_docsis_dccreq ();
+      proto_register_docsis_dccrsp ();
+      proto_register_docsis_dccack ();
       proto_register_docsis_dsareq ();
       proto_register_docsis_dsdrsp ();
       proto_register_docsis_rngreq ();
@@ -846,4 +814,40 @@ plugin_init (plugin_address_table_t * pat
     }
 }
 
+G_MODULE_EXPORT void
+plugin_reg_handoff (void)
+{
+  proto_reg_handoff_docsis ();
+  proto_reg_handoff_docsis_bpkmattr ();
+  proto_reg_handoff_docsis_dsarsp ();
+  proto_reg_handoff_docsis_mgmt ();
+  proto_reg_handoff_docsis_rngreq ();
+  proto_reg_handoff_docsis_rngrsp ();
+  proto_reg_handoff_docsis_bpkmreq ();
+  proto_reg_handoff_docsis_dscack ();
+  proto_reg_handoff_docsis_map ();
+  proto_reg_handoff_docsis_tlv ();
+  proto_reg_handoff_docsis_bpkmrsp ();
+  proto_reg_handoff_docsis_dscreq ();
+  proto_reg_handoff_docsis_regack ();
+  proto_reg_handoff_docsis_uccreq ();
+  proto_reg_handoff_docsis_dscrsp ();
+  proto_reg_handoff_docsis_regreq ();
+  proto_reg_handoff_docsis_uccrsp ();
+  proto_reg_handoff_docsis_dsaack ();
+  proto_reg_handoff_docsis_dsdreq ();
+  proto_reg_handoff_docsis_regrsp ();
+  proto_reg_handoff_docsis_ucd ();
+  proto_reg_handoff_docsis_type29ucd ();
+  proto_reg_handoff_docsis_dcd ();
+  proto_reg_handoff_docsis_dccreq ();
+  proto_reg_handoff_docsis_dccrsp ();
+  proto_reg_handoff_docsis_dccack ();
+  proto_reg_handoff_docsis_dsareq ();
+  proto_reg_handoff_docsis_dsdrsp ();
+  proto_reg_handoff_docsis_vsif ();
+  proto_reg_handoff_docsis_intrngreq ();
+
+}
+
 #endif
