@@ -1,7 +1,7 @@
 /* summary.h
  * Definitions for capture file summary data
  *
- * $Id: summary.h 11400 2004-07-18 00:24:25Z guy $
+ * $Id: summary.h 17327 2006-02-17 02:18:48Z ulfl $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -25,8 +25,12 @@
 #ifndef __SUMMARY_H__
 #define __SUMMARY_H__
 
+#ifdef HAVE_LIBPCAP
+#include "capture.h"
+#endif
+
 typedef struct _summary_tally {
-    guint32	bytes;		/* total bytes */
+    guint64	bytes;		/* total bytes */
     double	start_time;	/* seconds, with msec resolution */
     double	stop_time;	/* seconds, with msec resolution */
     double	elapsed_time;	/* seconds, with msec resolution,
@@ -35,7 +39,7 @@ typedef struct _summary_tally {
     int		marked_count;	/* number of marked packets */
     int		packet_count;	/* total number of packets in trace */
     int		filtered_count; /* number of filtered packets */
-    guint32	filtered_bytes;	/* total bytes in the filtered packets */
+    guint64	filtered_bytes;	/* total bytes in the filtered packets */
     double 	filtered_start; /* time in seconds, with msec resolution */
     double 	filtered_stop;  /* time in seconds, with msec resolution */
     const char	*filename;
@@ -44,13 +48,22 @@ typedef struct _summary_tally {
     gboolean	has_snap;	/* TRUE if maximum capture packet length is known */
     int		snap;		/* Maximum captured packet length */
     gboolean    drops_known;	/* TRUE if number of packet drops is known */
-    guint32     drops;		/* number of packet drops */
-    const char	*iface;		/* interface name */
+    guint64     drops;		/* number of packet drops */
     const char	*dfilter;	/* display filter */
+
+    /* capture related, use summary_fill_in_capture() to get values */
     const char	*cfilter;	/* capture filter */
+    const char	*iface;		/* interface name */
+    const char	*iface_descr;/* descriptive interface name */
 } summary_tally;
 
-void summary_fill_in(summary_tally *st);
+extern void 
+summary_fill_in(capture_file *cf, summary_tally *st);
+
+#ifdef HAVE_LIBPCAP
+extern void
+summary_fill_in_capture(capture_options *capture_opts, summary_tally *st);
+#endif
 
 #endif /* summary.h */
 
