@@ -1,7 +1,7 @@
 /* tap-dcerpcstat.c
  * dcerpcstat   2002 Ronnie Sahlberg
  *
- * $Id: tap-dcerpcstat.c 15043 2005-07-24 19:01:28Z guy $
+ * $Id: tap-dcerpcstat.c 17252 2006-02-11 13:05:24Z lego $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -35,6 +35,7 @@
 #include <string.h>
 #include "epan/packet_info.h"
 #include <epan/tap.h>
+#include <epan/stat_cmd_args.h>
 #include <epan/dissectors/packet-dcerpc.h>
 #include "register.h"
 
@@ -103,12 +104,7 @@ dcerpcstat_packet(void *prs, packet_info *pinfo, epan_dissect_t *edt _U_, const
 	rp=&(rs->procedures[ri->call_data->opnum]);
 
 	/* calculate time delta between request and reply */
-	delta.secs=pinfo->fd->abs_secs-ri->call_data->req_time.secs;
-	delta.nsecs=pinfo->fd->abs_usecs*1000-ri->call_data->req_time.nsecs;
-	if(delta.nsecs<0){
-		delta.nsecs+=1000000000;
-		delta.secs--;
-	}
+	nstime_delta(&delta, &pinfo->fd->abs_ts, &ri->call_data->req_time);
 
 	if(rp->num==0){
 		rp->max.secs=delta.secs;
@@ -186,7 +182,7 @@ dcerpcstat_draw(void *prs)
 
 
 static void
-dcerpcstat_init(char *optarg)
+dcerpcstat_init(const char *optarg, void* userdata _U_)
 {
 	rpcstat_t *rs;
 	guint32 i, max_procs;
@@ -196,7 +192,7 @@ dcerpcstat_init(char *optarg)
 	int major, minor;
 	guint16 ver;
 	int pos=0;
-        char *filter=NULL;
+        const char *filter=NULL;
         GString *error_string;
     
 	/*
@@ -305,5 +301,5 @@ dcerpcstat_init(char *optarg)
 void
 register_tap_listener_dcerpcstat(void)
 {
-	register_tap_listener_cmd_arg("dcerpc,rtt,", dcerpcstat_init);
+	register_stat_cmd_arg("dcerpc,rtt,", dcerpcstat_init,NULL);
 }
