@@ -3,7 +3,7 @@
  * Copyright 2003, Michael Lum <mlum [AT] telostech.com>
  * In association with Telos Technology Inc.
  *
- * $Id: tap-gsm_astat.c 13993 2005-04-01 09:08:27Z guy $
+ * $Id: tap-gsm_astat.c 17252 2006-02-11 13:05:24Z lego $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -42,6 +42,7 @@
 #include "epan/packet_info.h"
 #include "epan/value_string.h"
 #include <epan/tap.h>
+#include <epan/stat_cmd_args.h>
 #include <epan/dissectors/packet-bssap.h>
 #include <epan/dissectors/packet-gsm_a.h>
 #include "register.h"
@@ -269,14 +270,11 @@ gsm_a_stat_draw(
 
 
 static void
-gsm_a_stat_init(char *optarg)
+gsm_a_stat_init(const char *optarg _U_,void* userdata _U_)
 {
     gsm_a_stat_t	*stat_p;
     GString		*err_p;
 
-
-    optarg = optarg;
-
     stat_p = g_malloc(sizeof(gsm_a_stat_t));
 
     memset(stat_p, 0, sizeof(gsm_a_stat_t));
@@ -300,5 +298,5 @@ gsm_a_stat_init(char *optarg)
 void
 register_tap_listener_gsm_astat(void)
 {
-    register_tap_listener_cmd_arg("gsm_a,", gsm_a_stat_init);
+    register_stat_cmd_arg("gsm_a,", gsm_a_stat_init,NULL);
 }
