@@ -1,7 +1,7 @@
 /* tap-iousers.c
  * iostat   2003 Ronnie Sahlberg
  *
- * $Id: tap-iousers.c 15015 2005-07-23 11:41:25Z jmayer $
+ * $Id: tap-iousers.c 17681 2006-03-20 10:52:53Z sahlberg $
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@ethereal.com>
@@ -35,8 +35,10 @@
 #include <string.h>
 #include <epan/packet_info.h>
 #include <epan/packet.h>
+#include <epan/emem.h>
 #include <epan/addr_resolv.h>
 #include <epan/tap.h>
+#include <epan/stat_cmd_args.h>
 #include "register.h"
 #include <epan/dissectors/packet-ip.h>
 #include <epan/dissectors/packet-ipx.h>
@@ -47,7 +49,6 @@
 #include <epan/dissectors/packet-tr.h>
 #include <epan/dissectors/packet-fc.h>
 #include <epan/dissectors/packet-fddi.h>
-#include <string.h>
 
 typedef struct _io_users_t {
 	const char *type;
@@ -79,20 +80,20 @@ iousers_udpip_packet(void *arg, packet_info *pinfo, epan_dissect_t *edt _U_, con
 
 	if(udph->uh_sport>udph->uh_dport){
 		direction=0;
-		snprintf(name1,256,"%s:%s",address_to_str(&udph->ip_src),get_udp_port(udph->uh_sport));
-		snprintf(name2,256,"%s:%s",address_to_str(&udph->ip_dst),get_udp_port(udph->uh_dport));
+		g_snprintf(name1,256,"%s:%s",address_to_str(&udph->ip_src),get_udp_port(udph->uh_sport));
+		g_snprintf(name2,256,"%s:%s",address_to_str(&udph->ip_dst),get_udp_port(udph->uh_dport));
 	} else if(udph->uh_sport<udph->uh_dport){
 		direction=1;
-		snprintf(name2,256,"%s:%s",address_to_str(&udph->ip_src),get_udp_port(udph->uh_sport));
-		snprintf(name1,256,"%s:%s",address_to_str(&udph->ip_dst),get_udp_port(udph->uh_dport));
+		g_snprintf(name2,256,"%s:%s",address_to_str(&udph->ip_src),get_udp_port(udph->uh_sport));
+		g_snprintf(name1,256,"%s:%s",address_to_str(&udph->ip_dst),get_udp_port(udph->uh_dport));
 	} else if(CMP_ADDRESS(&udph->ip_src, &udph->ip_dst)>0){
 		direction=0;
-		snprintf(name1,256,"%s:%s",address_to_str(&udph->ip_src),get_udp_port(udph->uh_sport));
-		snprintf(name2,256,"%s:%s",address_to_str(&udph->ip_dst),get_udp_port(udph->uh_dport));
+		g_snprintf(name1,256,"%s:%s",address_to_str(&udph->ip_src),get_udp_port(udph->uh_sport));
+		g_snprintf(name2,256,"%s:%s",address_to_str(&udph->ip_dst),get_udp_port(udph->uh_dport));
 	} else {
 		direction=1;
-		snprintf(name2,256,"%s:%s",address_to_str(&udph->ip_src),get_udp_port(udph->uh_sport));
-		snprintf(name1,256,"%s:%s",address_to_str(&udph->ip_dst),get_udp_port(udph->uh_dport));
+		g_snprintf(name2,256,"%s:%s",address_to_str(&udph->ip_src),get_udp_port(udph->uh_sport));
+		g_snprintf(name1,256,"%s:%s",address_to_str(&udph->ip_dst),get_udp_port(udph->uh_dport));
 	}
 
 	for(iui=iu->items;iui;iui=iui->next){
@@ -137,21 +138,21 @@ iousers_sctp_packet(void *arg, packet_info *pinfo, epan_dissect_t *edt _U_, cons
 	io_users_item_t *iui;
 	int direction=0;
 
-	sprintf(s_sport,"%d",sctph->sport);
-	sprintf(s_dport,"%d",sctph->dport);
+	g_snprintf(s_sport, sizeof s_sport, "%d",sctph->sport);
+	g_snprintf(s_dport, sizeof s_dport, "%d",sctph->dport);
 	
 	if(sctph->sport > sctph->dport) {
 		direction=0;
-		snprintf(name1,256,"%s:%s",address_to_str(&sctph->ip_src),s_sport);	
-		snprintf(name2,256,"%s:%s",address_to_str(&sctph->ip_dst),s_dport);	
+		g_snprintf(name1,256,"%s:%s",address_to_str(&sctph->ip_src),s_sport);	
+		g_snprintf(name2,256,"%s:%s",address_to_str(&sctph->ip_dst),s_dport);	
 	} else if(sctph->sport < sctph->dport) {
 		direction=1;
-		snprintf(name1,256,"%s:%s",address_to_str(&sctph->ip_src),s_sport);	
-		snprintf(name2,256,"%s:%s",address_to_str(&sctph->ip_dst),s_dport);	
+		g_snprintf(name1,256,"%s:%s",address_to_str(&sctph->ip_src),s_sport);	
+		g_snprintf(name2,256,"%s:%s",address_to_str(&sctph->ip_dst),s_dport);	
 	} else {
 		direction=0;
-		snprintf(name1,256,"%s:%s",address_to_str(&sctph->ip_src),s_sport);	
-		snprintf(name2,256,"%s:%s",address_to_str(&sctph->ip_dst),s_dport);	
+		g_snprintf(name1,256,"%s:%s",address_to_str(&sctph->ip_src),s_sport);	
+		g_snprintf(name2,256,"%s:%s",address_to_str(&sctph->ip_dst),s_dport);	
 	}
 
 	for(iui=iu->items;iui;iui=iui->next){
@@ -198,20 +199,20 @@ iousers_tcpip_packet(void *arg, packet_info *pinfo, epan_dissect_t *edt _U_, con
 
 	if(tcph->th_sport>tcph->th_dport){
 		direction=0;
-		snprintf(name1,256,"%s:%s",address_to_str(&tcph->ip_src),get_tcp_port(tcph->th_sport));
-		snprintf(name2,256,"%s:%s",address_to_str(&tcph->ip_dst),get_tcp_port(tcph->th_dport));
+		g_snprintf(name1,256,"%s:%s",address_to_str(&tcph->ip_src),get_tcp_port(tcph->th_sport));
+		g_snprintf(name2,256,"%s:%s",address_to_str(&tcph->ip_dst),get_tcp_port(tcph->th_dport));
 	} else if(tcph->th_sport<tcph->th_dport){
 		direction=1;
-		snprintf(name2,256,"%s:%s",address_to_str(&tcph->ip_src),get_tcp_port(tcph->th_sport));
-		snprintf(name1,256,"%s:%s",address_to_str(&tcph->ip_dst),get_tcp_port(tcph->th_dport));
+		g_snprintf(name2,256,"%s:%s",address_to_str(&tcph->ip_src),get_tcp_port(tcph->th_sport));
+		g_snprintf(name1,256,"%s:%s",address_to_str(&tcph->ip_dst),get_tcp_port(tcph->th_dport));
 	} else if(CMP_ADDRESS(&tcph->ip_src, &tcph->ip_dst)>0){
 		direction=0;
-		snprintf(name1,256,"%s:%s",address_to_str(&tcph->ip_src),get_tcp_port(tcph->th_sport));
-		snprintf(name2,256,"%s:%s",address_to_str(&tcph->ip_dst),get_tcp_port(tcph->th_dport));
+		g_snprintf(name1,256,"%s:%s",address_to_str(&tcph->ip_src),get_tcp_port(tcph->th_sport));
+		g_snprintf(name2,256,"%s:%s",address_to_str(&tcph->ip_dst),get_tcp_port(tcph->th_dport));
 	} else {
 		direction=1;
-		snprintf(name2,256,"%s:%s",address_to_str(&tcph->ip_src),get_tcp_port(tcph->th_sport));
-		snprintf(name1,256,"%s:%s",address_to_str(&tcph->ip_dst),get_tcp_port(tcph->th_dport));
+		g_snprintf(name2,256,"%s:%s",address_to_str(&tcph->ip_src),get_tcp_port(tcph->th_sport));
+		g_snprintf(name1,256,"%s:%s",address_to_str(&tcph->ip_dst),get_tcp_port(tcph->th_dport));
 	}
 
 	for(iui=iu->items;iui;iui=iui->next){
@@ -579,9 +580,9 @@ iousers_draw(void *arg)
 }
 
 void
-iousers_init(char *optarg)
+iousers_init(const char *optarg, void* userdata _U_)
 {
-	char *filter=NULL;
+	const char *filter=NULL;
 	const char *tap_type, *tap_type_name;
 	tap_packet_cb packet_func;
 	io_users_t *iu=NULL;
@@ -710,5 +711,5 @@ iousers_init(char *optarg)
 void
 register_tap_listener_iousers(void)
 {
-	register_tap_listener_cmd_arg("conv,", iousers_init);
+	register_stat_cmd_arg("conv,", iousers_init, NULL);
 }
