@@ -1,6 +1,6 @@
 /* libpcap.c
  *
- * $Id: libpcap.c 14994 2005-07-22 15:55:06Z etxrab $
+ * $Id: libpcap.c 17450 2006-03-02 21:28:05Z etxrab $
  *
  * Wiretap Library
  * Copyright (c) 1998 by Gilbert Ramirez <gram@alumni.rice.edu>
@@ -81,6 +81,21 @@ struct mtp2_hdr {
 	guint16 link_number;
 };
 
+#ifndef ETH_P_LAPD
+#define ETH_P_LAPD 0x0030
+#endif
+
+/*
+ * The fake link-layer header of LAPD packets 
+ */
+struct lapd_sll_hdr {
+    guint16 sll_pkttype;    /* packet type */
+    guint16 sll_hatype;
+    guint16 sll_halen;
+    guint8 sll_addr[8];
+    guint16 sll_protocol;   /* protocol, should be ETH_P_LAPD */
+};
+
 /* See source to the "libpcap" library for information on the "libpcap"
    file format. */
 
@@ -119,6 +134,10 @@ static gboolean libpcap_get_mtp2_pseudoheader(const struct mtp2_hdr *mtp2_hdr,
     union wtap_pseudo_header *pseudo_header);
 static gboolean libpcap_read_mtp2_pseudoheader(FILE_T fh,
     union wtap_pseudo_header *pseudo_header, int *err, gchar **err_info);
+static gboolean libpcap_get_lapd_pseudoheader(const struct lapd_sll_hdr *lapd_phdr,
+    union wtap_pseudo_header *pseudo_header, int *err, gchar **err_info);
+static gboolean libpcap_read_lapd_pseudoheader(FILE_T fh,
+    union wtap_pseudo_header *pseudo_header, int *err, gchar **err_info);
 static gboolean libpcap_read_rec_data(FILE_T fh, guchar *pd, int length,
     int *err);
 static void libpcap_close(wtap *wth);
@@ -308,8 +327,10 @@ static const struct {
 	{ 128,		WTAP_ENCAP_TZSP },	/* Tazmen Sniffer Protocol */
 	{ 129,		WTAP_ENCAP_ARCNET_LINUX },
         { 130,          WTAP_ENCAP_JUNIPER_MLPPP }, /* Juniper MLPPP on ML-, LS-, AS- PICs */
+        { 131,          WTAP_ENCAP_JUNIPER_MLFR }, /* Juniper MLFR (FRF.15) on ML-, LS-, AS- PICs */
+    {133, WTAP_ENCAP_JUNIPER_GGSN},
 	/*
-	 * Values 131-134, 136 not listed here are reserved for use
+	 * Values 132-134, 136 not listed here are reserved for use
 	 * in Juniper hardware.
 	 */
 	{ 135,		WTAP_ENCAP_JUNIPER_ATM2 }, /* various encapsulations captured on the ATM2 PIC */
@@ -377,6 +398,18 @@ static const struct {
 	{ 172,		WTAP_GCOM_TIE1 },
 	{ 173,		WTAP_GCOM_SERIAL },
 
+	{ 177,		WTAP_ENCAP_LINUX_LAPD },
+
+        /* Ethernet frames prepended with meta-information */
+        { 178,          WTAP_ENCAP_JUNIPER_ETHER },
+        /* PPP frames prepended with meta-information */
+        { 179,          WTAP_ENCAP_JUNIPER_PPP },
+        /* Frame-Relay frames prepended with meta-information */
+        { 180,          WTAP_ENCAP_JUNIPER_FRELAY },
+        /* C-HDLC frames prepended with meta-information */
+        { 181,          WTAP_ENCAP_JUNIPER_CHDLC },
+
+
 	/*
 	 * To repeat:
 	 *
@@ -600,6 +633,7 @@ int libpcap_open(wtap *wth, int *err, gchar **err_info)
 	gboolean aix;
 	int file_encap;
 
+	
 	/* Read in the number that should be at the start of a "libpcap" file */
 	errno = WTAP_ERR_CANT_READ;
 	bytes_read = file_read(&magic, 1, sizeof magic, wth->fh);
@@ -618,6 +652,7 @@ int libpcap_open(wtap *wth, int *err, gchar **err_info)
 		   a program using either standard or ss990417 libpcap. */
 		byte_swapped = FALSE;
 		modified = FALSE;
+		wth->tsprecision = WTAP_FILE_TSPREC_USEC;
 		break;
 
 	case PCAP_MODIFIED_MAGIC:
@@ -625,6 +660,7 @@ int libpcap_open(wtap *wth, int *err, gchar **err_info)
 		   a program using either ss990915 or ss991029 libpcap. */
 		byte_swapped = FALSE;
 		modified = TRUE;
+		wth->tsprecision = WTAP_FILE_TSPREC_USEC;
 		break;
 
 	case PCAP_SWAPPED_MAGIC:
@@ -633,6 +669,7 @@ int libpcap_open(wtap *wth, int *err, gchar **err_info)
 		   ss990417 libpcap. */
 		byte_swapped = TRUE;
 		modified = FALSE;
+		wth->tsprecision = WTAP_FILE_TSPREC_USEC;
 		break;
 
 	case PCAP_SWAPPED_MODIFIED_MAGIC:
@@ -641,6 +678,24 @@ int libpcap_open(wtap *wth, int *err, gchar **err_info)
 		   or ss991029 libpcap. */
 		byte_swapped = TRUE;
 		modified = TRUE;
+		wth->tsprecision = WTAP_FILE_TSPREC_USEC;
+		break;
+
+	case PCAP_NSEC_MAGIC:
+		/* Host that wrote it has our byte order, and was running
+		   a program using either standard or ss990417 libpcap. */
+		byte_swapped = FALSE;
+		modified = FALSE;
+		wth->tsprecision = WTAP_FILE_TSPREC_NSEC;
+		break;
+
+	case PCAP_SWAPPED_NSEC_MAGIC:
+		/* Host that wrote it out has a byte order opposite to
+		   ours, and was running a program using either ss990915 
+		   or ss991029 libpcap. */
+		byte_swapped = TRUE;
+		modified = FALSE;
+		wth->tsprecision = WTAP_FILE_TSPREC_NSEC;
 		break;
 
 	default:
@@ -898,7 +953,11 @@ int libpcap_open(wtap *wth, int *err, gchar **err_info)
 		 *
 		 * Try the standard format first.
 		 */
-		wth->file_type = WTAP_FILE_PCAP;
+		if(wth->tsprecision == WTAP_FILE_TSPREC_NSEC) {
+			wth->file_type = WTAP_FILE_PCAP_NSEC;
+		} else {
+			wth->file_type = WTAP_FILE_PCAP;
+		}
 		switch (libpcap_try(wth, err)) {
 
 		case BAD_READ:
@@ -1258,6 +1317,29 @@ static gboolean libpcap_read(wtap *wth, int *err, gchar **err_info,
 		packet_size -= sizeof (struct mtp2_hdr);
 		wth->data_offset += sizeof (struct mtp2_hdr);
 		break;
+
+	case WTAP_ENCAP_LINUX_LAPD:
+		if (packet_size < sizeof (struct lapd_sll_hdr)) {
+			/*
+			 * Uh-oh, the packet isn't big enough to even
+			 * have a pseudo-header.
+			 */
+			*err = WTAP_ERR_BAD_RECORD;
+			*err_info = g_strdup_printf("libpcap: LAPD file has a %u-byte packet, too small to have even a LAPD pseudo-header\n",
+			    packet_size);
+			return FALSE;
+		}
+		if (!libpcap_read_lapd_pseudoheader(wth->fh, &wth->pseudo_header,
+		    err, err_info))
+			return FALSE;	/* Read error */
+
+		/*
+		 * Don't count the pseudo-header as part of the packet.
+		 */
+		orig_size -= sizeof (struct lapd_sll_hdr);
+		packet_size -= sizeof (struct lapd_sll_hdr);
+		wth->data_offset += sizeof (struct lapd_sll_hdr);
+		break;
 	}
 
 	buffer_assure_space(wth->frame_buffer, packet_size);
@@ -1266,8 +1348,12 @@ static gboolean libpcap_read(wtap *wth, int *err, gchar **err_info,
 		return FALSE;	/* Read error */
 	wth->data_offset += packet_size;
 
-	wth->phdr.ts.tv_sec = hdr.hdr.ts_sec;
-	wth->phdr.ts.tv_usec = hdr.hdr.ts_usec;
+	wth->phdr.ts.secs = hdr.hdr.ts_sec;
+	if(wth->tsprecision == WTAP_FILE_TSPREC_NSEC) {
+		wth->phdr.ts.nsecs = hdr.hdr.ts_usec;
+	} else {
+		wth->phdr.ts.nsecs = hdr.hdr.ts_usec * 1000;
+	}
 	wth->phdr.caplen = packet_size;
 	wth->phdr.len = orig_size;
 
@@ -1364,6 +1450,14 @@ libpcap_seek_read(wtap *wth, long seek_off,
 			return FALSE;
 		}
 		break;
+
+	case WTAP_ENCAP_LINUX_LAPD:
+		if (!libpcap_read_lapd_pseudoheader(wth->random_fh, pseudo_header,
+		    err, err_info)) {
+			/* Read error */
+			return FALSE;
+		}
+		break;
 	}
 
 	/*
@@ -1410,6 +1504,7 @@ static int libpcap_read_header(wtap *wth, int *err, gchar **err_info,
 
 	case WTAP_FILE_PCAP:
 	case WTAP_FILE_PCAP_AIX:
+	case WTAP_FILE_PCAP_NSEC:
 		bytes_to_read = sizeof (struct pcaprec_hdr);
 		break;
 
@@ -1716,6 +1811,43 @@ libpcap_read_mtp2_pseudoheader(FILE_T fh, union wtap_pseudo_header *pseudo_heade
 	     
 }
 
+static gboolean
+libpcap_get_lapd_pseudoheader(const struct lapd_sll_hdr *lapd_phdr,
+    union wtap_pseudo_header *pseudo_header, int *err, gchar **err_info)
+{
+	if (pntohs(&lapd_phdr->sll_protocol) != ETH_P_LAPD) {
+		*err = WTAP_ERR_BAD_RECORD;
+		if (err_info != NULL)
+			*err_info = g_strdup("libpcap: LAPD capture has a packet with an invalid sll_protocol field\n");
+		return FALSE;
+	}
+
+	pseudo_header->lapd.pkttype = pntohs(&lapd_phdr->sll_pkttype);
+	pseudo_header->lapd.we_network = !!lapd_phdr->sll_addr[0];
+
+	return TRUE;
+}
+
+static gboolean
+libpcap_read_lapd_pseudoheader(FILE_T fh, union wtap_pseudo_header *pseudo_header,
+    int *err, gchar **err_info)
+{
+	struct lapd_sll_hdr lapd_phdr;
+	int	bytes_read;
+
+	errno = WTAP_ERR_CANT_READ;
+	bytes_read = file_read(&lapd_phdr, 1, sizeof (struct lapd_sll_hdr), fh);
+	if (bytes_read != sizeof (struct lapd_sll_hdr)) {
+		*err = file_error(fh);
+		if (*err == 0)
+			*err = WTAP_ERR_SHORT_READ;
+		return FALSE;
+	}
+
+	return libpcap_get_lapd_pseudoheader(&lapd_phdr, pseudo_header, err,
+	    err_info);
+}
+
 static gboolean
 libpcap_read_rec_data(FILE_T fh, guchar *pd, int length, int *err)
 {
@@ -1818,8 +1950,8 @@ wtap_process_pcap_packet(gint linktype, const struct pcap_pkthdr *phdr,
 	   be a "struct bpf_timeval", with member sizes wired to 32
 	   bits - and we may go that way ourselves in the future, so
 	   copy the members individually. */
-	whdr->ts.tv_sec = phdr->ts.tv_sec;
-	whdr->ts.tv_usec = phdr->ts.tv_usec;
+	whdr->ts.secs = phdr->ts.tv_sec;
+	whdr->ts.nsecs = phdr->ts.tv_usec * 1000;
 	whdr->caplen = phdr->caplen;
 	whdr->len = phdr->len;
 	whdr->pkt_encap = linktype;
@@ -1901,6 +2033,28 @@ wtap_process_pcap_packet(gint linktype, const struct pcap_pkthdr *phdr,
 		whdr->caplen -= sizeof (struct mtp2_hdr);
 		pd += sizeof (struct mtp2_hdr);
 	}
+	else if (linktype == WTAP_ENCAP_LINUX_LAPD) {
+		if (whdr->caplen < sizeof (struct lapd_sll_hdr)) {
+			/*
+			 * Uh-oh, the packet isn't big enough to even
+			 * have a pseudo-header.
+			 */
+			g_message("libpcap: LAPD capture has a %u-byte packet, too small to have even an LAPD pseudo-header\n",
+			    whdr->caplen);
+			*err = WTAP_ERR_BAD_RECORD;
+			return NULL;
+		}
+		if (!libpcap_get_lapd_pseudoheader((const struct lapd_sll_hdr *)pd,
+			pseudo_header, err, NULL))
+			return NULL;
+
+		/*
+		 * Don't count the pseudo-header as part of the packet.
+		 */
+		whdr->len -= sizeof (struct lapd_sll_hdr);
+		whdr->caplen -= sizeof (struct lapd_sll_hdr);
+		pd += sizeof (struct lapd_sll_hdr);
+	}
 	return pd;
 }
 #endif
@@ -1938,11 +2092,18 @@ gboolean libpcap_dump_open(wtap_dumper *wdh, gboolean cant_seek _U_, int *err)
 	case WTAP_FILE_PCAP_SS990417:	/* modified, but with the old magic, sigh */
 	case WTAP_FILE_PCAP_NOKIA:	/* Nokia libpcap of some sort */
 		magic = PCAP_MAGIC;
+		wdh->tsprecision = WTAP_FILE_TSPREC_USEC;
 		break;
 
 	case WTAP_FILE_PCAP_SS990915:	/* new magic, extra crap */
 	case WTAP_FILE_PCAP_SS991029:
 		magic = PCAP_MODIFIED_MAGIC;
+		wdh->tsprecision = WTAP_FILE_TSPREC_USEC;
+		break;
+
+	case WTAP_FILE_PCAP_NSEC:		/* same as WTAP_FILE_PCAP, but nsec precision */
+		magic = PCAP_NSEC_MAGIC;
+		wdh->tsprecision = WTAP_FILE_TSPREC_NSEC;
 		break;
 
 	default:
@@ -1952,10 +2113,10 @@ gboolean libpcap_dump_open(wtap_dumper *wdh, gboolean cant_seek _U_, int *err)
 		return FALSE;
 	}
 
-	nwritten = fwrite(&magic, 1, sizeof magic, wdh->fh);
+	nwritten = wtap_dump_file_write(wdh, &magic, sizeof magic);
 	if (nwritten != sizeof magic) {
-		if (nwritten == 0 && ferror(wdh->fh))
-			*err = errno;
+		if (nwritten == 0 && wtap_dump_file_ferror(wdh))
+			*err = wtap_dump_file_ferror(wdh);
 		else
 			*err = WTAP_ERR_SHORT_WRITE;
 		return FALSE;
@@ -1981,10 +2142,10 @@ gboolean libpcap_dump_open(wtap_dumper *wdh, gboolean cant_seek _U_, int *err)
 	file_hdr.snaplen = (wdh->snaplen != 0) ? wdh->snaplen :
 						 WTAP_MAX_PACKET_SIZE;
 	file_hdr.network = wtap_wtap_encap_to_pcap_encap(wdh->encap);
-	nwritten = fwrite(&file_hdr, 1, sizeof file_hdr, wdh->fh);
+	nwritten = wtap_dump_file_write(wdh, &file_hdr, sizeof file_hdr);
 	if (nwritten != sizeof file_hdr) {
-		if (nwritten == 0 && ferror(wdh->fh))
-			*err = errno;
+		if (nwritten == 0 && wtap_dump_file_ferror(wdh))
+			*err = wtap_dump_file_ferror(wdh);
 		else
 			*err = WTAP_ERR_SHORT_WRITE;
 		return FALSE;
@@ -2006,6 +2167,7 @@ static gboolean libpcap_dump(wtap_dumper *wdh,
 	size_t nwritten;
 	struct sunatm_hdr atm_hdr;
 	struct irda_sll_hdr irda_hdr;
+	struct lapd_sll_hdr lapd_hdr;
 	struct mtp2_hdr mtp2_hdr;
 	int hdrsize;
 
@@ -2013,16 +2175,23 @@ static gboolean libpcap_dump(wtap_dumper *wdh,
 		hdrsize = sizeof (struct sunatm_hdr);
 	else if (wdh->encap == WTAP_ENCAP_IRDA)
 		hdrsize = sizeof (struct irda_sll_hdr);
+	else if (wdh->encap == WTAP_ENCAP_LINUX_LAPD)
+		hdrsize = sizeof (struct lapd_sll_hdr);
 	else
 		hdrsize = 0;
 
-	rec_hdr.hdr.ts_sec = phdr->ts.tv_sec;
-	rec_hdr.hdr.ts_usec = phdr->ts.tv_usec;
+	rec_hdr.hdr.ts_sec = phdr->ts.secs;
+	if(wdh->tsprecision == WTAP_FILE_TSPREC_NSEC) {
+		rec_hdr.hdr.ts_usec = phdr->ts.nsecs;
+	} else {
+		rec_hdr.hdr.ts_usec = phdr->ts.nsecs / 1000;
+	}
 	rec_hdr.hdr.incl_len = phdr->caplen + hdrsize;
 	rec_hdr.hdr.orig_len = phdr->len + hdrsize;
 	switch (wdh->file_type) {
 
 	case WTAP_FILE_PCAP:
+	case WTAP_FILE_PCAP_NSEC:
 		hdr_size = sizeof (struct pcaprec_hdr);
 		break;
 
@@ -2079,10 +2248,10 @@ static gboolean libpcap_dump(wtap_dumper *wdh,
 		return FALSE;
 	}
 
-	nwritten = fwrite(&rec_hdr, 1, hdr_size, wdh->fh);
+	nwritten = wtap_dump_file_write(wdh, &rec_hdr, hdr_size);
 	if (nwritten != hdr_size) {
-		if (nwritten == 0 && ferror(wdh->fh))
-			*err = errno;
+		if (nwritten == 0 && wtap_dump_file_ferror(wdh))
+			*err = wtap_dump_file_ferror(wdh);
 		else
 			*err = WTAP_ERR_SHORT_WRITE;
 		return FALSE;
@@ -2124,10 +2293,10 @@ static gboolean libpcap_dump(wtap_dumper *wdh,
 		}
 		atm_hdr.vpi = (guint8) pseudo_header->atm.vpi;
 		atm_hdr.vci = phtons(&pseudo_header->atm.vci);
-		nwritten = fwrite(&atm_hdr, 1, sizeof atm_hdr, wdh->fh);
+		nwritten = wtap_dump_file_write(wdh, &atm_hdr, sizeof atm_hdr);
 		if (nwritten != sizeof atm_hdr) {
-			if (nwritten == 0 && ferror(wdh->fh))
-				*err = errno;
+			if (nwritten == 0 && wtap_dump_file_ferror(wdh))
+				*err = wtap_dump_file_ferror(wdh);
 			else
 				*err = WTAP_ERR_SHORT_WRITE;
 			return FALSE;
@@ -2141,10 +2310,10 @@ static gboolean libpcap_dump(wtap_dumper *wdh,
 		memset(&irda_hdr, 0, sizeof(irda_hdr));
 		irda_hdr.sll_pkttype  = phtons(&pseudo_header->irda.pkttype);
 		irda_hdr.sll_protocol = g_htons(0x0017);
-		nwritten = fwrite(&irda_hdr, 1, sizeof(irda_hdr), wdh->fh);
+		nwritten = wtap_dump_file_write(wdh, &irda_hdr, sizeof(irda_hdr));
 		if (nwritten != sizeof(irda_hdr)) {
-			if (nwritten == 0 && ferror(wdh->fh))
-				*err = errno;
+			if (nwritten == 0 && wtap_dump_file_ferror(wdh))
+				*err = wtap_dump_file_ferror(wdh);
 			else
 				*err = WTAP_ERR_SHORT_WRITE;
 			return FALSE;
@@ -2159,21 +2328,39 @@ static gboolean libpcap_dump(wtap_dumper *wdh,
 		mtp2_hdr.sent         = pseudo_header->mtp2.sent;
 		mtp2_hdr.annex_a_used = pseudo_header->mtp2.annex_a_used;
 		mtp2_hdr.link_number  = phtons(&pseudo_header->mtp2.link_number);
-		nwritten = fwrite(&mtp2_hdr, 1, sizeof(mtp2_hdr), wdh->fh);
+		nwritten = wtap_dump_file_write(wdh, &mtp2_hdr, sizeof(mtp2_hdr));
 		if (nwritten != sizeof(mtp2_hdr)) {
-			if (nwritten == 0 && ferror(wdh->fh))
-				*err = errno;
+			if (nwritten == 0 && wtap_dump_file_ferror(wdh))
+				*err = wtap_dump_file_ferror(wdh);
 			else
 				*err = WTAP_ERR_SHORT_WRITE;
 			return FALSE;
 		}
 		wdh->bytes_dumped += sizeof(mtp2_hdr);
 	}
+	else if (wdh->encap == WTAP_ENCAP_LINUX_LAPD) {
+		/*
+		 * Write the LAPD header.
+		 */
+		memset(&lapd_hdr, 0, sizeof(lapd_hdr));
+		lapd_hdr.sll_pkttype  = phtons(&pseudo_header->lapd.pkttype);
+		lapd_hdr.sll_protocol = g_htons(ETH_P_LAPD);
+		lapd_hdr.sll_addr[0] = pseudo_header->lapd.we_network?0x01:0x00;
+		nwritten = fwrite(&lapd_hdr, 1, sizeof(lapd_hdr), wdh->fh);
+		if (nwritten != sizeof(lapd_hdr)) {
+			if (nwritten == 0 && ferror(wdh->fh))
+				*err = errno;
+			else
+				*err = WTAP_ERR_SHORT_WRITE;
+			return FALSE;
+		}
+		wdh->bytes_dumped += sizeof(lapd_hdr);
+	}
 
-	nwritten = fwrite(pd, 1, phdr->caplen, wdh->fh);
+	nwritten = wtap_dump_file_write(wdh, pd, phdr->caplen);
 	if (nwritten != phdr->caplen) {
-		if (nwritten == 0 && ferror(wdh->fh))
-			*err = errno;
+		if (nwritten == 0 && wtap_dump_file_ferror(wdh))
+			*err = wtap_dump_file_ferror(wdh);
 		else
 			*err = WTAP_ERR_SHORT_WRITE;
 		return FALSE;
